/*
*
* email-inbox.js
*
*	Note on validation and options settings:  
*		All the dynamic settings appearing in the inbox area -- issue, pro-con, template --	are controlled in this script.
*		They are transmitted by ajax on the submission of an action and are not saved otherwise.
*	  	Note that these values persist until the email inbox page is abandoned.
*
*	See also notes in email-process.js.
*
*	INTERFACE NOTE:
*   The inbox list loaded by loadInbox includes the following interface variables coming from the server
*		-- the class "trained-subject" which identifies sweepable lines
*		-- the list of UIDs for each subject line
*		-- the message count for each line (which could be reconstructed, but is convenient)
*	The other list elements are just display.  Single view is the same as multi view on the client side.
*
*
*/

// self-executing anonymous namespace
( function( wpIssuesCRM, $, undefined ) {

	var pendingInboxLoad = false, // prevent line loads while box load pending
		// box view variables	
		inboxGrouped = {
			inbox: true,
			sent: false,
			done: false,
			outbox: false,
			saved: false
		},
		nextPage = {
			inbox: 0,
			sent: 0,
			done: 0,
			outbox: 0,
			saved: 0
		},
		// search control
		latestSearch = '', 	// to discard old filter requests
		acTimer = false, 		// timer for filter search delay 
		selectedPage = 'inbox'; 
	// define this inbox view variable as global for access in restoring undo
	wpIssuesCRM.inboxAscending = {
			inbox: false,
			sent: false,
			done: false,
			outbox: false,
			saved: false,
		};


	// fired on load inbox
	wpIssuesCRM.initializeInboxVars = function () {
		// do nothing if the inbox is not open
		if ( ! $( "#wic-email-inbox" )[0] ) { return }
		// initialize a load parm object for tracking inbox content
		wpIssuesCRM.lastLoadParms = {}, // reported by last load operation
		// reset last search
		latestSearch = '';  
		// initialize a counter for pending email process requests ( can't use other xhr queue )
		wpIssuesCRM.pendingMailRequests = 0;
		// initialize a sweep flag used to prevent double sweep submits
		wpIssuesCRM.pendingSweep = false;
		// set up current message variables -- initialized to blank for use in sweeps (controls detail view scrolling)
		wpIssuesCRM.currentMessageVars = { // used for interface from inbox to detail (processing gets detail from form)
			activeSubject		 : '',
			countMessages		 : 0,
			uidArray			 : [],
			activeMessage		 : 0,
		}
		// will be set to the clicked (or scrolled to) inbox line element	
		wpIssuesCRM.activeLine = false; 
		// bind all listeners for inbox pages (to already loaded page containers or other already loaded elements )
		loadInboxListeners() 
		// load listeners for the settings page and get options (need header for email preview
		wpIssuesCRM.initializeEmailProcess ();
		// get page load started
		$( "#inbox_page" ).trigger( "change" );
	}

	/*
	*
	* define functions for switching and presenting different inbox and detail views
	* -- doSelectedPage for switching among major pages;
	*
	* within inbox page, states are 
	*	the inbox list
	*   the view of a group of messages within a subject line
	*		the templated view mode vs template edit mode in right hand side
	*		the scroll of messages within subject line (which changes LHS and the 
	* so, transition functions are:		
	* -- loadInbox loads the inbox list
	* -- jumpToSubjectLine switches subject lines within list 
	*		always then does scrollMessage to get first message from subject line
	*		triggers showMessageDetails to switch view after message is ready (matters if coming from list).
	* -- showMessageDetails -- for switching from inbox list view to particular subject line view 
	* -- closeMessageDetails --switching back to inbox hide/show
	* -- scrollMessage -- for switching/loading messages within subject line view
	*/

	// switch among selected subpages
	function doSelectedPage( event ) {
		selectedPage = $( event.target ).val()
		// note: this full hide logic is slightly more efficient than a stacked layer visibility swapping model
		$( ".wic-selectmenu-wrapper.wic-input.inbox-page" ).find( ".wic-selectmenu-list-entry-value" ).each( function () {
			var page = $( this ).text(); 
			$( "#wic-email-" + page + "-content" ).css( "display",  page == selectedPage ? "block" : "none" )
		});
		if ( ['inbox','sent','done','outbox','saved'].indexOf( selectedPage ) > -1 ) {
			// set up buttons/legend	
			$( ".inbox-mode-button, .page-link, #wic-inbox-sweep-button, #wic-inbox-title-legend, #wic-email-inbox #subject ").show();
			$( "#wic-inbox-title-alt-legend").hide(); 	 
			// initialize link-base for edit of issues if on first load
			if ( ! wpIssuesCRM.editMappedIssueLinkBase ) {
				wpIssuesCRM.editMappedIssueLinkBase = $( "#edit_mapped_issue_link" ).attr( "href" );
			}
			// do a refresh (since 3.2 -- refresh from local)
			wpIssuesCRM.loadSelectedPage();
			if ( 'outbox' == selectedPage ) {
				wpIssuesCRM.getMailerStatus()
			} 
		} else {
			// do show/hide of relevant buttons
			$( ".inbox-mode-button, .page-link, #wic-inbox-sweep-button, #wic-inbox-title-legend, #wic-email-inbox #subject ").hide();
			// set up the header slot for the requested page 
			$( "#wic-inbox-title-alt-legend" ).css("display", "inline-block");
			if ( 'manage-subjects' == selectedPage ) {
				wpIssuesCRM.loadSubjectList()
			} else if ( 'manage-blocks' == selectedPage ) {
				wpIssuesCRM.loadBlockList()
			} else if ( 'inbox-synch' == selectedPage ) {
				wpIssuesCRM.synchInbox( false ) // not doing resynch
			} else if ( 'settings' == selectedPage ) {
				wpIssuesCRM.initializeEmailProcess()
			}
		}
	}

	// load the inbox list of subjects ( show on the inbox subpage )
	wpIssuesCRM.loadSelectedPage = function() { 
		// on submit, if requests pending, notify user
		if ( wpIssuesCRM.pendingMailRequests && 'inbox' == selectedPage ) {
			wpIssuesCRM.alert ( 
				'<p>Inbox initiated reload before all action requests had completed processing -- may not be up to date.  In case of doubt, refresh in a few seconds.</p>'
			)
		}
		// set up loading UI
		$( ".inbox-mode-button, .page-link" ).prop( "disabled", true ); // prevent double submits
		var $loader = $( '#' + selectedPage + '-ajax-loader' );
		$loader.show();
		pendingInboxLoad = true;
		var inboxParms = {
			mode:  		inboxGrouped[selectedPage] ? 'grouped' : 'single',
			sort:  		wpIssuesCRM.inboxAscending[selectedPage] ? 'ASC' : 'DESC',
			page:		nextPage[selectedPage],
			filter:		$( "#wic-email-inbox #subject" ).val()
		}
		wpIssuesCRM.ajaxPost( 'email_inbox', 'load_' + selectedPage,  0, inboxParms,  function( response ) {
			// if this is not the last search submitted, discard it and leave buttons disabled
			if ( response.last_load_parms['filter'] != latestSearch ) {
				return;
			// reload page if page_ok not set -- indicates pages shifted due to consolidation	
			} else if ( ! response.last_load_parms['page_ok'] ) { 
				nextPage.inbox = 0;
				wpIssuesCRM.loadSelectedPage();
				return;
			}
			// release loading UI
			$( ".inbox-mode-button, .page-link" ).prop( "disabled", false );
			$loader.hide();
			pendingInboxLoad = false;
			$( "#subject" ).removeClass ( "flash" );
			// set button value for group/ungroup here since inboxGrouped persists across inbox reentries from top form buttons
			$( "#wic-inbox-ungroup-button" ).html( inboxGrouped[selectedPage] ? '1' : 'n' );
			// record last tracking loaded folder for processing purposes -- not used by passive boxes
			if ( 'inbox' == selectedPage ) {
				 wpIssuesCRM.lastLoadParms = response.last_load_parms;
			}
			// load the header and remove previously loaded buttons
			$( "#wic-inbox-title-legend" ).html( response.inbox_header )
			// disable/enable buttons
			$prev = $( ".page-link.prev" );
			$next = $( ".page-link.next" );
			toggleDisabled ( $prev, response.nav_buttons['disable_prev'] );
			toggleDisabled ( $next, response.nav_buttons['disable_next'] );
			// $( "#wic-inbox-title-legend" ).next().after( $.parseHTML ( response.nav_buttons) );
			// load the inbox
			$( "#wic-load-" + selectedPage + "-inner-wrapper" ).html( response.inbox );
			// set styling of sweep button
			var sweepButton = $( "#wic-inbox-sweep-button")
 			if ( ! $( ".inbox-subject-line.trained-subject" ).length || 'inbox' != selectedPage  ) {
 				sweepButton.css("color","#ddd");
 			} else {
 				sweepButton.css("color","#0a0");
 			}
			// alert if polling has been halted due to connection failures
			if ( response.connection ) {
				wpIssuesCRM.alert ( response.connection );
			} else if ( response.stuck ) {
				// show alert for stuck reservation queue
				wpIssuesCRM.alert ( response.stuck );
				$( "#clear-reservation-queue-button" ).click ( function () {
					$( "#clear-reservation-queue-button" ).prop( "disabled", true );
					wpIssuesCRM.ajaxPost( 'email_uid_reservation', 'clear_old_uid_reservations',  '', '',  function( response ) {
						$( "#clear-reservation-queue-button" ).text( "Cleared" );
					});
				});
			} 
 		});		
	}


	// shift from inbox view to individual message detail view or from line to line in view
	wpIssuesCRM.jumpToSubjectLine = function ( subjectLine, delay ) {
		if ( undefined !== subjectLine ) {
			wpIssuesCRM.activeLine = $( subjectLine );
			wpIssuesCRM.currentMessageVars = {
				activeSubject	 	: wpIssuesCRM.activeLine.find ( ".subject-line-item.subject .actual-email-subject" ).text(),
				fromEmail			: wpIssuesCRM.activeLine.find ( ".subject-line-item.from-email" ).text(), // note used only by message block in single line view
				fromDomain			: wpIssuesCRM.activeLine.find ( ".subject-line-item.from-domain" ).text(), // note used only by message block in single line view
				countMessages	 	: Number( wpIssuesCRM.activeLine.find(".subject-line-item.count .inner-count").text() ),
				uidArray		 	: wpIssuesCRM.activeLine.find(".subject-line-item.UIDs").text().split(','),
				activeMessage	 	: 0, // first in array
			}
			// not carrying values from prior line -- too error prone -- force review
			// load message information 
			// true means switching Subject Line (passed through to php to reset issue/pro_con/template)
			wpIssuesCRM.scrollMessage( true );
		} else {
			/*
			* introduce headstart delay in case where going straight back to inbox after starting asynch transaction
			* in localhost testing a few milliseconds is enough for 30 delete/block transaction to get right sequence
			* allowing 250 for safety in other environments; only consequence of misorder is that user may get not found message when click on subject line
			*/
			if ( delay ) { 
				setTimeout ( wpIssuesCRM.closeMessageDetails, delay );
			} else { 
				wpIssuesCRM.closeMessageDetails()
			}
		}
	}
	/*
	* doMessageViewer -- for deleted or sent messages
	*/
	function doMessageViewer( inboxSubjectLine ) {
		var $line = $( inboxSubjectLine );
		if ( Number( $line.find( ".inner-count").text() ) > 1 ) {
			wpIssuesCRM.alert ( 'Switch to single line view (1 button) -- except in inbox, can only view one reply or message at a time.' );
		} else {
			if ( 'saved' == selectedPage ) { 
				wpIssuesCRM.viewSaved ( inboxSubjectLine ) 
			} else {
				wpIssuesCRM.viewMessage ( selectedPage, inboxSubjectLine );
			}
		}
	}

	/*******************************************************************
	*
	* supporting functions
	*
	********************************************************************/
	function toggleDisabled ( $elem, state ) {
		elemDisabled = $elem.hasClass( "ui-state-disabled" );
		if ( elemDisabled != state ) {
			$elem.toggleClass( "ui-state-disabled" );
		}
	}

	/*
	*
	*
	* set up listeners on initialized form elements
	*
	*
	*/
	function loadInboxListeners() {
		$( "#wic-email-inbox" )
		// set up tab-like page swapper for inbox buttons
		.on ( "change", "#inbox_page", doSelectedPage )

		// click to open particular subject line (from inbox)
		.on ( "click", ".inbox-subject-line", function () {
			if ( ! $( this ).hasClass( "item-sending" ) && ! pendingInboxLoad && 'inbox' == selectedPage ) {
				wpIssuesCRM.jumpToSubjectLine( this, 0 )
			} else if ( 'inbox' != selectedPage ) {
				doMessageViewer( this );
			}
		})
	
		// listen to inbox mode buttons  
		.on( "click", ".inbox-mode-button", function() {
			var buttonValue = $( this ).val(); // use this, not event.target to avoid chrome treating button html as target
			switch ( buttonValue ) {
				case 'sort':
					wpIssuesCRM.inboxAscending[selectedPage] = !wpIssuesCRM.inboxAscending[selectedPage];
					break;
				case 'group':
					inboxGrouped[selectedPage] = !inboxGrouped[selectedPage];
					break;
			}	
			if ( buttonValue != 'refresh' ) {
				nextPage[selectedPage] = 0;
			}
			wpIssuesCRM.loadSelectedPage(); 
		})

		.on( "click", ".page-link", function() {
			if ( ! $( this ).hasClass( 'ui-state-disabled' ) ) {
				var increment = ( $( this ).val() == 'next' ) ? 1 : -1; 
				nextPage[selectedPage] = nextPage[selectedPage] + increment;
				wpIssuesCRM.loadSelectedPage()			
			}
		})


		// handle keystrokes in subject finder
		.on ( "keyup", "#subject", function ( event ) { 
			var testVal = $( this ).val();
			if ( testVal != latestSearch ) {
				wpIssuesCRM.clearTimer( acTimer );
				latestSearch = testVal;
				$( this ).addClass ( "flash" );
				nextPage[selectedPage] = 0;
				acTimer = setTimeout (  wpIssuesCRM.loadSelectedPage, 600  );
			}
		})

		// have to do this here for the sweep button, bind the other action buttons in the message view
		.on( "click", "#wic-inbox-sweep-button", function( event ) {
			if ( 'inbox' == selectedPage ) {
				wpIssuesCRM.processEmail( event ) 
			} else {
				wpIssuesCRM.alert( 'Sweeping only available from Inbox.' );
			}
		})

		/*
		*
		* listeners for other areas, bound now since already loaded and so not repeated until reload
		*
		*/
		wpIssuesCRM.loadOutboxListeners();
		wpIssuesCRM.loadBlockListeners();
		wpIssuesCRM.loadSubjectListeners();
		wpIssuesCRM.loadSynchListeners();
	}



}( window.wpIssuesCRM = window.wpIssuesCRM || {}, jQuery )); // end  namespace enclosure	
