/*
*
* email-settings.js
*
*
*/

// self-executing anonymous namespace
( function( wpIssuesCRM, $, undefined ) { 

	// enclosed variable storing all the set values of the processing settings form 
	var processingOptionsObject = {};

	// form initialization -- 
	wpIssuesCRM.initializeEmailProcess = function () { 
		// quit if already initialized
		if ( $( "#mapped_threshold" ).hasClass( 'ui-spinner-input' ) ) {
			return;
		}
		$( "#mapped_threshold" ).spinner( {
				min: 0,
				max: 100
		})
		.on( "keydown", function( event ) {
			 event.preventDefault()
		});

		$( "#word_minimum_threshold" ).spinner( {
				min: 0,
				step: 5,
				max: 200
		})
		.on( "keydown", function( event ) {
			 event.preventDefault()
		});

		// change tracker for form fields
		$(" #wic-form-email-settings ").on( "change spinchange", ":input", function ( event ) {
			onChangeSaveProcessingOptions( this );
		})

		$("#wic-form-tabbed").tabs({
			heightStyle: "content",
		});	
		
		// remove any extant tinymce instances -- even when form area is reloaded, a conflicting instance object persists
		tinymce.EditorManager.editors = []; 
		// new tinymce instance for the signature field 
		wpIssuesCRM.tinyMCEInit ( "signature", false, false, function() {
	     	wpIssuesCRM.ajaxPost( 'user', 'set_wic_user_preference',  'signature', $( "#signature" ).val(),  function( response ) {
 			});
 		});
		// tinymce instance for the header field	
		wpIssuesCRM.tinyMCEInit ( "email_header", false, false, function() {
	     	$("#email_header").trigger("change");
	    });	
		
		// load options (signature loaded on startup of form
		getProcessingOptions();
		
	}


	// for saving processing options on change (excluding signature which is user specific and updated on startup
	function onChangeSaveProcessingOptions ( element) { 
		if ( 'signature' != $( element ).attr("id")  ) {
			sanitizePipeSeparated( element );
			$ ( "#wic-form-email-settings :input:not(:button)" ).not( "#wp_issues_crm_post_form_nonce_field, .wic-selectmenu-input-display, .signature" ).each( function () {
				inputElement = $ ( this );
				if ( 'checkbox' == inputElement.attr('type')  ) {
					processingOptionsObject[inputElement.attr("id")] = inputElement.prop("checked" ) // attr is initial value only
				} else {
					processingOptionsObject[inputElement.attr("id")] = inputElement.val();
				}
			});
			wpIssuesCRM.ajaxPost( 'email_process', 'save_processing_options',  0, processingOptionsObject,  function( response ) {
 			});	
 		}		
	}

	/*
	*	getProcessingOptions
	*	
	*	This function is run on load of the tabbed email settings form and is mainly how the form is populated
	*
	*	Note that if wp-issues-crm-email-processing-options is unset (first run or deleted), 
	*		(a) get_processing_options returns object with only the term variables (titles, etc.);
	*		(b) form values for other selects will come from defaults in dictionary
	*/
	function getProcessingOptions () {
		wpIssuesCRM.ajaxPost( 'email_process', 'get_processing_options',  0,'',  function( response ) {
			processingOptionsObject = response;
			// load the option values
			for (var prop in processingOptionsObject) {
				var propID = "#wic-form-email-settings #" + prop;
				inputElement = $ ( propID );
				if (  'checkbox' == inputElement.attr('type') ) {
					inputElement.prop ( "checked" ,  processingOptionsObject[prop] );
				} else {
					wpIssuesCRM.setVal ( inputElement[0], processingOptionsObject[prop], '' );
				}
			}
			// load the tinymce value for header -- was initialized possibly before return of this function;
			// by contrast signature is loaded with the form before the tinymce initialization
			tinyMCE.get('email_header').setContent( $("#email_header").val() );
			// check settings value at start up of email (this does fire on open email).
			if ( '' == $( "#disqualifiers" ).val().trim() ) {
				wpIssuesCRM.alert( 
					'<p>Please review the "Disqualifiers" setting, usually essential to good parse results.  The "Rules", "Terms" and "Titles" settings may affect results in more particular cases.</p>'
				);
			}
 		});			
	}


	/*
	* Bullet proof pipe separated fields.
	*
	*/
	function sanitizePipeSeparated ( element ) {
		var pipeSeparated = [ 'streets', 'states', 'apartments', 'special_streets', 'post_titles', 'pre_titles', 'disqualifiers' ];
		elementObject = $ ( element )
		if ( pipeSeparated.indexOf ( elementObject.attr( "id") ) > -1 ) {
			var uncleanString = elementObject.val();
			var cleanString = uncleanString.replace (new RegExp (/[^|A-Za-z0-9 -]/, 'g' ), '' );
			var termsArray = cleanString.split ( '|' );
			var trimmedArray = []; 
			var trimmedTerm = '';
			for ( i = 0; i < termsArray.length; i++ ) {
				trimmedTerm = termsArray[i].trim()
				if ( trimmedTerm > '' ) {
					trimmedArray.push ( trimmedTerm );
				}
			}
			cleanString = trimmedArray.join( '|' );
			elementObject.val( cleanString );
		}
	}


}( window.wpIssuesCRM = window.wpIssuesCRM || {}, jQuery )); // end  namespace enclosure	
