<?php
/**
*
* class-wic-admin-navigation.php
*
*/


class WIC_Admin_Navigation {
	/* 
	*
	*	All user/form requests to the plugin are routed through this navigation class, which is light -- does no database access (except dictionary, needed on one non-wic page -- for post metabox in post edit)
	*		GET Requests -- Wordpress menu and 'permitted' and 'default' strings only
	*		AJAX Form Posts -- all forms are submitted via AJAX to lower Wordpress Admin overhead
	*		AJAX Requests -- specific actions below the form level
	*		Download Button submits
	*
	*/

	// set up routing responses for each access type
	public function __construct() {
		// needed for post metabox, so load always on admin (two quick db calls)	 
		add_action ( '_admin_menu', 'WIC_Admin_Navigation::dictionary_setup' ); 	
		// get requests 
		add_action( 'admin_menu', array ( $this, 'menu_setup' ) ); // precedes admin_init
		// AJAX Form posts
		add_action( 'wp_ajax_wp_issues_crm_form', array ( $this, 'route_ajax_form' ) );
		// AJAX Requests
		add_action( 'wp_ajax_wp_issues_crm', array ( $this, 'route_ajax' ) );
		// PL Upload Requests
		add_action( 'wp_ajax_wp_issues_crm_upload', array ( $this, 'route_ajax_upload' ) );
		// Download button posts
		add_action( 'admin_init', array( $this, 'do_download' ) );			
	}	

	// dictionary necessary for almost all classes
	public static function dictionary_setup () {
		// set up dictionary
 		global $wic_db_dictionary;
		if ( empty ( $wic_db_dictionary ) ) {
	 		$wic_db_dictionary = new WIC_DB_Dictionary;
	 	} 	
	}

	/*
	*	All $_GET processing is mediated through Wordpress -- the menu and submenu page structure.
	*	Two main functions -- menu_setup and do_page control this flow. 
	*		$nav_array is used in both and limits valid GET requests (otherwise, nav logic could attempt route to arbitrary class functions)	
 	* 			keys are pages as in add_menu_page or add_sub_menu_page; values are parameters governing the page display
 	*			array also used for reverse look up for security level on AJAX requests	
	*/
	private $nav_array = array (
		'wp-issues-crm-main' =>	array (
			'name'		=>  'WP Issues CRM', 
			'default'	=>	array ( 'dashboard', 'dashboard' ),// default entity/action to invoke if $_GET does not contain permitted entity and actions
			'permitted'	=>	array ( 							// permissible entity/action pairs for the page on a GET request
								array (	'dashboard', 		'my_cases'		),
								array (	'dashboard', 		'my_issues'		),
								array (	'dashboard', 		'search_log'	),
								array (	'constituent',		'id_search'		),
								array (	'constituent',		'new_blank_form'),
								array (	'constituent',		'new_form'		),
								array (	'issue',			'id_search'		),
								array (	'issue',			'new_blank_form'),
								array (	'issue',			'new_form'		),
								array (	'search_log',		'id_search'		),
								array (	'advanced_search',	'new_blank_form' ),
								array (	'user',				'id_search' 	),
							),
			'mobile'	=>	true,								// works on small screens
			'security'	=>	false								// no special security level
			),
		'wp-issues-crm-options'	=> array (
			'name'		=>  'Options', 
			'default'	=>	array ( 'option_group', 'list_option_groups' ),
			'permitted'	=>	array ( 
								array ( 'option_group', 'new_blank_form' ),
								array ( 'option_group', 'id_search' ),
							),
			'mobile'	=>	false,
			'security'	=>	'activate_plugins'
			),
		'wp-issues-crm-fields'	=> array (
			'name'		=>  'Fields', 
			'default'	=>	array ( 'data_dictionary', 'list_custom_fields' ),
			'permitted'	=>	array ( 
								array ( 'data_dictionary', 'new_blank_form' ),
								array ( 'data_dictionary', 'id_search' ),
							),
			'mobile'	=>	false,
			'security'	=>	'activate_plugins'
			),
		'wp-issues-crm-externals'	=> array (
			'name'		=>  'Interfaces', 
			'default'	=>	array ( 'external', 'list_external_interfaces' ),
			'permitted'	=>	array ( 
								array ( 'external', 'new_blank_form' ),
								array ( 'external', 'id_search' ),
							),
			'mobile'	=>	false,
			'security'	=>	'activate_plugins'
			),
		'wp-issues-crm-settings'	=> array (
			'name'		=>  'WP Issues CRM Main Settings', 
			'default'	=>	array ( false, array( 'WIC_Admin_Settings', 'wp_issues_crm_settings' ) ),
			'permitted'	=>	array (),
			'mobile'	=>	false,
			'security'	=>	'activate_plugins'
			),
		'wp-issues-crm-storage'	=> array (
			'name'		=>  'Manage Storage', 
			'default'	=>	array ( 'manage_storage', 'form_manage_storage' ),
			'permitted'	=>	array (),
			'mobile'	=>	false,
			'security'	=>	'activate_plugins'
			),
	);
	
	// set up menu and submenus
	public function menu_setup () {  
		// get menu positioning and required security level		
		$wic_plugin_options = get_option( 'wp_issues_crm_plugin_options_array' ); 
		$menu_position = '4.19544595294'; // pick arbitrary decimal number to avoid possibility of conflicts
		$main_security_setting = isset( $wic_plugin_options['access_level_required'] ) ? $wic_plugin_options['access_level_required'] : 'activate_plugins';
		// need to run add setting  before add page -- too late to register if try not to do the work until on the page 		
		$wic_admin_settings = new WIC_Admin_Settings; 

		// add menu page
		add_menu_page( 'WP Issues CRM', 'WP Issues CRM', $main_security_setting, 'wp-issues-crm-main', array ( $this, 'do_page' ), 'dashicons-smiley', $menu_position ); 		

		// add submenu pages
		foreach ( $this->nav_array as $page => $parms ) {
			add_submenu_page( 
				$page == 'wp-issues-crm-settings' ? null : 'wp-issues-crm-main', 		// submenu of $parent_slug
				$parms['name'], 			// The text to be displayed in the title tags of the page when the menu is selected.
				$parms['name'], 			// The text to be used for the menu.
				$parms['security'] ? $parms['security'] : $main_security_setting, // The capability required for this menu to be displayed to the user.
				$page,						// The slug name to refer to this menu by (should be unique for this menu).
				array ( $this, 'do_page' )  // ) The function to be called to output the content for this page.
			);
		}
	}

	// format a page html frame and invoke class/action based on $_GET
	public function do_page (){
	
		// get page parms from nav_array (page will always be in array since it covers all the pages that access the plugin)	
		$page 	= $_GET['page'];
		$parms  = $this->nav_array[$page];
		
		// determine framing for responsive and non-responsive forms
		if ( ! $parms['mobile'] ) {
			$small_screen_plug 	= '<h3 id = "wp-issues-crm-small-screen-size-plug">' . $parms['name'] . ' not available on screens below 960px in width.' . '</h3>';
			$full_screen		= 'full-screen';
		} else {
			$small_screen_plug 	= '';
			$full_screen		= '';
		}

		// emit page
		echo $small_screen_plug;
		echo '<div class="wrap ' . $full_screen .'" id="wp-issues-crm" >';
			echo '<h1 id="wic-main-header">' . $parms['name'] . '</h1>';
			
			// processing allowed get strings or defaults for pages
			if ( 0 < count ( $_GET ) ) { 
				if ( isset ( $_GET['entity'] ) && isset ( $_GET['action'] ) && isset ( $_GET['id_requested'] )  ) {
					// filter allowed get actions
					if ( in_array ( array ( $_GET['entity'] , $_GET['action'] ), $parms['permitted'] ) ) {
						$class_short = $_GET['entity'];
						$action = $_GET['action'];
						$args   = array( 'id_requested' => ( 'user' == $class_short ? get_current_user_id() : $_GET['id_requested'] ) );
					}
				} 
				if ( !isset ( $class_short ) ) {
					$class_short = $parms['default'][0];
					$action = $parms['default'][1];
					$args	= 'user' == $class_short ? array( 'id_requested' => get_current_user_id() ) : array(); 
				}
				if ( 'wp-issues-crm-main' == $page ) { 
					$this->show_top_menu ( $class_short, $action );
				} 
				echo '<div id="wic-main-form-html-wrapper">';	
					if ( $class_short ) {  
						$class 	= 'WIC_Entity_' . $class_short; 
						$class_action = new $class ( $action, $args );
					} else {
						$action[0]::$action[1](); // static function in settings case
					}
				echo '</div>';
			}

		echo '</div>';
	}

	// button navigation for wp-issues-crm-main page
	private function show_top_menu ( $class_requested, $action_requested ) {  

		echo '<form id = "wic-top-level-form" method="POST" autocomplete = "off">';
			// go to home
			$this->a2b ( 	array ( 'dashboard', 	'dashboard',	'<span class="dashicons dashicons-admin-home"></span></span>', __( 'Dashboard.', 'wp-issues-crm' ), '', false ) );
			// search box
			$search_box = WIC_Control_Factory::make_a_control ( 'autocomplete' );
			$search_box->initialize_default_values(  'list', 'wic-main-search-box', '' );
			echo ( $search_box->form_control() );
			
			// go to constituent add
			$this->a2b ( array ( 'constituent', 	'new_blank_form',	'<span class="dashicons dashicons-smiley"></span>' ,		__( 'New Constituent', 'wp-issues-crm' ), '', false ) ); // new
			// go to issue add
			$this->a2b ( array ( 'issue', 			'new_blank_form',	'<span class="dashicons dashicons-format-aside"></span>', 	__( 'New Issue', 'wp-issues-crm' ), 		'', false ) );
			// complex search
			$this->a2b ( array ( 'advanced_search', 'new_blank_form',	'<span class="dashicons dashicons-search"></span>', 		__( 'Advanced Search', 'wp-issues-crm' ), 	'', false ) );
			// go to email processing
			$this->a2b ( array ( 'email_inbox', 	'new_blank_form',	'<span class="dashicons dashicons-email-alt"></span>', 		__( 'Process Email', 'wp-issues-crm' ), 	'', false ) );		
			// go to uploads ( but wrap in div)
			echo '<div class="wic-upload-button-wrapper">';
			$this->a2b ( array ( 'upload', 			'new_blank_form',	'<span class="dashicons dashicons-upload"></span>', 		__( 'Upload Files', 'wp-issues-crm' ), 		'wic_upload_select_button', false ) );		
			echo '</div>';
			// go to user preferences
			$this->a2b ( array ( 'settings', 		'link',				'<span class="dashicons dashicons-admin-generic"></span>', 	__( 'Main Settings', 'wp-issues-crm' ), 	'wic_settings_button', false ) );							
			// go to help link
			$this->a2b ( array ( 'help', 			'link',				'<span class="dashicons dashicons-editor-help"></span>', 	__( 'Help', 'wp-issues-crm' ),	 			'wic_help_button', false ) );		
			// nonce field
			wp_nonce_field( 'wp_issues_crm_post', 'wp_issues_crm_post_form_nonce_field', true, true ); 
			// hidden button populated as needed by jQuery to post as a top form using the routine wpIssuesCRM.mainFormButtonPost 
			echo WIC_Form_Parent::create_wic_form_button( array ( 'id' => 'wic_hidden_top_level_form_button' ) );
			// hidden input control used to pass values for exports -- note that all submittable buttons in the top level form have name wic_form_button and so go through wpIssuesCRM.mainFormButtonPost
			// the download method is via a submit (so don't have to parse attachment headers out of an AJAX response ) -- do_download runs on every post it filters out wic_form_button
			$export_parameters = WIC_Control_Factory::make_a_control ( 'text' );
			$export_parameters->initialize_default_values(  'list', 'wic-export-parameters', '' );
			echo ( $export_parameters->form_control() );
		echo '</form>';		
	}

	// a2b (array to button) 
	private function a2b ( $top_menu_button ) {
		$button_args = array (
				'entity_requested'		=>  $top_menu_button[0],
				'action_requested'		=>  $top_menu_button[1],
				'button_class'			=>  'wic-form-button wic-top-menu-button ', 	
				'button_label'			=>	$top_menu_button[2],
				'title'					=>	$top_menu_button[3],
				'id'					=>	$top_menu_button[4],
				'name'					=>  $top_menu_button[4] > '' ? $top_menu_button[4] : 'wic_form_button',
				'type'					=>	$top_menu_button[4] > '' ? 'button' : 'submit',
				'disabled'				=>	$top_menu_button[5],
			);
		echo WIC_Form_Parent::create_wic_form_button( $button_args );
	}

	/*
	*	AJAX Routing functions
	*		FORM
	*		Plain request
	*
	*/

	public function route_ajax_form() { 
	
		$this->dictionary_setup(); 
	
		$control_array = explode( ',', $_POST['wic_form_button'] );
		
		// check security and as a byproduct, get requesting page for use in history URL 		
		$entity = $control_array[0];
		$requesting_page = $this->check_security ( $entity );
		if ( false === $requesting_page ) {
			$requesting_page = 'wp-issues-crm-main';
		} 
		
		$class 	= 'WIC_Entity_' . $entity;
		$action = $control_array[1];
		$id_requested = $control_array[2];
		$args = array (
			'id_requested'			=>	$id_requested,
		);

		// do the request
		ob_start();
		$new_entity = new $class ( $action, $args );
		$output = ob_get_clean();

		/*
		* response is a state instruction to the client
		*	return_type 	= full_form or error_only
		*	state_action	= push or replace
		*	state			= new URL to show_source
		*	state_data		= new form response to show
		*/

		// set up default push state instructions
		$response				= new StdClass();
		$response->return_type 	= 'full_form'; // not returning validation errors alone, but allowing for future;
		$response->state_action = 'pushState';
		$response->state_data  	= $output;
		$resource				= '&entity=' . $entity . '&action=' . $action . '&id_requested=' . $id_requested; 

		// adjust the push state instructions for special cases
		if ( 'form_save_update' == $action ) {
			// do not change state on error messages
			if ( false === $new_entity->get_outcome() ) {
				$response->state_action = ''; 
			// on success shift resource to updated id search -- back goes to the result but not the pre-update form
			} else {
				$response->state_action = 'replaceState';
				$resource				= '&entity=' . $entity . '&action=id_search&id_requested=' . $new_entity->get_ID();
			}
		} elseif ( 'form_search' == $action ) {
			$resource 					= '&entity=search_log&action=id_search&id_requested=' . $new_entity->get_search_log_id ();
		}
		
		// complete the push state instructions
		$response->state = admin_url ( 'admin.php?page=' . $requesting_page . $resource );
	
		// send AJAX response
		wp_die( json_encode ( $response ) );
	}


	public function route_ajax () {
	
		$this->dictionary_setup();

		/**
		*	
		*
		* on client side, sending:
		*	var postData = {
		*		action: 'wp_issues_crm', 
		*		wic_ajax_nonce: wic_ajax_object.wic_ajax_nonce,
		*		entity: entity,
		*		sub_action: action,
		*		id_requested: idRequested,
		*		wic_data: JSON.stringify( data )
		*		};
		*		 
		*/	
		$entity = $_POST['entity'];
		// check security -- dies on failure
		$this->check_security( $entity ); 	
		
		// respond to request ( response function terminate with wp_die and return json_encoded response )
		$class = 'WIC_Entity_' . $entity;
		$method = $_POST['sub_action'];
		$method_response = $class::$method(  $_POST['id_requested'],  json_decode ( stripslashes ( $_POST['wic_data'] ) ) );

		wp_die ( json_encode ( (object)  $method_response ) );
						
	}	


	/* handler for calls from plupload */
	public function route_ajax_upload() {
		$this->dictionary_setup();
		$this->check_security( 'constituent' );
		WIC_Entity_Upload_Upload::handle_upload();
	}

	/*
	* action to intercept press of download button before any headers sent 
	* 
	* intended to fire whenever top level form is posted, but not if posted (through wpIssuesCRM.mainFormButtonPost) with a wic_form_button button in $_POST
	* 
	*/ 
	public function do_download () {
	
		// don't fire if doing a routine button submission or if not actually the top level form 
		if ( isset ( $_POST['wic_form_button'] ) || ! isset ( $_POST['wic-export-parameters'] ) ) {
			return;
		}
		
		$this->check_security ( 'download');

		$parameters = explode (  ',', $_POST['wic-export-parameters'] );
		$class 		= 'attachment' == $parameters[0] ? 'WIC_Entity_Email_Message' : 'WIC_List_' . $parameters[0] . '_Export';
		$method 	= 'do_' 	  . $parameters[1] . '_download';
		$type		= $parameters[2];
		$id_data	= $parameters[3];
		$class::$method ( $type, $id_data );

 	}

	
	/*
	*
	* check capabilities and nonces ( note that capability checking done by WP on gets -- no nonces on our gets (no updates) )
	*
	* if all security is passed, return requesting page (which may be false) -- false is not a security error -- die on security error
	*
	*/
	private function check_security ( $entity ) {
	
		// do a reverse lookup in the nav_array to find if entity has special security
		$found_entity = false;
		$entity_special_security = false;
		$requesting_page = false;
		foreach ( $this->nav_array as $page => $parms ) {
			foreach ( $parms['permitted'] as $permitted ) {
				if ( $entity == $permitted[0] ) {
					$found_entity = true;
					break;
				}
			}
			if ( $found_entity ) {
				$entity_special_security = $parms['security'];
				$requesting_page = $page;
				break;
			}
		}

		// set security level based on entity special or based on general access level
		$wic_plugin_options = get_option( 'wp_issues_crm_plugin_options_array' ); 
		if ( $entity_special_security )  {
			$required_capability = $entity_special_security; 
		} elseif ( 'download' == $entity ) { // covers multiple entities in download context
			$required_capability = isset( $wic_plugin_options['access_level_required_downloads'] ) ? $wic_plugin_options['access_level_required_downloads'] : 'activate_plugins';
		} else {
			$required_capability = isset( $wic_plugin_options['access_level_required'] ) ? $wic_plugin_options['access_level_required'] : 'activate_plugins';
		}
		
		// should never be failing this except as an application error or a deliberate security breach, so just die
		if ( ! current_user_can ( $required_capability ) ) {
			wp_die( sprintf( __( 'Please consult your administrator for access to %s functions', 'wp-issues-crm' ), $entity ) );	
		}

		// now check nonce -- either of two possible nonces are OK
		$request_nonce 	= isset ( $_POST['wic_ajax_nonce'] ) ?  $_POST['wic_ajax_nonce'] : '';
		$form_nonce 	= isset ( $_REQUEST['wp_issues_crm_post_form_nonce_field'] ) ? $_REQUEST['wp_issues_crm_post_form_nonce_field'] : '' ; // $_REQUEST accomodates nonce from download gets in dialog boxes

		if ( ! wp_verify_nonce ( $request_nonce, 'wic_ajax_nonce' )  && 
			 ! wp_verify_nonce ( $form_nonce, 'wp_issues_crm_post' )
			) { 
			 die ( __( 'Bad or expired security code.  Try refreshing page.', 'wp_issues_crm' ) );		
		}
		
		return ( $requesting_page );
		
	}

}