<?php
/*
* class-wic-db-email-message-object.php
*	supports retrieval of lists in format like $wpdb output
*
* use utf-8 on where pattern matching may include utf-8
*/

class WIC_DB_Address_Block_Object {

	public $email 		= ''; // address blocks often have an email
	public $phone 		= ''; // address blocks often have a phone
	public $first_name	= '';	
	public $middle_name	= '';
	public $last_name	= '';
	public $apartment_line = '';
	public $street_address_line		= '';
	public $city		= '';
	public $state 		= '';
	public $zip 		= '';	

	public function populate ( $text_block ) {

		/*
		* set up address parsing pattern strings
		* state and territory abbreviations from http://pe.usps.gov/text/pub28/28apb.htm
		* street abbreviations from http://pe.usps.gov/text/pub28/28apc_002.htm
		* secondary unit designators from http://pe.usps.gov/text/pub28/28apc_003.htm#ep538629
		*/
		
		$form_variables_object = unserialize ( get_option ( 'wp-issues-crm-email-processing-options' ) ); 
		$states 			= $form_variables_object->states;
		$streets 			= $form_variables_object->streets;
		$special_streets 	= $form_variables_object->special_streets;
		$apartments 		= $form_variables_object->apartments;
		
   		$line_array = preg_split ( '#\n#', $text_block, NULL, PREG_SPLIT_NO_EMPTY );

		/*
		* read block lines in reverse -- city/state/zip line is most reliably identifiable
		* -- use it as screener for identifying other address lines positively
		* -- also, would like to nail down city/state/zip before name:
		* 	title in name can get interpreted as state abbrevations -- John Doe, MD looks city in Maryland
		* -- capture phone and additional email if included
		* 
		*/
		$found_city_state_line = -1;
		$unused_preaddress_line_index = -1;
			
		for ( $i = ( count ( $line_array ) - 1 ); $i > -1; $i-- ) {
			
			// Note that lines are already filtered by text sanitizer A-Za-z0-9 .@_%+/\-\n\r
			// Blocks are trimmed, but could be white space at end of lines -- strip that
			$line = trim ( $line_array[$i] , ". " ); 
			$test = 0; // set test to zero for each line -- avoid unnecessary scanning
			// if we don't have an email yet, see if this line includes one
			if ( '' == $this->email ) {
				$email_matches = array();	
				$test = preg_match ( '#(?i)\b[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,}\b#', $line, $email_matches, PREG_OFFSET_CAPTURE, 0);
				if ( 1 == $test ) {
					if ( 
						// if email source selection set to auto, accept if at first position in line
						( 1 == $form_variables_object->email_source && 0 == $email_matches[0][1] ) ||
						// also accept email if allowing or forcing acceptance of body email regardless of line positioning
						2 == $form_variables_object->email_source ||
						3 == $form_variables_object->email_source
						) { 
							$this->email = $email_matches[0][0];
					}
				}
				
			}
			// if it isn't an email
			if ( 0 == $test ) {
				// and we don't have a phone, see if this is one
				if ( '' == $this->phone ) {
					$raw_phone = preg_replace ( '#[^0-9]#', '', $line);
					if ( 10 == strlen ( $raw_phone ) ) {
						$test = 1;
						$this->phone = $raw_phone;
					} 
				}			
			}

			// if this isn't an email or a phone, look for true address elements
			if ( 0 == $test ) {
				// and we don't have a city/state line, see if it is one
				if ( '' == $this->city ) {
					// set up variable reflecting zip strictness options
					if ( 2 == $form_variables_object->zip_required  ||
						 3 == $form_variables_object->zip_required ) {
						$zip_connector_regex = '[ -]{0,3}';	 
					} else {
						$zip_connector_regex = '-';
					}
					// set up variable reflecting capitalization strictness option
					$case_insensitive = ( 2 == $form_variables_object->capitalization ) ? '(?i)' : '';
					// set up variable for city word count
					$city_word_count = $form_variables_object->city_word_count;
					// assemble pattern
					$state_or_zip_last_pattern = 
						'#' . $case_insensitive // delimiter ( variable case sensitivity )
						// open first capturing group -- city words ( note that 0 position in match array is occupied by the full pattern )
						. '('
						.'^(?:[A-Z][A-Za-z\-]*[ .]{1,3}){1,' .	$city_word_count . '}' // one to four words (might be single letter) each followed by by spaces or punctuation
						. ')'
						// note that in the php regex, \b asserts the existence of a change rather than representing characters
						. '('  // open second capturing group -- state
						. $states
						. ')' // end capturing group (state)
						. '(?:[ .]{1,3}|\b)' // one to three word boundary characters punctuation or end of line-- 
						. '(' // optional zip code in capturing third group 
						. '\d{5}(?:' . $zip_connector_regex . '\d{4})?' // zip code pattern   
					 	. ')?'	// close paren for zip code capture and marked optional
					 	. '$#u'; // end of string (already trimmed above) and delimiter
					$state_zip_matches = array(); 
					if 	( 1 == preg_match ( $state_or_zip_last_pattern, $line, $state_zip_matches, PREG_OFFSET_CAPTURE ) ) { 
						// if last token is zip code or 
						$this->city = trim ( $state_zip_matches[1][0], '. ' );
						$this->state = $state_zip_matches[2][0];
						$this->zip 	= isset ( $state_zip_matches[3] ) ? $state_zip_matches[3][0] : '';
						$found_city_state_line = $i;
						$test = 1;
					}
				} 
			}	
			// if haven't already used the line
			if ( 0 == $test ) {
				// and we don't have a street_address line
				if ( '' == $this->street_address_line && 
					// and we have seen a city state line working up from the bottom
					$found_city_state_line > -1 &&
					// and we haven't gone more than two lines further up (could have apt line after street address)
					$found_city_state_line - $i < 3 ) {
						/*
						* taking a loose approach to street address confirmation
						* have already narrowed block to one with a city/state/zip, so 
						* false positive rate should be low although many non-address phrases would screen in
						* 	-- e.g., "three dogs played on the beach"
						* deeming it better to capture sloppy user input in incoming emails
						*/
						$street_address_line_pattern = 
						'#(?i)' // delimiter and case insensitive mode
						// begin group of possible numbers
						. '^(?:one|two|three|four|five|six|seven|eight|nine|zero|'
						// continue possible numbers -- street number (up to six digits combined with up to five letters 
						. '\d{1,6}[A-Z]{0,5})'
						// initial number must be followed by space (not punctuation, up to three)
						. ' {1,3}' 
						// follow with one to four 1+ letter word/numbers followed by by spaces or punctuation
						// OR, a special street, standing alone (no street designator)
						. '((?:' . $special_streets . ')|'
						. '(?:[A-Z0-9-]{1,20}[ .]{1,3}){1,4}' 
						. '(?:' // open a none-capturing group for street designator
						. $streets
						. ')' // end non-capturing group (street designator)
						. ')' // end wraparound group allowing special one word streets'
						. '([ .]|\b){1,3}' // one to three word boundary characters or (boundary so end is OK) -- 
						. '(?:[A-Z0-9-]{1,10}([ .]|\b){0,3}){0,4}' // post directional and apt designator "SW APT 1 REAR"  
					 	. '$#u';
					// look for street match pattern
					if 	( 1 == preg_match ( $street_address_line_pattern, $line ) ) { 
						$this->street_address_line = $line;
						$test = 1;
						// this flag may be set > -1 below in prior loops 
						// this is a reset of the flag
						$unused_preaddress_line_index = -1; 
					}
				} 
			}		

			// if haven't already used the line
			if ( 0 == $test ) {
				// and we don't have an apartment line, see if it is one
				if ( '' == $this->apartment_line && 
					// and we have seen a city state line working up from the bottom
					$found_city_state_line > -1 &&
					// and we haven't gone more than two lines further up (could have apt line before street address)
					$found_city_state_line - $i < 3 ) {
						/*
						* As above, taking a loose approach given narrow context
						* -- can imagine long strings before or after this type of designator, especially for departments
						* 
						* Just want to avoid initial numbers as in street
						*/
						$apt_line_pattern = 
						'#(?i)' // delimiter and case insensitive mode
						. '^(?:' 
						.  $apartments // starts with an apartment word
						. ')'
						. '([ .]|\b){1,3}'  // followed by space period or end
						. '(?:[A-Z0-9-]{1,10}([ .]|\b){0,3}){0,2}' // up to two trailing words not more than 10 chars, not more than 3 spaces after (.  )
						. '$#u'; // delimiter -- with end anchor							
					if 	( 1 == preg_match ( $apt_line_pattern, $line ) ) { 
						$this->apartment_line = $line;
						$test = 1;
					}
				} 
			}
			// if on line before GOOD city/state/zip and haven't burned it 
			// and it isn't first line, take note -- may need to use it as plug later
			if ( 0 == $test && $i == $found_city_state_line - 1 && $i > 0 ) {
				$unused_preaddress_line_index = $i;			
			}
			// if on first line and haven't burned it otherwise and have an address,
			//  . . .try it as a name line 
			if ( 0 == $test && 0 == $i && $found_city_state_line > -1 ) {
				$name = $this->parse_name( $line );
				if ( false !== $name ) {
					$this->first_name 	= $name['first'];
					$this->middle_name 	= $name['middle'];
					$this->last_name 	= $name['last'];
				}
			}	
		}  // process line array 

		// also, since done, if didn't burn pre-address line and didn't find another good streetaddress
		// ( if find a good street address line, reset to -1 )
		// likely institutional address or well-known building name like State House
		if ( $unused_preaddress_line_index > -1  ) {
			$this->street_address_line = trim ( $line_array[$unused_preaddress_line_index] , ". " ); 
		}	

		// at this stage, do not want to discard the block . . . 
		if 	( 
			  // if it has an email that is part of an address block -- auto email_source
			  (
			  	$this->email > '' && 
			  	1 == $form_variables_object->email_source &&
			  	( $this->zip > '' || $this->city > '' )
			  ) || 
			  // if it has an email at all -- soft
			  (
			  	$this->email > '' && 
			  	( 2 == $form_variables_object->email_source || 3 == $form_variables_object->email_source  )
			  ) ||			    
		 	  // if it has a zip code 
			  $this->zip > '' 	||  
			  // or if has address without zip and but zip not required in settings	
			  ( $this->city > '' && $form_variables_object->zip_required == 3 ) ) {	
			return ( true );
		} else {
			return ( false );
		}
	} // populate
	
	public static function parse_name ( $name_line ) {

		// set up parse variables
		$form_variables_object = unserialize ( get_option ( 'wp-issues-crm-email-processing-options' ) ); 
		$pre_titles 			= $form_variables_object->pre_titles;
		$post_titles 			= $form_variables_object->post_titles;
		$titles 				= $pre_titles . '|' . $post_titles;
		$minimum_name_length 	= $form_variables_object->minimum_name_length;
		$maximum_name_length 	= $form_variables_object->maximum_name_length;

		// set up return array
		$name = array (
			'first' => '',
			'middle' => '',
			'last' => '',
			'found_title' => false
		);

		// make sure line as no padding 
		$name_line = trim ( $name_line, ',. ');
		
		// match a single pre-title and any trailing punctuation -- must have boundary
		$pre_title_check_pattern = '#(?i)' 
			. '^(?:' . $pre_titles . ')[, \.]+' // start, a pretitle, some punctuation or space
			. '#u';
		
		// match a series of post titles to the end
		$post_title_check_pattern = '#(?i)' 
			. '([, \.]+(?:' . $post_titles . ')){1,4}[, \.]*$' // some punctuation, a post_title and more punctuation or end
			. '#u';
		
		// test for and strip pre and post titles
		$found_title = false;
		if ( 1 == preg_match ( $pre_title_check_pattern, $name_line ) ) {
			$name['found_title'] = true;	 
			$name_line = preg_replace ( $pre_title_check_pattern, '', $name_line ); 
		} 
		if ( 1 == preg_match ( $post_title_check_pattern, $name_line ) ){	
			$name['found_title'] = true;
			$name_line = preg_replace ( $post_title_check_pattern, '', $name_line );
		
		}							

		// having stripped titles, if have less than min or more than max words, probably not really a name -- quit
		$name_match_pattern = '#'  // delimiter
			. '^([\pL-]{1,20}\b[, \. ]{0,3}){' . $minimum_name_length . ',' . $maximum_name_length . '}' 
			. '$#u'; // end and delimiter
		if ( 0 == preg_match ( $name_match_pattern, $name_line ) ) {
			return ( false );
		}
		
		
		// if there is a remaining comma, it should be signalling that first word is last name
		// this will not occur normally when called from address processing because commas are filtered out
		// but this branch will be exercised when function is called statically from elsewhere
		$ln_matches = array();
		// if have a string of words followed by a comma (know that have 2-4 words), then assume that these are last names
		// commas have already been stripped from end (initially on trim and further on removal of any titles)
		// so, this implies have two names at least, no need to test minimum
		if ( 1 ==  preg_match( '#(^[\p{L}_\- ]{3,60}),#u', $name_line, $ln_matches ) ) {
			$name['last'] = $ln_matches[1];
			$remaining_names = preg_replace ( '#'. $ln_matches[0] . '#u', '', $name_line );
			$name_tokens = preg_split ( '#[\., ]+#',  $remaining_names, NULL, PREG_SPLIT_NO_EMPTY );
			$name['first'] = isset ( $name_tokens[0] ) ? trim($name_tokens[0]) : '' ;
			if ( count ( $name_tokens ) > 1 ) {
				array_shift ( $name_tokens );
				$name['middle'] = implode ( ' ', $name_tokens );
			}
		// if no comma, split into tokens and process in order -- assume last name is last token
		// assign extra names to middle name
		} else {
			$name_tokens = preg_split ( '#[\. ]+#', $name_line, NULL, PREG_SPLIT_NO_EMPTY );
			$num_names = count ( $name_tokens ); 
			if ( $num_names > 0 ) {
				$name['last'] = $name_tokens[ $num_names - 1 ];
				if ( $num_names > 1 ) {
					$name['first'] = $name_tokens[ 0 ];
					if ( $num_names > 2 ) {
						for ( $k = 1; $k < $num_names - 1; $k++ ) {
							$name['middle'] .= $name_tokens[$k] . ' ';
						}
						$name['middle'] = trim( $name['middle'] );
					} // > 2 tokens
				} // > 1 token 	
			} else { //  0 tokens
				return ( false );
			}	
		} // no comma else
		return ( $name );
	} // parse name		
} // class 