<?php
/*
* 	class-wic-db-email-message-object.php
*
*	takes an email (identified by imap_stream and UID) and extracts and sanitizes results
*	
*	see classification of outcome codes	
*
*   this object is saved as serialized by the parse process and is retrieved by the inbox detail view and by the email processor
*
*/

class WIC_DB_Email_Message_Object {
	
	/*
	* this is the master template 
	*   upload tables derived directly from it for maintainability
	*	-- see wic_db_access_upload_email.php
	*
	* use utf-8 options on for preg_match where pattern may contain utf-8
	*/
	// final product properties based on all information
	public $email_address	= '';
	public $phone_number 	= ''; 
	public $first_name	= '';	
	public $middle_name	= '';
	public $last_name	= '';
	public $address_line = '';
	public $city		= '';
	public $state 		= '';
	public $zip 		= '';
	public $activity_date = '';	
	public $apartment_only_line = '';
	public $street_address_only_line = '';
	
	// summary statement of quality -- see discussion below
	public $outcome;
	/*	
	*	OUTCOME IS CLASSIFICATION (NOT RANKING) OF CASES 
	*	(NAME PARSEABLE FROM BODY X ADDRESS PARSEABLE FROM BODY -- 3 VALID PERMUTATIONS) 
	*
	*	NO ADDRESS OR NAME PARSEABLE FROM BODY TEXT 
	*	(NAME, IF PARSEABLE FROM HEADERS, STILL REQUIRED TO CHECK TO BODY)
	*	(Subcases: email in body x name in header)
	*	======================================================================================
	*	0 = no successful body or header parse at all (just have email from headers);
	*	1 = good name parsed from headers, but no body email 
	*		( name came from headers and matches into body )
	*		can't be sure in this case that bulker included a good email in headers --
	*		might just do name; should never overwrite constituent email address based on this
	*	2 = body email found -- but no name parsed from headers
	*		this is probably good email, but not sure who to attribute it to, may match anyway;
	*		This case could happen if headers not parseable for name and no full address block;
	*		probably is present, just not found if supplied body email.
	*	3 = body email found and good name parsed from headers -- this is a highly 
	*		probable good name and address combination
	*
	*	ADDRESS, BUT NOT NAME PARSEABLE FROM BODY TEXT
	*	(Subcases: email in body x name in header)
	*	==================================================================================
	*	4 = no body email, no good name from header
	*	5 =	no body email, good name from headers 
	*	6 = body email, no good name from headers
	*   7 = body email, good name from headers 
	*
	*	ADDRESS AND NAME PARSEABLE FROM BODY TEXT
	*	(Believe avoided getting tricked by to address -- not worrying that choice now)
	*	(Subcases: email in body x verify name in header)
	*	==================================================================================
	*	8 = no body email -- header name not found in body
	*	9 = no body email -- header name found in body
	*   10 = body email -- header name not found in body
	*	11 = body email -- header name found in body 
	*
	*	GROUPING OUTCOMES AS FOLLOWS ( RANKING )
	*		STRICT WITH NAME & ADDRESS-- have both name and address and good source for both email and name
	*		Outcomes 7,10,11 
	*		MEDIUM WITH NAME & ADDRESS -- have both name and address, but less confirmation on name and/or email
	*		Outcomes 5,8,9 
	*		MEDIUM WITH NAME ONLY
	*		Outcome 1,3
	*		MEDIUM WITH NO NAME (EMAIL ONLY OR ADDRESS WITHOUT NAME)
	*		Outcome 2,6
	*		LOOSE
	*		Outcomes 0,4
	*
	*/
	
	// object properties corresponding to parsed object from headerinfo()
	public $to_personal;		// extracted from to object
	public $to_email;			// extracted from to object
	public $from_personal;		// extracted from from object
	public $from_email;			// extracted from from object	
	public $from_domain; 		// domain portion of $from_email
	public $reply_to_personal;	// extracted from reply_to object
	public $reply_to_email;		// extracted from reply_to object
	public $to_array; 			// processed from to object;
	public $cc_array; 			// processed from from object	
	public $sender_personal;	// extracted from sender object
	public $sender_email;		// extracted from sender object	
	public $raw_date;			// The message date as found in its headers 
	public $email_date_time;	// The message date in timestamp format (for display and sorting only)	
	public $subject; 			// The message subject 
	public $message_id; 		// Unique identifier (permanent unique identifier -- retained only for research purposes)
	// structure parts accessed through imap body fetch functions
	public $text_body;  		// stripped for parsing -- prefer to generate from raw text body but can strip from raw html body
	public $raw_text_body;		// used as copy of original in reply emails and in activity records
	public $raw_html_body;		// only used for display in message viewer
	public $sentence_md5_array; // used to lookup previous similar mappings
	// temporary fields for control of inlining images 
	private $temp_image_id_regex_array; 
	private $temp_image_array; 
	private $total_parts_size;
	private $max_packet_size;
	// error properties
	public $uid_still_valid; // was uid available at start of process -- not captured by a process_email running ahead of this one
	// identifier -- need for attachment links
	public $inbox_image_id;
	public $has_attachments = false;
	/* 
	* 
	* main message object build function
	*
	* ID is inbox image message ID, UID is the server UID
	*/
	public function build_from_stream_uid ( $ID, &$IMAP_stream, $UID, $max_packet_size ) {
		$this->inbox_image_id = $ID;
	
		// set max_packet_size for use in walker (also test vs 16m reasonable memory usage)
		$this->max_packet_size = $max_packet_size < 16000000 ? $max_packet_size : 16000000; // waste no room for overhead -- better to downsize in parse_messages if necessary 
	
		// get header info object, fail if not found uid
		$this->uid_still_valid = true; 
		$header = false;
		$header = @imap_fetchheader ( $IMAP_stream, $UID, FT_UID );
		if ( ! $header ) {
			$this->uid_still_valid = false;
			return;		
		}
		$imap_headerinfo			= imap_rfc822_parse_headers( $header  );

		// load properties from header info -- note that all the non-address fields can be encoded, so decode them all
		$this->to_personal			= isset ( $imap_headerinfo->to ) ? $this->email_array_to_display_safe_personal ( $imap_headerinfo->to ) : '';
		$this->to_email				= isset ( $imap_headerinfo->to ) ? $this->email_array_to_display_safe_address  ( $imap_headerinfo->to ) : '';
		$this->from_personal		= isset ( $imap_headerinfo->from ) ? $this->email_array_to_display_safe_personal ( $imap_headerinfo->from ) : '';
		$this->from_email			= isset ( $imap_headerinfo->from ) ?  $this->email_array_to_display_safe_address  ( $imap_headerinfo->from ) : '';
		$this->from_domain			= $this->domain_only ( $this->from_email );
		$this->reply_to_personal	= isset ( $imap_headerinfo->reply_to ) ? $this->email_array_to_display_safe_personal ( $imap_headerinfo->reply_to) : '';
		$this->reply_to_email		= isset ( $imap_headerinfo->reply_to ) ? $this->email_array_to_display_safe_address  ( $imap_headerinfo->reply_to) : '';	
		$this->sender_personal		= isset ( $imap_headerinfo->sender ) ? $this->email_array_to_display_safe_personal ( $imap_headerinfo->sender ) : '';
		$this->sender_email			= isset ( $imap_headerinfo->sender ) ? $this->email_array_to_display_safe_address ( $imap_headerinfo->sender ) : '';			
		$this->raw_date		 		= isset ( $imap_headerinfo->date ) 		?  	self::sanitize_incoming ( $imap_headerinfo->date ) : '' ;			
		$this->subject				= isset ( $imap_headerinfo->subject ) 	?	self::sanitize_incoming( $imap_headerinfo->subject ) : '' ;
		$this->message_id			= self::sanitize_incoming ( $imap_headerinfo->message_id ); 			
		// repack arrays of from/to information
		$this->to_array = self::repack_address_array( isset ( $imap_headerinfo->to ) ? $imap_headerinfo->to : false );
		$this->cc_array = self::repack_address_array( isset ( $imap_headerinfo->cc ) ? $imap_headerinfo->cc : false );
		// sanitize_date ( will be blank if cannot process )
		$this->activity_date = WIC_Control_Date::sanitize_date( $this->raw_date );		
		$this->email_date_time = self::sanitize_date_time( $this->raw_date );		
		/*
		*
		* now extract body parts
		*
		*/
		// begin structure walk by fetching top level structure
		$imap_structure = imap_fetchstructure( $IMAP_stream, $UID, FT_UID );
		if ( ! isset ( $imap_structure->parts ) ) { // simple
        	$this->message_structure_walker( $IMAP_stream, $UID, $imap_structure, 0 );  // pass 0 as part-number
   		} else {  
			/*
			* multipart: cycle through each part
			* assumption is that 1 + partno0 ( the index in the array ) corresponds to the numbering of the parts 
			* $partno = '1', '2', '2.1', '2.1.3', etc for multipart, 0 if simple (see recursion below)
			*/
			foreach ( $imap_structure->parts as $partno0=>$p ) {
				$this->message_structure_walker( $IMAP_stream, $UID, $p, $partno0 + 1 );
			}
        }
        
		/*
		* after completing structure walk, may or may not have raw text and html bodies -- if do, both have been through kses system already
		*
		* if have raw_text_body, strip it for later use in parsing; if not, strip the html body for parsing
		*
		* if have raw_html_body, use it for display after inlining any available images; if not, format the text body as html for display
		*/
		// text_body is hard strip of raw_text or html body, basically preserving words and breaks, no tags (prefer text version -- does better in MS styled submissions)
		$this->text_body = $this->raw_text_body ? self::utf8_to_sanitized_text( $this->raw_text_body ) : self::utf8_to_sanitized_text( $this->raw_html_body ) ;
		// html body, if present at this stage is safe html and want to inline images (already validated as safe); otherwise use text, but with p's inserted and tags balanced
		$this->raw_html_body = $this->raw_html_body ? 
			( $this->temp_image_id_regex_array ? preg_replace( $this->temp_image_id_regex_array, $this->temp_image_array, $this->raw_html_body, -1, $count_replacements ) : $this->raw_html_body )  : 
			wpautop( balanceTags( $this->raw_text_body ) ) ;	
			// note that this regex insertion could break packet size rules if large image was used repeatedly -- catch this in the ultimate update
	
		// no further use for this and gets saved with object, blowing up size
		unset ( $this->temp_image_array );
		
		// do address extraction and build non_address_block md5 map
		$this->extract_address();

	}

	// reinstatiate object from database
	public static function build_from_image_uid ( $folder, $UID ) {
	
		global $wpdb;
		$inbox_table = $wpdb->prefix . 'wic_inbox_image';

		$sql = "
			SELECT * 
				FROM $inbox_table
				WHERE 
					full_folder_string = '$folder' AND
					folder_uid = $UID AND
					no_longer_in_server_folder = 0 AND
					to_be_moved_on_server = 0 AND
					serialized_email_object > ''	
				";
		$message_array = $wpdb->get_results( $sql );

		// unserialize the main object
		if ( $message_array ) {
			$message_object = unserialize( $message_array[0]->serialized_email_object );
		} else {
			return false;
		}
		
		if ( $message_object ) {
			// add in properties maintained unserialized on record
			$message_object->inbox_image_id				=  $message_array[0]->ID;
			$message_object->mapped_issue 				=  $message_array[0]->mapped_issue;
			$message_object->mapped_pro_con 			=  $message_array[0]->mapped_pro_con;
			$message_object->guess_mapped_issue 		=  $message_array[0]->guess_mapped_issue;
			$message_object->guess_mapped_issue_confidence = $message_array[0]->guess_mapped_issue_confidence;
			$message_object->guess_mapped_pro_con 		=  $message_array[0]->guess_mapped_pro_con;
			$message_object->non_address_word_count 	=  $message_array[0]->non_address_word_count;
			$message_object->assigned_constituent	 	=  $message_array[0]->assigned_constituent;
			$message_object->parse_quality	 			=  $message_array[0]->parse_quality;
			$message_object->display_date				=  $message_object->raw_date;
			// at time of parse, attempted to find matching constituent
			if ( $message_array[0]->assigned_constituent ) {
				$constituent_object = WIC_DB_Access_WIC::get_constituent_names( $message_array[0]->assigned_constituent );
				if ( $constituent_object ) {
					$message_object->first_name = $constituent_object->first_name 	? $constituent_object->first_name 	: $message_object->first_name;
					$message_object->last_name 	= $constituent_object->last_name 	? $constituent_object->last_name 	: $message_object->last_name;
				}
			} 
			return ( $message_object );
		} else {
			return false;
		}
		
	}

	// reinstatiate object from database by ID -- streamlined for message viewer
	public static function build_from_id ( $selected_page, $ID ) {
	
		global $wpdb;
		$table = 'done' == $selected_page ? $wpdb->prefix . 'wic_inbox_image' : $wpdb->prefix . 'wic_outbox' ;

		$sql = "
			SELECT * 
				FROM $table
				WHERE ID = $ID
				";
		$message_array = $wpdb->get_results( $sql );

		// unserialize the main object if found
		if ( $message_array ) {
			$message_object = unserialize( $message_array[0]->serialized_email_object );
		} else {
			return false;
		}
		
		// return the unserialized object if successful
		if ( $message_object ) {
			switch ( $selected_page ) {
				case 'done':
					$message_object->display_date =  $message_object->raw_date;
					break;
				case 'outbox':
					$message_object->display_date =  $message_array[0]->queued_date_time;
					break;
				case 'sent':
					$message_object->display_date =  $message_array[0]->sent_date_time;
					break;			
			}
			return $message_object;
		} else {
			return false;
		}
		
	}
	
	/*
	*
	*
	* message body parts assembly functions
	*
	*
	*/
	
	// using function modeled on user comment at http://php.net/manual/en/function.imap-fetchstructure.php
	private function message_structure_walker ( $IMAP_stream, $UID, $p, $partno ) {

		// set up type, disposition, size and id variables
		$subtype 		= $p->ifsubtype ?  strtolower( $p->subtype ) : '';
		$disposition 	= $p->ifdisposition ? strtolower( $p->disposition ) : '';
		$id 			= $p->ifid ? $p->id : '';
		$bytes 			= isset ( $p->bytes ) ? $p->bytes : 0;
	
		/*
		* fetch body and clean it up
		*/
		$data = ( 0 < $partno ) ?
			imap_fetchbody( $IMAP_stream, $UID, $partno, FT_UID ) : // multipart
			imap_body( $IMAP_stream, $UID, FT_UID ); // simple

		// Any part may be encoded, even plain text messages, so check everything.
		if ( $p->encoding == ENCQUOTEDPRINTABLE ) {
			$data = quoted_printable_decode( $data );
		} elseif ( $p->encoding == ENCBASE64 ) {
			$data = base64_decode($data);
		}

		// if a known charset, do conversion
		$charset = false;
		foreach ( $p->parameters as $parameter ) {
			if ( 'charset' == strtolower( $parameter->attribute ) ) {
				$charset = $parameter->value;
			}
		}	
		if ( $charset ) {
			$data =  iconv ( $charset, "UTF-8//IGNORE", $data );
		}

		/* 
		* handle retrieved body	
		* 
		*/
		// text attachments
		if ( $p->type == TYPETEXT && $data ) {
			// want a legible but UTF-8 sanitized copy for display as message text (JSON-encode requires UTF-8 and MYSQL requires 3byte UTF-8)
			if ( $charset ) {
				// strip high plane, see sanitize_incoming
				$temp_body = preg_replace('/[\x{10000}-\x{10FFFF}]/u', "\xEF\xBF\xBD", $data );
			} else {
				// otherwise just make sure it is ASCII ( with a UTF-8 place holder character )
				$temp_body = preg_replace("/[^\x01-\x7F]/","\xEF\xBF\xBD",$data );
			}
			// further sanitize and use the body if haven't got main parts yet
			if ( ( ''  == $subtype || 'plain' == $subtype ) && '' == $this->raw_text_body ) {
				// note: here sanitize raw_text_body as if html -- all incoming is fully sanitized to display safe later
				$this->raw_text_body = wp_kses_post( self::strip_html_head( $temp_body ) ); 
				$this->total_parts_size = $this->total_parts_size + 2 * strlen( $this->raw_text_body ); // count into part size ( as raw and rough sanitized )
			} elseif ( 'html' == $subtype && '' == $this->raw_html_body ) {
				$this->raw_html_body = wp_kses_post( self::strip_html_head( $temp_body ) );
				$this->total_parts_size = $this->total_parts_size + strlen( $this->raw_html_body ); // count into part size ( only once; likely that text present and have catch all at storage step )
			} else {
				$this->handle_attachment( $p, $partno, $data );
			}
		// inline images ( presume can inline if not marked as attachments )
		} elseif ( 
			TYPEIMAGE == $p->type && 										// have an image
			$data &&														// not trivial
			ENCBASE64 == $p->encoding &&									// base64 encoding claimed
			0 === preg_match( '#[^A-Za-z0-9+/=\s]#', $data ) &&  			// base64-ish in fact -- no tag breaking characters
			$this->max_packet_size > $this->total_parts_size + $p->bytes &&	// won't make for too big a message
			$subtype &&														// have a nonblank image subtype
			$disposition != 'attachment' &&									// not directed to be an attachment
			$p->id															// have a non-blank identifier for the file
			 ) 	{
			//prepare to inline image
 			$this->total_parts_size = $this->total_parts_size + $p->bytes;   			
			$this->temp_image_array[] = 'src="data:' . 'image/' . $subtype . ';base64, ' . $data . '"'; // assumes base64 -- warranted? 
			$this->temp_image_id_regex_array[] = '#src\s*=\s*(\'|")(cid:)?' . preg_replace( '#(^<|>$)#', '', $p->id ) . '\1#';
		// other attachments
		} elseif ( $bytes && $data) {										// is a base level part with content
			$this->handle_attachment( $p, $partno, $data );
		}
		/*
		* re base64 verification, see:
		* 	https://en.wikipedia.org/wiki/Base64
		* 	http://stackoverflow.com/questions/475074/regex-to-parse-or-validate-base64-data
		*		note: php cannot handle this regex: preg_match ( '#^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=)?$#', $datax );
		*   	generates segmentation fault even though groups are non-capturing!!!!
		*	 http://www.ietf.org/rfc/rfc4648.txt
		*/
		// SUBPART RECURSION
		if ( isset ( $p->parts ) ) {
			foreach ( $p->parts as $partno0=>$p2 ) {
				$this->message_structure_walker( $IMAP_stream, $UID, $p2, $partno . '.' . ( $partno0 + 1) ); // 1.2, 1.2.1, etc.
			}
		}
		
	} // message_structure walker

	private function handle_attachment(  $p, $partno, $data ) {

		// array of type codes -- http://php.net/manual/en/function.imap-fetchstructure.php
		$attachment_types = array ( 'TEXT', 'MULTIPART', 'MESSAGE', 'APPLICATION', 'AUDIO', 'IMAGE', 'VIDEO', 'MODEL', 'OTHER' );

		// first check if have a proper file name -- check in dparameters (s/b as in MIME header) and parameters
		// http://php.net/manual/en/function.imap-fetchstructure.php
    	$file_name = '';
        if ( $p->ifdparameters ) {
        	if ( is_array ( $p->dparameters ) ) {
            	foreach ( $p->dparameters as $pair ) {
                	if ( strtolower ( $pair->attribute ) == 'name' || strtolower ( $pair->attribute ) == 'filename' ) {
                    	$file_name = strtolower ( $pair->value );
                    	break;
              	  	}
            	}        	
        	}
        } 
        if ( $p->parameters && ! $file_name ) {	
        	if ( is_array ( $p->parameters ) ) {
            	foreach ( $p->parameters as $pair ) {
                	if ( strtolower ( $pair->attribute ) == 'name' || strtolower ( $pair->attribute ) == 'filename' ) {
                    	$file_name = strtolower ( $pair->value );
                    	break;
              	  	}
            	}        	
        	}
        } 

		// strip any embedded tags that might break html; doing prepare to protect against injection
		$file_name = sanitize_text_field( $file_name );
		// enforce length reasonability
		$file_name = ( strlen($file_name ) > 3 && strlen( $file_name ) < 254 ) ? $file_name : ''; 
		
		// note that have attachments
		$this->has_attachments = true;
		// do the save
		global $wpdb;
		$attachments_table = $wpdb->prefix . 'wic_inbox_image_attachments';		
		$sql = $wpdb->prepare ( 
			"
			INSERT INTO $attachments_table  
			( 
				message_id,
				attachment_number,
				attachment_filename,
				attachment_size,
				attachment_type,
				attachment_subtype,
				attachment,
				attachment_saved
			) VALUES
			( %d,%s,%s,%s,%s,%s,%s,%d )
			",
			array ( 
				$this->inbox_image_id,
				$partno,
				$file_name > '' ? $file_name : 'Bad or missing file name.',
				$p->bytes,
				strtolower( $attachment_types[$p->type] ),
				$p->ifsubtype ? sanitize_text_field( strtolower($p->subtype) ) : '',					
				( $this->max_packet_size - 1000 < $p->bytes || !$file_name ) ? '' : $data, // do not actually save item if no filename or if too big
				( $this->max_packet_size - 1000 < $p->bytes || !$file_name ) ? 0 : 1
			)
		);
		$wpdb->query ( $sql );
	}


	/*
	*
	*
	* Address extraction functions
	*
	*
	*
	*/
	public static function sanitized_text_to_block_array ( &$text ) {
		return preg_split ( '#[ \t\r]*[\f\v\n][ \t\r]*[\f\v\n]\s*#', $text, NULL, PREG_SPLIT_NO_EMPTY );
	}

	private function extract_address() { 
		
		// get variables
		$form_variables_object = unserialize ( get_option ( 'wp-issues-crm-email-processing-options' ) ); 
		$disqualifiers = $form_variables_object->disqualifiers;

		/*
		*	Process text body into blocks containing emails or physical address_line
		*	Block separation is by at least two vertical carriage movements interspersed with horizontal movements
		*		-- this captures both unix (\n) and windows (\r\n) line movements 
		*	Split body string into array of blocks using preg split 
		*		note that originally wrote this as capturing groups rather than character classes 
		*		(results in php Segmentation fault if too many white spaces).
		*	Use nonaddress blocks for md5 characterization of message
		*
		*/
   		$block_array = self::sanitized_text_to_block_array( $this->text_body );
		$address_block_object_array = array(); 
		$non_address_block_array = array();
		// now parse and narrow down text blocks for examination
		foreach ( $block_array as $block ) {
			// skip any blocks containing disqualifying phrases
			if ( $disqualifiers > '' ) {
				$disqualifier_pattern = '#(?i)\b(?:' . $disqualifiers . ')\b#u';
				if ( 1 == preg_match ( $disqualifier_pattern, $block ) ) {
					continue;
				}
			}
			// if not disqualified, parse block using the block object
			$block_object = new WIC_DB_Address_Block_Object ();
			$block_object_OK = $block_object->populate( $block );
			// if have email_address or physical address keep it in play
			if ( $block_object_OK ) {
				$address_block_object_array[] = $block_object;
			// otherwise save it for md5 processing
			} else {
				$non_address_block_array[] = $block;
			}
		}
		
		/*
		*  Now choose email_address.  Hierarchy is:
		*		1A) In body, != to email (on its own line in the first full address block) ( meets email_source 1, 2, 3 standards )
		*		1B) In body, != to email and user chose usebody soft ( meets email_source 2 and 3 standard )
		*		2) 	Reply_to email ( meets all standards, 1-4 )
		*		3)	From email (meets all standards, 1-4 )
		*
		*	Email line can be standalone, in a block without address.
		*	Email bears no necessary relationship to name or address.
		*	So, handle this choice first, separately from address choice.
		*/
		if ( 
				// auto -- take email from body, but only if at start of a line (enforced in block object) and only if not a reply or forward email
				(
					'1' == $form_variables_object->email_source && 
					're:' != strtolower( substr( $this->subject, 0, 3 ) ) &&
					'fw'  != strtolower( substr( $this->subject, 0, 2 ) ) 

				) ||
				// auto soft -- taking email from body if available 
				'2' == $form_variables_object->email_source || 
				// force body -- prefer body email, will not substitute reply_to or from below
				'3' == $form_variables_object->email_source 
				// if 4, then forcing use of reply or from
			) {
			foreach ( $address_block_object_array as $block_object ) {
				// skip blocks if email blank or equal to_email
				if ( 
					'' == $block_object->email ||
					$this->to_email ==	$block_object->email 
					) {
					// keep searching if auto soft or force body
					if ( '1' != $form_variables_object->email_source ) { 
						continue;
					// if doing auto, only accept email if it comes from the first ( not-previously-disqualified ) address block
					} else {
						break;
					}
				} else {
					// note that not setting $block_object_email unless it should be used an
					// not in this branch unless at all unless it should be used
					$this->email_address = $block_object->email;
					$this->outcome += 2; // body email is coded as two points in the outcome scale
					break; // always using first email, don't want to do it twice and double points
				}
			}
		}
		// if not found in body or force ignored, go to reply to, then from
		if ( '' == $this->email_address ) {
			if ( $this->reply_to_email > '' ) {
				$this->email_address = $this->reply_to_email;
			} else {
				$this->email_address = $this->from_email;			
			}
		}
		
		/*
		*	Now choose address block. Strategy is to exclude 
		*	the to block if possible. There will probably only be two full address blocks.
		*
		*	Exclusion is through application of disqualifying phrases (previously)
		*	Or here through comparison to the to_email or to_email name
		*	
		*	( A bulk emailer that included own address block with a name that matched from or reply_to
		*	Could subvert this logic, but this can be protected by disqualifying phrases. )
		*	
		*	If have a block that includes the selected email_address, then take it.
		*	Other wise, score the blocks and choose if have both soft fn and soft ln match to reply or from.
		*
		*	If still unresolved and still have candidates, accept best scoring remaining block 
		*	  (reply and from scores positive, to score negative)
		*
		*/
		$unresolved_address_block_object_array = array(); // for residual after this stage
		foreach ( $address_block_object_array as $block_object ) {
			
			// skip blocks that came through only because had email
			// note that if zip is required and doesn't have zip, then block already gone
			if ( '' == $block_object->city ) {
				continue;
			}
			
			/*
			* also skip blocks that look like they are to blocks based on email
			* user should have set up disqualifiers already, but help them
			* down side is might exclude a good address block from person with 
			* same name as to person
			* address block includes email and it matches the to email OR
			* first and last from address block match the to email
			*/
			if ( $block_object->email == $this->to_email ||
				// email or names in address block match to to_email
				( $block_object->first_name > '' && $block_object->last_name > '' &&
					1 == preg_match ( 
						'#(?i)^'  // case insensitive 
						 . $block_object->first_name 
						 . '[._%+-]?'
						 . $block_object->last_name
						 . '#u'
						 , $this->to_email
					) ) ) { 
				continue;
			}
			
			
			// best case: address block includes chosen email or matches it by name 
			if 	( $block_object->email == $this->email_address  ||
				( $block_object->first_name > '' && $block_object->last_name > '' &&
					1 == preg_match ( 
						'#(?i)^'  // case insensitive 
						 . $block_object->first_name 
						 . '[._%+-]?'
						 . $block_object->last_name
						 . '#u'
						 , $this->email_address
					) ) ) { 
				$this->use_this_block ( $block_object ); 
				break;
			}
			/*
			* If no clear choice based on block_object's email or chosen email,
			* compare names in block to available names in email header.
			* With proper settings of disqualifiers, this is belt and suspenders.
			*
			* > 15 means first and last names match in first 4 or better
			*/	

			$name_scores = $this->score_name_match( $block_object );
			if (  $name_scores['reply_to'] > 15 && '3' != $form_variables_object->email_source ) {
				$this->email_address = $this->reply_to_email;
				$this->use_this_block ( $block_object );
				break;
			} elseif ( $name_scores['from'] > 15 && '3' != $form_variables_object->email_source ) {
				$this->email_address = $this->from_email;
				$this->use_this_block ( $block_object );
				break;
			} elseif ( $name_scores['to'] > 15 ) {
				continue;
			} 
				 
			// still no resolution 				
			$unresolved_address_block_object_array[] = array(
				'block' 	=> $block_object,
				'score' 	=> $name_scores['reply_to'] + $name_scores['from'] - $name_scores['to']
			);
		} // close address_block review loop
	
		/*
		*	If still haven't chosen an address block, but have some not excluded still,
		*		take the best one based on score -- if no name, could be zero.
		*	If reached this stage, user having set disqualifiers will assure better results.
		*
		*/
		if ( '' == $this->city && count ( $unresolved_address_block_object_array ) > 0 ) { 
			$max_score = -100;
			$max_scoring_block = -1;
			foreach ( $unresolved_address_block_object_array as $i => $block_score_array ) {
				if ( $block_score_array['score'] > $max_score ) {
					$max_score = $block_score_array['score'];
					$max_scoring_block = $i;
				}
			}
			$this->use_this_block ( $unresolved_address_block_object_array[$max_scoring_block]['block'] );
		}

		/*
		*	Finally, attempt to complete name if still missing.
		*
		*	May have taken email_address from one level (body/reply_to/from), but matched
		*		address block name at another level.  This is not necessarily wrong.  
		*		Could have name in from or reply_to with an "on behalf of", with true
		*		email in body.
		*
		*	But may not have set name yet, so look at options -- nothing left in address_blocks.
		*		Try parse reply to and from (may require adding more phrases like 'on behalf of'
		*			to pre_title setting to be successful).
		*		Insist that name be found in body text and fail ( leave name blank if not )
		*
		*	This step structured to provide information about quality of match, even if name found already
		*/

		// try parse the reply address
		$name_array = WIC_DB_Address_Block_Object::parse_name( $this->reply_to_personal );
		// if fail, try parse the from address
		if ( false === $name_array ) {
			$name_array = WIC_DB_Address_Block_Object::parse_name( $this->from_personal );
		}

		// if have success on one or the other, test it vs. the body text 
		// there better be a text block that contains it; if so, use it.
		if ( false !== $name_array ) {
			$name_test_pattern = '#(?i)' 
				. '\b' . $name_array['first'] . '\b' 
				. '[\. ]{1,3}'
				. '(?:\b' . $name_array['middle'] . '[\. ]{1,3})?' 
				. '\b' . $name_array['last'] . '\b'
				. '#u';
			if ( 1 == preg_match ( $name_test_pattern, $this->text_body  ) ){
				$this->outcome++; // add one to the outcome code for a successful header name check
				if ( '' == $this->first_name && '' == $this->last_name ) {
					$this->first_name = $name_array['first'];
					$this->last_name = $name_array['last'];
					$this->middle_name = $name_array['middle'];
				} // missing name	
			} // match?
		} // have name_array?
		
		/* 
		* done using address blocks, now use the non-address blocks to create the md5 representation of the non-address text
		*/
		$this->sentence_md5_array = WIC_Entity_Email_MD5::get_md5_array_from_block_array (  $non_address_block_array );
	
	} // close function extract_address
	
	private function use_this_block ( $address_block_object ) {
		$this->phone_number		= $address_block_object->phone; 
		$this->first_name		= $address_block_object->first_name;	
		$this->middle_name		= $address_block_object->middle_name;
		$this->last_name		= $address_block_object->last_name;			
		$this->apartment_only_line 		= $address_block_object->apartment_line;
		$this->street_address_only_line = $address_block_object->street_address_line;
		if ( $address_block_object->street_address_line > '' ) {
			$this->address_line 	= $address_block_object->street_address_line .
									(  
										( $address_block_object->apartment_line > '' ) ?  
										( ', ' . $address_block_object->apartment_line ) : ''
									);
		}
		$this->city				= $address_block_object->city;
		$this->state 			= $address_block_object->state;
		$this->zip 				= $address_block_object->zip;
		// do scoring
		if ( $this->city > '' ) {
			$this->outcome +=4;
			if ( $this->last_name > '' ) { // if single name is treated as last; never have name w/o address
				$this->outcome +=4;
			}
		}	
	}
	
	private function score_name_match ( $block_object ) {
		// array of name type_strings
		$test_name_types = array ( 'from', 'to', 'reply_to' );
		$name_score_array = array(); // will be 'type'=>score
		$block_object_name_array = array( 
			'first' 	=> $block_object->first_name, 
			'middle' 	=> $block_object->middle_name, 
			'last' 		=> $block_object->last_name 
		);
		foreach ( $test_name_types as $name_type ) {
			$name_array = WIC_DB_Address_Block_Object::parse_name( $this->{ $name_type . '_personal' } );
			$name_score_array[$name_type] = $this->compare_arrays_of_names ( $name_array, $block_object_name_array ); 
		}
		return  ( $name_score_array );
	}
	
	private function compare_arrays_of_names ( $name_array_1, $name_array_2 ) {
		if ( false === $name_array_1 ) {
			return ( 0 ); // handling bad parse return 
		}
		/*
		* no negative if name component missing in one or both names
		* middle mismatch does not subtract but full disagreement on either fn or ln will
		* disqualify if set minimum match score to 16.
		*/
		$name_component_array = array('first', 'middle', 'last' );
		$name_score = 0;
		foreach ( $name_component_array as $name_component ) {
			if ( $name_array_1[$name_component] > '' && $name_array_2[$name_component] > '' ) {
				if ( $name_array_1[$name_component] == $name_array_2[$name_component] ) {
					$name_score += 10;
				} elseif ( substr( $name_array_1[$name_component], 0, 4 ) == substr ( $name_array_2[$name_component], 0, 4) ) {
					$name_score += 8;
				} elseif ( $name_component != 'middle' ) {
					$name_score -= 5;
				}
			}
		}
		return ( $name_score );
	}

	/*
	*
	* sanitization functions
	*
	*/
	// translate RFC2822 header sent time to blog local MYSQL format date time
	private static function sanitize_date_time ( $possible_date_time ) { 
		// first discard possible string literal describing time zone in otherwise RFC2822 form -- Wed, 28 Jun 2017 08:43:21 +0000 (UTC)  . . . drop the `(UTC)`
		$possible_date_time = implode( ' ', array_slice( explode ( ' ', $possible_date_time ), 0 , 6 ) );
		// second -- get time_zone string for blog if available in either format, otherwise default to GMT
		if ( ! $time_zone_string = get_option ( 'timezone_string' ) ) {
			$time_zone_offset = get_option ( 'gmt_offset' );
			if ( is_numeric( $time_zone_offset ) ) {
				$time_zone_string = sprintf( "%+'.05d\n", 100 *  intval( $time_zone_offset ) + 60 * ( $time_zone_offset - intval( $time_zone_offset ) ) ); 
			} else {
				$time_zone_string = "+0000";
			}
		}
		// third attempt php timezone object from string
		try {
			$tz =  new DateTimeZone( $time_zone_string );
		} catch ( Exception $e ) {
			$tz = new DateTimeZone ( "+0000" ); // had option setting and used it, but it was bad, default again to GMT
		}
		// now try to parse as RFC2822 which includes the sender's time zone -- first normalize to GMT, then adjust to blog local
		if ( $date = DateTime::createFromFormat ( DateTime::RFC2822, $possible_date_time ) ) { 
			if( $date->setTimezone( $tz ) ) {
				return $date->format('Y-m-d H:i:s');
			} 
		}
		// if failed as RFC2822, try without assuming proper RFC2822 formatting to catch bad most bad dates	
		try {
			$date = new DateTime( $possible_date_time );
			$date->setTimezone( $tz );
	 		return $date->format('Y-m-d H:i:s');
		}	catch ( Exception $e ) {
			return ( '' );
		}	   			
	}


	public static function sanitize_incoming ( $raw_incoming ) {
		/*
		* WIC_Entity_Email_Subject::sanitize_incoming
		*
		* (1) Remove transfer encoding and convert to UTF8
		* (2) Strip any 4 byte (high plane) UTF-8 characters like emoticons 
		*		-- these will not store properly in MySQL ( unless upgraded https://mathiasbynens.be/notes/mysql-utf8mb4 )
		*		-- replace with characters to avoid creating unsafe strings
		*			http://stackoverflow.com/questions/8491431/how-to-replace-remove-4-byte-characters-from-a-utf-8-string-in-php
		*			http://unicode.org/reports/tr36/#Deletion_of_Noncharacters
		* (3) Do a sanitize text field to strip any tags
		*
		* NOTE: function imap_utf8 does NOT work as advertized -- iconv_mime_decode is much more reliable		
		*/
		$sanitized = sanitize_text_field ( 
			preg_replace(
				'/[\x{10000}-\x{10FFFF}]/u', "\xEF\xBF\xBD", 
				iconv_mime_decode ( 
					$raw_incoming, 
					ICONV_MIME_DECODE_CONTINUE_ON_ERROR, 
					'UTF-8' 
				) 
			) 
		);
		return $sanitized;
	}

	
	private function email_array_to_display_safe_address ( $mail_object_array ) {
		$mailbox = isset( $mail_object_array[0]->mailbox ) 	? $mail_object_array[0]->mailbox : '';
		$host 	 = isset( $mail_object_array[0]->host ) 	? $mail_object_array[0]->host  	 : '';
		$test_email = sanitize_text_field( $mailbox . '@' . $host ); // sanitize to avoid any unsafe that might pass filter https://tools.ietf.org/html/rfc2822#section-3.4
		return filter_var( $test_email, FILTER_VALIDATE_EMAIL ) ? $test_email : '';
	}

	private function domain_only ( $email_address ) {
		$from_array = preg_split( '#@#', $email_address );
		return isset ( $from_array[1] ) ? $from_array[1] : '';
	}
	
	private function email_array_to_display_safe_personal ( $mail_object_array ) {
		return isset ( $mail_object_array[0]->personal ) ? self::sanitize_incoming( $mail_object_array[0]->personal ) : '';
	}

	// in lieu of a class constant array, not available prior to php 5.6
	public static function break_patterns_array() {
		return array ( 
			'break_patterns' => array (
				'#(?i)<(\s)*/?(\s)*(div|p|h\d)(\b)?[^>]*>#', // visual paragraph boundaries -- div, p, h tags (open or close )
				'#(?i)<(\s)*/?(\s)*br(\s)*/?(\s)*>#', // visual line break patterns -- br tag
			),
			'replacements' => array( 
				"\n\n", 
				"\n", // repeated line breaks will be treated as a paragraph break
			)
		);
	}
	
	// hard strip, but preserves paragraph breaks
	public static function utf8_to_sanitized_text ( $string, $force_ascii = false ) {
	
		$break_patterns_array = self::break_patterns_array();
	
		// character filter -- see http://php.net/manual/en/regexp.reference.unicode.php
		// include all unicode letters, digits, email characters, plus slash and white spaces, NO COMMAS OR PARENS OR TAG CHARACTERS
		// if $force_ascii is set, use only ASCII letters, punctuation and unicode whitespace
		$character_filter = $force_ascii ? '#[^A-Za-z0-9.@_%+/\-\n\r\s]#u' : '#[^\pL0-9 .@_%+/\-\n\r]#u'; 
		$new_line_filter  = '#\r(\n)?#';
		
		$clean_string = preg_replace ( $new_line_filter, "\n",					 // replace CR or CRLF with LF
			preg_replace ( $character_filter, '',			 // lose inessential chars
				html_entity_decode (											 // decode entities
					strip_tags( 												 // get rid of all other tags
						preg_replace( $break_patterns_array['break_patterns'], $break_patterns_array['replacements'], // put in CRLF for boundary tags
							$string 
						)
					),
					0, // no flags
					'UTF-8'
				)
			)
		);
		return ( $clean_string );
	}

	/* 
	* want to use wp_kses_post to limit tags to safe presentation material
	* wp_kses_post will strip style or head tags, but will leave enclosed material
	* elect to just strip head and styles entirely before running kses_post
	*/
	public static function strip_html_head( $html ) {
		// the question mark in the capturing group makes it lazy -- stop after first hit what comes after it
		// http://www.regular-expressions.info/examples.html
		$patterns_to_strip = array( 
			'#(?i)<(head|style|script)\b[^>]*>(?:[\s\S]*?)</\1\s*>#', // strip head, style and script tag (AND, unlike KSES, the content between them)
			'#(?i)<!--(?:[\s\S]*?)-->#', // strip comments, including mso conditional comments
		);
		return 	balanceTags ( preg_replace (  $patterns_to_strip, '', $html ), true );
	}



	
	// make an array of address that is safe and always an array from unreliable incoming address information
	private function repack_address_array ( $address_array ) { 
		$clean_array = array();
		if ( !$address_array || !is_array( $address_array ) ) {
			return $clean_array;
		} else  {
			foreach ( $address_array as $address ) {
				$test_email = isset ( $address->mailbox ) && isset( $address->host ) ? self::sanitize_incoming ( $address->mailbox . '@' . $address->host ) : '';
				$clean_array[] = array( 
					isset ( $address->personal ) ? self::sanitize_incoming( $address->personal ) : '', 
					filter_var( $test_email, FILTER_VALIDATE_EMAIL ) ? $test_email : ''
				);
			}
		}
		return ( $clean_array );		
	}
	
} // class 