<?php
/*
*
*	wic-entity-activity.php
*
*/



class WIC_Entity_Activity extends WIC_Entity_Parent {

	protected function set_entity_parms( $args ) {
		$this->entity = 'activity';
	} 

	// supports creation of this entity without taking an action in construct
	protected function nothing(){}

	// remove any non numeric characters from amount -- commas, currency most likely in csv upload (done by js online)
	public static function activity_amount_sanitizor ( $raw_amount ) { 
		return preg_replace("/[^0-9.]/", '', $raw_amount );
	}

	public static function set_up_activity_area( $parent_entity_id, $parameters_object ) {

		$init = $parameters_object->initialLoad;
		$parent = $parameters_object->parentForm;

		global $wpdb;
		
		/* prepare list before buttons because want to see found count */
		$activity_list = '<p id="wic_no_activities_message"></p>
						  <ul id="wic_activity_list">';
		
		$found_count = 0;
		$count_activities = 0;	
		if ( $parent_entity_id > 0 ) {

			$activity_table 	= $wpdb->prefix . 'wic_activity';
			$constituent_table 	= $wpdb->prefix . 'wic_constituent';
			$post_table 		= $wpdb->posts;
			
			$where_predicate = 'constituent' == $parent ? 'a.constituent_id' : 'a.issue';
			$limit_phrase = $init ? "LIMIT 0, 10" : ''; // limit only on init -- init false means user requested show all
			
			// query normally works for either parent (issue or post) 
			// LEFT JOIN to post to cover case where issue deleted	
			$sql = $wpdb->prepare(
					"SELECT SQL_CALC_FOUND_ROWS a.ID, a.constituent_id, a.activity_type, a.activity_date, a.activity_amount, a.issue, a.pro_con, a.activity_note, 
						IF( p.ID IS NULL, 
							CONCAT('Hard deleted issue ( ID was ', a.issue,' )' ), 
							CONCAT( IF( post_status != 'publish' AND post_status != 'private', 'Trashed or unpublished issue: ', ''), post_title )
						) as post_title, 
						c.first_name, c.last_name
					FROM $activity_table a
					INNER JOIN $constituent_table c on c.ID = a.constituent_id
					LEFT JOIN $post_table p on p.ID = a.issue
					WHERE $where_predicate = %d 
					ORDER BY activity_date DESC
					$limit_phrase",
					array ( $parent_entity_id )
				);		
			$activities = $wpdb->get_results ( $sql );
			$count_activities = count( $activities) ; 

			$sql_found = "SELECT FOUND_ROWS() as found_count";
			$found_count_object_array = $wpdb->get_results( $sql_found );
			$found_count = $found_count_object_array[0]->found_count; 
					
			if ( $count_activities > 0 ) {
				foreach ( $activities as $activity ) {
					$activity_list .= self::format_activity_list_item ( $activity );
				}
			}

		}
		
		$activity_list .= '</ul>';

		if ( $found_count > $count_activities ) { 
			$activity_list .= '<h4 id="show_all_activities"> . . . load all activities ( ' . ( $found_count - $count_activities ) . ' more ) &raquo;</h4>';
		}		
		
		
		/* prepare buttons */
		$buttons = '';
		
		$button_args_main = array(
			'button_label'				=> '<span class="dashicons dashicons-plus-alt"></span>',
			'type'						=> 'button',
			'id'						=> 'add-new-activity-button',			
			'name'						=> 'add-new-activity-button'			
		);	
		
		$buttons .= WIC_Form_Parent::create_wic_form_button ( $button_args_main );

		$button_args_main = array(
			'button_label'				=> '<span class="dashicons dashicons-download"></span>',
			'type'						=> 'button',
			'title'						=> 'Download activities (all for this ' . $parent . ').',
			'name'						=> 'download-activities-form-button',			
			'id'						=> 'download-activities-form-button',			
			'button_class'				=> 'wic-form-button wic-download-button',
			'value'						=> 'activity,activity,' . $parent . ',' . $parent_entity_id,
		);	
		
		$buttons .= WIC_Form_Parent::create_wic_form_button ( $button_args_main );
		
		if ( 'issue' == $parent && WIC_List_Parent::is_email_send_enabled() ) {
			$buttons .= WIC_List_Parent::make_send_email_to_found_button ( 'issue_activity_list_email_send', $parent_entity_id, $found_count );
		}

		$buttons .= self::make_activity_type_filter_button();	
		
		ob_start();
		new WIC_Entity_Activity ('new_blank_form', array() ) ;
		$hidden_form = ob_get_clean();
		
		return array ( 'response_code' => true, 'output' => (object) array ( 'activityList' => $buttons . $activity_list, 'activityForm' => $hidden_form ) );	
	
	}

	// takes object with properties having field names; prepares sanitized output for list display and possible use in update form
	protected static function format_activity_list_item ( $activity ) {
	
		global $wic_db_dictionary;

		$activity_type_array 	= $wic_db_dictionary->lookup_option_values( 'activity_type_options' );
		$pro_con_array			= $wic_db_dictionary->lookup_option_values( 'pro_con_options' );
		$financial_types_array 	= self::get_financial_types_array();

		// carry lable through from form to list item if coming that way
		if ( isset ( $activity->constituent_id_label ) ) { 
			$name_show = $activity->constituent_id_label; 
		} else {
			$name_show =  $activity->first_name . ' ' . $activity->last_name ;
		}
		$name_show = ( '' == trim ( $name_show ) ? 'Name unknown' : trim ( $name_show ) );
		$sanitized_note = sanitize_text_field ( $activity->activity_note ); // strip tags, etc.
		// especially for comments and incoming email, $note may be utf-8 -- can't just use substr
		$shortened_note = WIC_Entity_Email_Process::first_n1_chars_or_first_n2_words ( $sanitized_note, 50, 8 );

		return 	'<li>' .
			( 'wic_reserved_88888888' == $activity->activity_type ? '<span class="dashicons dashicons-admin-comments" title = "' . esc_attr( $sanitized_note ) . '"></span>' : '<span class="dashicons dashicons-edit"></span>' ) .
			'<span class="activity_list_ID">' . $activity->ID . '</span>' .
			'<span class="activity_list_activity_date">' . esc_attr ( $activity->activity_date ) . '</span>, ' .
			'<span class="activity_list_activity_type">' . esc_attr ( $activity->activity_type ). '</span>' .
			'<span class="activity_list_activity_type_show">' . ( $activity->activity_type > '' ? esc_html( WIC_Function_Utilities::value_label_lookup ( $activity->activity_type, $activity_type_array ) ) : '' )  . '</span>' . ( $activity->activity_type > '' ? ', '  : '' ) .
			'<span class="activity_list_activity_amount">' . esc_attr( $activity->activity_amount ) .'</span>' .
			'<span class="activity_list_activity_amount_show">' . ( in_array( $activity->activity_type, $financial_types_array ) ? esc_html( $activity->activity_amount ) . ', ' : '' ) . '</span> ' .
			'<span class="activity_list_issue">' . esc_attr ( $activity->issue ) . '</span>' .
			'<span class="activity_list_issue_show"><a class = "activity_list_issue_show_link" target = "_blank" href = "' . admin_url() . '?page=wp-issues-crm-main&entity=issue&action=id_search&id_requested=' . esc_attr( $activity->issue ). '">' . esc_html( $activity->post_title ) .'</a></span>' .
			'<span class="activity_list_constituent_id">' . $activity->constituent_id . '</span>' .
			'<span class="activity_list_constituent_show"><a class="activity_list_constituent_show_link" target = "_blank" href = "' . admin_url() . '?page=wp-issues-crm-main&entity=constituent&action=id_search&id_requested=' . $activity->constituent_id . '">' . esc_html(  $name_show ) .'</a></span>' .
			'<span class="activity_list_pro_con">' . esc_attr( $activity->pro_con ) . '</span>' .
			'<span class="activity_list_pro_con_show">' . ( $activity->pro_con > '' ?  ( ' (' . esc_html ( WIC_Function_Utilities::value_label_lookup ( $activity->pro_con, $pro_con_array ) ) . ') ' ) : '' ) . '</span> ' .
			'<span class="activity_list_activity_note">' . esc_textarea ( $activity->activity_note ) . '</span> ' .
			( $shortened_note > '' ? 
				( 
					'<span class="activity_list_activity_note_show">  -- note: ' . 
						esc_html ( $shortened_note ) .  // can have multi-byte characters in comments, use multibyte safe substring
					'</span> ' 
				) :
				'' 
			) . 
		'</li>';
	}



	public static function popup_save_update ( $dummy_id, $form_data ) {
		$activity_entity = new WIC_Entity_Activity ( 'nothing', array() );
		return $activity_entity->form_save_update_message_only ( $form_data );	
	}	
	
	// differs from parent save/update in that does not send back form, only message -- nothing to refresh except ID and notes could change in sanitization
	protected function form_save_update_message_only ( $form_data ) {

		// populate the array from the submitted form
		$this->initialize_data_object_array();

		foreach ( $this->fields as $field ) {  	
			if ( isset ( $form_data->{$field->field_slug} ) ) {		
				$this->data_object_array[$field->field_slug]->set_value( $form_data->{$field->field_slug} );
			}	
		} 

		$save = ( 0 == $this->data_object_array['ID']->get_value() );

		// return with no changes message if nothing to do
		if ( '0' === $this->data_object_array['is_changed']->get_value() ) {
			return array ( 
				'response_code' => true, 
				'output' => (object) array ( 
					'message' => __( 'No changes to save -- no action taken.', 'wp-issues-crm' ), 
					'message_level' => 'error'  
				) 
			);	
		}

		// sanitize, validate and check for completeness the array
		$this->sanitize_values();
		$this->validate_values();
		// don't clutter with additional messages if validation fails.
		if ( '' === $this->outcome ) {
			$this->required_check();
		}
		if ( false === $this->outcome ) {
			return array ( 	
				'response_code' => true, 
				'output' => (object) array ( 
					'message' => __( 'Activity save not successful: ', 'wp-issues-crm' ) . $this->explanation, 
					'message_level' => 'error'  
				) 
			);	
		}
		
		// form was good so do save/update
		$wic_access_object = WIC_DB_Access_Factory::make_a_db_access_object( $this->entity );
		$wic_access_object->save_update( $this->data_object_array );
		// handle failed save/update 
		if ( false === $wic_access_object->outcome ) {
			return array ( 	
				'response_code' => true, 
				'output' => (object) array ( 
					'message' => __( 'Activity save not successful: ', 'wp-issues-crm' ) . $wic_access_object->explanation, 
					'message_level' => 'error'  
				) 
			);
		// proceed on successful save/update
		} else {
			// retrieve the new activity ID from the save process
			if ( $save ) { 
				$this->data_object_array['ID']->set_value( $wic_access_object->insert_id );
				$form_data->ID = $wic_access_object->insert_id;
			}
			// populate form data with sanitized values
			foreach ( $this->data_object_array as $field_slug => $control ) {
				if ( isset ( $form_data->{$field_slug} ) ) {
					$form_data->{$field_slug} = $control->get_value();				
				}
			}
	
			// use label for form list title
			$form_data->post_title = $form_data->issue_label;
	
			return array ( 	
				'response_code' => true, 
				'output' => (object) array ( 
					'message' 		=> __( 'Activity save/update successful. ', 'wp-issues-crm' ),  
					'message_level' => 'OK', 
					'activity_id'	=>	$form_data->ID,
					'list_item'		=>	self::format_activity_list_item( $form_data ),
				) 
			);					
		}
	}


	public static function popup_delete ( $id, $dummy ) {
		$wic_access_object = WIC_DB_Access_Factory::make_a_db_access_object( 'activity' );
		$wic_access_object->delete_by_id ( $id );
		// note that a database error will generate a non-JSON error message which will intern trigger a "server error" message.
		return array ( 'response_code' => true, 'output' => '' );	
	}


/**************************************************
*
*  Activity utility functions and object properties
*
***************************************************/
	
	// freeze date setting -- passed to js
	public static function get_freeze_date() {
		$wic_option_array = get_option('wp_issues_crm_plugin_options_array'); 
		if ( isset ( $wic_option_array['freeze_older_activities'] ) ) {
			$date_value = $wic_option_array['freeze_older_activities'] > '' ? 
				WIC_Control_Date::sanitize_date( $wic_option_array['freeze_older_activities'] ) :
				'';
		} else {
			$date_value = '';		
		}
		return ( $date_value );
	}

	// get possibly empty financial types array
	public static function get_financial_types_array() {
		$wic_option_array 		= get_option( 'wp_issues_crm_plugin_options_array' ); 
		return explode (',' , isset ( $wic_option_array['financial_activity_types'] ) ? $wic_option_array['financial_activity_types'] : '') ;
	}
	
	// basic option retrieval function for activity issue drop down
	public static function get_issue_options( $value ) {

		// top element in options array is always the unclassified issue -- allow one touch to unclassified
		$issues_array = array( 
			self::get_unclassified_post_array(),
		);	

		// variable tested to see if need to add an already saved activity's issue to the array
		$value_in_option_list = false;	

		// 3.4.1+ limits retrieval to open issues
		$allowed_issues = WIC_DB_Access_WP::get_wic_live_issues();
		// assemble array		
		foreach ( $allowed_issues as $allowed_issue ) {
			if (  $allowed_issue->ID != $issues_array[0]['value'] ) { // don't dup not classified value
				$issues_array[] = array(
					'value'	=> $allowed_issue->ID,
					'label'	=>	esc_html ( $allowed_issue->post_title ),
				);
				if ( $value == $allowed_issue->ID ) {
					$value_in_option_list = true;
				}
			}
		}
		
		// add current value if missing
		if ( ! $value_in_option_list && $value > '' ) {
			$issues_array[] = array (
				'value'	=> $value,			
				'label'	=> WIC_DB_Access_WP::format_title_by_id( $value ),
			);
		}		

		return ( $issues_array );
	}


	// generate upload id options -- for advanced searches on activities
	public static function upload_id( $value ) {
		
		// base value is a non-uploaded activity
		$uploads_array = array( 
			array ( 'value' => 0 , 'label' => __( 'Not Uploaded', 'wp-issues-crm' ) ),
		);			
		// get upload list

		global $wpdb;
		$table = $wpdb->prefix . 'wic_upload';
		$sql = "SELECT ID, upload_file, upload_time from $table ORDER BY upload_time DESC";
		$uploads = $wpdb->get_results( $sql );

		// assume value not found
		$value_found = false;
		// if got a list of uploads, populate the option list
		if ( is_array ( $uploads ) ) {
			foreach ( $uploads as $upload ) {
				$uploads_array[] = array ( 'value' => $upload->ID , 'label' => $upload->upload_file . ' (' . $upload->upload_time . ') ' );		
				if ( $upload->ID == $value ) {
					$value_found = true;
				}
			}
		}
		if ( false === $value_found && $value > 0 ) {
			$uploads_array[] = array ( 'value' => $value , 'label' => __( 'Upload history purged.', 'wp-issues-crm' ) );	
		
		}
	    return ( $uploads_array );
	}

	public static function make_activity_type_filter_button() {
		$button_args_main = array(
			'button_label'				=> '<span class="dashicons dashicons-filter"></span>',
			'type'						=> 'button',
			'title'						=> 'Show/hide activity type filter.',
			'id'						=> 'filter-activities-button',			
			'name'						=> 'filter-activities-button'			
		);

		$output = WIC_Form_Parent::create_wic_form_button ( $button_args_main );

		$activity_type_control = WIC_Control_Factory::make_a_control( 'multiselect' );
		$activity_type_control->initialize_default_values(  'activity', 'activity_type', '' );
		$output .= '<div id="filter-activities-menu">' .
			$activity_type_control->form_control() 	
		.'</div>';
		return $output;
	}
	
	
	// check if any frozen financial activities for a constituent
	public static function has_frozen_financial_activities ( $id ) {
		global $wpdb;
		$activity_table = $wpdb->prefix . 'wic_activity';
		$freeze_date = self::get_freeze_date();
		$financial_phrase = '';
		$financial_types_array = self::get_financial_types_array();
		if ( count ( $financial_types_array ) > 0 ) {
			$financial_phrase = 'AND activity_type IN (';
			$did_first = false;
			foreach ( $financial_types_array as $financial_type ) {
				if ( ! $did_first ) {
					$did_first = true;
				} else {
					$financial_phrase .= ',';
				}
				$financial_phrase .= '%s';
			}
			$financial_phrase .= ')';
			$financial_phrase = $wpdb->prepare ( $financial_phrase, $financial_types_array );
		}
		$sql = "SELECT ID from $activity_table WHERE constituent_id = $id AND activity_date < '$freeze_date' $financial_phrase";
		$results = $wpdb->get_results ( $sql  );	
		return ( count ( $results ) > 0 );
	}

	// get the id/title array for the unclassified post, create it if necessary
	public static function get_unclassified_post_array () {
		$unclassified_post_title = 'Unclassified Activity (WP Issues CRM)';
		$unclassified_post_id = WIC_DB_Access_WP::fast_id_lookup_by_title ( $unclassified_post_title );
		$unclassified_post_content = // formatted for viewing in text editor
'It is safe to edit the content of this post, but altering the title creates minor problems as explained below.

The original title of this post is: ' . $unclassified_post_title . '

WP Issues CRM uses the post with that title as the default issue for unclassified issues and will recreate a post with the same title if not found.

If this post is trashed, issues assigned to the trashed post will show as linked to an invalid post.

Restoration from trash will solve the problem, but if you hard delete it, you will have some activities with a broken issue title.

If this post title is altered, issues previously assigned will show as classified to that title, but a new title will be created.  

This post was created ' . current_time( 'mysql' ) . '.
		';
		if ( !$unclassified_post_id ) {
			// set up save post object
			$wic_query = WIC_DB_Access_Factory::make_a_db_access_object( 'issue' );
			// create a template to spoof a form submission
			$save_array_template = array (
				array( 'key' 	=> 'ID', 
				 'value'	=> '', 
				 'wp_query_parameter' => 'p', 
				), 
				array( 'key' 	=> 'post_title', 
				 'value'	=> $unclassified_post_title, 
				 'wp_query_parameter' => 'post_title', 
				),
				array( 'key' 	=> 'post_content', 
				 'value'	=> $unclassified_post_content, 
				 'wp_query_parameter' => 'post_content', 
				), 
			); 
			// invoke query object save function directly 
			$wic_query->direct_save ( $save_array_template );
			$unclassified_post_id = $wic_query->insert_id;
		}
		return array ( 'value' => $unclassified_post_id, 'label' => $unclassified_post_title );
	}


} // class