<?php
/**
*
* class-wic-entity-dashboard.php
*/


class WIC_Entity_Dashboard extends WIC_Entity_Parent {
	
	/**************************************************************************************************
	*
	* Dashboard display and dashboard action functions	
	*
	***************************************************************************************************/	
	
	protected function set_entity_parms ( $args ) {}
	
	
	protected function dashboard () { 
		/*
		* at top of dashboard, show explanation about need to create live issues
		*
		*/
		// check how many times have shown the 3.5 upgrade notice
		$shown_already = get_option( 'wp_issues_crm_3_5_issues_change_alert');
		// check if have any live issues
		$live_issues = WIC_DB_Access_WP::have_wic_live_issues();
		// show notice if have no live issues, haven't shown warning, haven't shown warning three times
		if ( ! $live_issues || ! $shown_already || $shown_already < 3 ) {
			// update notice shown count
			if ( ! $shown_already ) {
				update_option ( 'wp_issues_crm_3_5_issues_change_alert', 1 );
			} else {
				if ( $shown_already < 3 ) {
					update_option ( 'wp_issues_crm_3_5_issues_change_alert', $shown_already + 1 );
				}
			}
			// explain
			echo '<p></p><div class="wic-dashboard-notice">
				<h3>In WP Issues CRM 3.4.1 and up, issues do not appear in the Issue drop down on the activity and email forms unless they are set to do so.</h3>
				<p>When creating or updating an issue, set the Activity Tracking rule to "Appear in issue dropdown."  In 3.4.1+, by default this rule is set when
				private issues are created in WP Issues CRM, but this was not so in earlier versions and some of your issues may not have this rule set.  The change in policy
				allows simplification of the settings and user preferences.</p>';
			// explain further if no live
			if ( ! $live_issues ) {
				echo '<h3>You currently have no issues that will appear in the issues drop down.</h3> <p>Perhaps you are just getting started. If not, you need to update the issues
				that you want to appear in the dropdown.  This message will disappear after you have set one issue to appear.</p>';
			// say when warning will disappear in case of some live issues
			} else {
				echo '<p>This message will appear on the first three visits to the dashboard after upgrade.</p>';
			}
			echo '</div>';
			
		}	
	
		// get sort order and wide display list
		$config = WIC_Entity_User::get_wic_user_preference ( 'wic_dashboard_config' );
		if ( $config ) {
			$sort_list = array_flip ( $config->sort ); // e.g., $sort_list['dashboard_activity'] = 0 . . .
			$wide_list = isset ( $config->wide ) ? $config->wide : array();
			$tall_list = isset ( $config->tall ) ? $config->tall : array();
		} else {
			$sort_list = array();
			$wide_list = array( 'dashboard_activity', 'dashboard_mycases', 'dashboard_activity_type' );
			$tall_list = array();
		}	
			
		// inventory of dashboard widgets and titles
		$dashboard_divs = array (
			'dashboard_activity_type'	=> 'Activities by Issue and Type',
			'dashboard_mycases'			=> 'My Cases',
			'dashboard_activity'		=> 'Constituents with Activity by Issue',
			'dashboard_myissues'		=> 'My Issues',
			'dashboard_recent' 			=> 'Recently Updated',
			'dashboard_searches' 		=> 'Search Log',
			'dashboard_uploads' 		=> 'Uploads', 
		//	'dashboard_email' 			=> 'Incoming Campaigns',
		);
		
		// sort the inventory
		$sorted_dashboard_divs = array();
		$unconfigged_div_counter = 100;
		foreach ( $dashboard_divs as $id => $title ) {
			if ( isset ( $sort_list[$id] ) ) {
				$new_index =  $sort_list[$id] ;
			} else {
				$new_index = $unconfigged_div_counter;
				$unconfigged_div_counter++;
			}
			$sorted_dashboard_divs[$new_index] = array( $id, $title );	
		} 
		ksort ( $sorted_dashboard_divs );
		echo '<ul id="dashboard-sortables" >';
		foreach ( $sorted_dashboard_divs as $key => $dashboard_div ) {
			$wide_class = in_array( $dashboard_div[0], $wide_list ) ? ' wic-dashboard-full ' : ' wic-dashboard-quarter ' ; 
			$tall_class = in_array( $dashboard_div[0], $tall_list ) ? ' wic-dashboard-tall ' : '' ; 
			echo 
				'<li id = "' . $dashboard_div[0] . '" class = "ui-state-default wic-dashboard ' . $wide_class . $tall_class . '">' .
					'<div class="wic-dashboard-title wic-dashboard-drag-handle" title="Drag to reorder dashboard widgets"><span class="dashicons dashicons-move"></span>' . $dashboard_div[1] . '</div>' .
					$this->special_buttons ( $dashboard_div[0] ) .
					'<button class="wic-dashboard-title wic-dashboard-refresh-button" type="button" title="Refresh"><span class="dashicons dashicons-update"></span></button>' .
					'<button class="wic-dashboard-title wic-dashboard-grow-button" type="button" title="Change Height"><span class="dashicons dashicons-sort"></span></button>' .
					'<button class="wic-dashboard-title wic-dashboard-expand-button" type="button" title="Change Width"><span class="dashicons dashicons-leftright"></span></button>' .
					'<div class = "wic-inner-dashboard" id="wic-inner-dashboard-' . $dashboard_div[0] . '">' . 
						'<img src="' . plugins_url(  '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'img' . DIRECTORY_SEPARATOR . 'ajax-loader.gif' , __FILE__ ) . '">' .
					'</div>' . 
				'</li>' 
			;			
		}
		echo '</ul>';
	}

	public static function dashboard_email ( $dummy_id, $dummy_data ) {
		return array ( 'response_code' => true, 'output' => '<div class="dashboard-not-found">Email Stuff</div>' );	
	}
	
	public static function save_dashboard_preferences ( $dummy_id, $data ) {
		return WIC_Entity_User::set_wic_user_preference ( 'wic_dashboard_config', $data );
	}
	// display a list of cases assigned to user
	public static function dashboard_mycases( ) {
		
		$user_ID = get_current_user_id();	
		
		$wic_query = WIC_DB_Access_Factory::make_a_db_access_object( 'constituent' );

		$search_parameters= array(
			'sort_order' => true,
			'compute_total' => false,
			'retrieve_limit' 	=> 99999999,
			'select_mode'		=> 'id',
		);

		$search_array = array (
			array (
				 'table'	=> 'constituent',
				 'key'	=> 'case_assigned',
				 'value'	=>  $user_ID, 
				 'compare'	=> '=', 
				 'wp_query_parameter' => '',
			),
			array (
				 'table'	=> 'constituent',
				 'key'	=> 'case_status',
				 'value'	=> '0', 
				 'compare'	=> '!=', 
				 'wp_query_parameter' => '',
			), 
		);

		$wic_query->search ( $search_array, $search_parameters ); // get a list of id's meeting search criteria
		$sql = $wic_query->sql;
		if ( 0 == $wic_query->found_count ) {
			return array ( 'response_code' => 'true', 'output' => '<div class="dashboard-not-found">' . __( 'No cases assigned.', 'wp-issues-crm' ) . '</div>' );		
		} else {
			$lister_class = 'WIC_List_Constituent' ;
			$lister = new $lister_class;
			$list = $lister->format_entity_list( $wic_query, __( 'My Cases: ', 'wp-issues-crm' ) );
			return array ( 'response_code' => true, 'output' => $list);			
		}
	}
		
	// display a list of issues assigned to user	
	public static function dashboard_myissues() { 
		
		$user_ID = get_current_user_id();	
		
		$wic_query = WIC_DB_Access_Factory::make_a_db_access_object( 'issue' );

		$search_parameters= array(
			'sort_order' => true,
			'compute_total' => false,
			'retrieve_limit' 	=> 99999999,
			'select_mode'		=> 'id',
		);

		$search_array = array (
			array (
				 'table'	=> 'issue',
				 'key'	=> 'issue_staff',
				 'value'	=> $user_ID,
				 'compare'	=> '=', 
				 'wp_query_parameter' => '',
			),
			array (
				 'table'	=> 'issue',
				 'key'	=> 'follow_up_status',
				 'value'	=> 'open', 
				 'compare'	=> '=', 
				 'wp_query_parameter' => '',
			), 
		);

		$wic_query->search ( $search_array, $search_parameters ); // get a list of id's meeting search criteria
		$sql = $wic_query->sql;
		if ( 0 == $wic_query->found_count ) {
			return array ( 'response_code' => 'true', 'output' => '<div class="dashboard-not-found">' . __( 'No issues assigned.', 'wp-issues-crm' ) . '</div>' );		
		} else {
			$lister_class = 'WIC_List_Issue' ;
			$lister = new $lister_class;
			$list = $lister->format_entity_list( $wic_query,  __( 'My Issues: ', 'wp-issues-crm' ) );
			return array ( 'response_code' => true, 'output' => $list);					
		} 
	}

	public static function dashboard_recent() {
	
		$user_ID = get_current_user_id();	
		
		// spoofing a query object and pass necessary values only
		$wic_query = (object ) array ( 'result' => '', 'entity' => 'constituent', 'found_count' => '', 'retrieve_limit' => 20 );
		global $wpdb;
		$constituent_table = $wpdb->prefix . 'wic_constituent';
		$sql = "SELECT ID from $constituent_table WHERE last_updated_by = $user_ID ORDER BY last_updated_time DESC LIMIT 0, 20";
		$wic_query->result = $wpdb->get_results( $sql );
		$wic_query->found_count = count ( $wic_query->result );
		if ( 0 == $wic_query->found_count ) {
			return array ( 'response_code' => 'true', 'output' => '<div class="dashboard-not-found">' . __( 'No constituents updated.', 'wp-issues-crm' ) . '</div>' );		
		} else {
			$lister_class = 'WIC_List_Constituent' ;
			$lister = new $lister_class;
			$list = $lister->format_entity_list( $wic_query,'' );
			return array ( 'response_code' => true, 'output' => $list);			
		}
	
	}

	// display user's search log ( which includes form searches, items selected from lists and also items saved )
	public static function dashboard_searches() {
	
		$wic_query = WIC_DB_Access_Factory::make_a_db_access_object( 'search_log' );
		$wic_query->retrieve_search_log_latest();
		$sql = $wic_query->sql;
		if ( 0 == $wic_query->found_count ) {
			return array ( 'response_code' => 'true', 'output' =>  '<div class="dashboard-not-found">' . __( 'Search log new or purged.', 'wp-issues-crm' ) . '</div>' );		
		} else {
			$lister_class = 'WIC_List_Search_Log' ;
			$lister = new $lister_class;
			$list = $lister->format_entity_list( $wic_query, '' );
			return array ( 'response_code' => true, 'output' => $list);				
		}
	}

	public static function dashboard_uploads () {
		// table entry in the access factory will make this a standard WIC DB object
		$wic_query = 	WIC_DB_Access_Factory::make_a_db_access_object( 'upload' );
		// select uploads that are beyond copied stage
		$wic_query->search (  
				array (
					array (
						'table'	=> 'upload',
						'key'	=> 'upload_status',
						'value'	=> 'copied',
						'compare'	=> '!=', 
						'wp_query_parameter' => '',
					),
				),	
				array( 'retrieve_limit' => 9999, 'sort_order' => true ) 
			);
		if ( 0 == $wic_query->found_count ) {
			return array ( 'response_code' => 'true', 'output' =>  '<div class="dashboard-not-found">' . __( 'Upload log new or purged.', 'wp-issues-crm' ) . '</div>' );		
		} else {
			$lister_class = 'WIC_List_Upload' ;
			$lister = new $lister_class;
			$list = $lister->format_entity_list( $wic_query, '' ); 
			return array ( 'response_code' => true, 'output' => $list);	
		}
	}
	
	private function special_buttons ( $dashboard_div ) {
		if ( 'dashboard_activity'  == $dashboard_div ) {
			$start_control = WIC_Control_Factory::make_a_control ( 'date' );
			$start_control->initialize_default_values( 'dashboard', 'date_range', 'start' );
			$end_control = WIC_Control_Factory::make_a_control ( 'date' );
			$end_control->initialize_default_values( 'dashboard', 'date_range', 'end' );
			return 	$start_control->form_control() . $end_control->form_control() . WIC_Entity_Activity::make_activity_type_filter_button();
		} elseif ( 'dashboard_activity_type'  == $dashboard_div ) {
			$start_control = WIC_Control_Factory::make_a_control ( 'date' );
			$start_control->initialize_default_values( 'dashboard', 'date_range', 'start_t' );
			$end_control = WIC_Control_Factory::make_a_control ( 'date' );
			$end_control->initialize_default_values( 'dashboard', 'date_range', 'end_t' );
			return $start_control->form_control() . $end_control->form_control();
		} else {
			return '';
		}
	}



	public static function dashboard_activity ( $dummy_id, $data ) {

		extract ( (array) $data );
		$start = $start ? $start : '1900-01-01';
		$end = $end ? $end : '2100-01-01';
		$type_string = '';
		$first = ''; 
		foreach ( $included as $type ) {
			$type_string .= "$first'" . $type . "'"; 
			$first = ',';
		}
		$in_clause = count ( $included ) > 0 ? " activity_type IN ( $type_string ) " : "  activity_type IS NULL "; // no null values

		// set global access object 
		global $wpdb;

		$join = $wpdb->prefix . 'wic_activity activity inner join ' . $wpdb->prefix . 'wic_constituent c on c.id = activity.constituent_id';
		$activity_sql = "
				SELECT constituent_id, issue, max(pro_con) as pro_con
				FROM $join
				WHERE activity_date <= '$end' and activity_date >= '$start' and $in_clause 
				GROUP BY activity.constituent_ID, activity.issue
				LIMIT 0, 9999999
					";	
		// $sql group by always returns single row, even if multivalues for some records 
		$sql =  "
				SELECT issue as id, count(constituent_id) as total, sum( if (pro_con = '0', 1, 0) ) as pro,  sum( if (pro_con = '1', 1, 0) ) as con  
				FROM ( $activity_sql ) as a 
				GROUP BY issue
				ORDER BY count(constituent_id) DESC
				";

		$result = $wpdb->get_results( $sql );
		$count = count ( $result );
		$wic_query = (object) array ( 
			'result' =>$result,
			'entity' => 'trend',
			'showing_count' => $count,
		);

		if ( 0 == count ( $wic_query->result ) ) {
			return array ( 'response_code' => 'true', 'output' => '<div class="dashboard-not-found">' . __( 'No activities found.', 'wp-issues-crm' ) . '</div>' );		
		} else { 
			$lister = new WIC_List_Trend;
			$list = $lister->format_entity_list( $wic_query,'' );
			return array ( 'response_code' => true, 'output' => $list);			
		}
		
	}


	public static function dashboard_activity_type ( $dummy_id, $data ) {

		extract ( (array) $data );
		$start = $start ? $start : '1900-01-01';
		$end = $end ? $end : '2100-01-01';
	
		// set global access object 
		global $wpdb;

		// get activity types
		$option_table = $wpdb->prefix  . 'wic_option_value';
		$sql = "SELECT option_value, option_label FROM $option_table WHERE parent_option_group_slug = 'activity_type_options'";
		$option_list = $wpdb->get_results ( $sql );
		// if none, return			
		if ( 0 == count ( $option_list ) ) {
			return array ( 'response_code' => 'true', 'output' => '<div class="dashboard-not-found">' . __( 'No activities types defined.', 'wp-issues-crm' ) . '</div>' );		
		}

		// pass type list to prepare select terms for each type, array of fields mapped to query values, and string of types
		$term_string = 'issue as id, count(activity_type) as total ';
		$fields = array ( array ( 'Issue', 'id' ), array ( 'Total', 'total') );
		$type_string = '';
		$first = '';
		foreach ( $option_list as $type ) {
			$value = $type->option_value;
			$label = 'type' . $value;
			$term_string .= ", sum( if( activity_type = '$value', 1, 0 ) ) as $label";
			$fields[] = array ( $type->option_label, $label );
			$type_string .= "$first'" . $value . "'"; 
			$first = ',';
		}
		// prepare extra term for not found type ( i.e., type code option was eliminated after start of search period );
		$term_string .= ", sum( if ( activity_type NOT IN( $type_string ), 1, 0 ) ) as nf";
		$fields[] = array ( 'NF type', 'nf' );
		
		$activity_table = $wpdb->prefix . 'wic_activity';
		$activity_sql = "
				SELECT $term_string 
				FROM  $activity_table a 
				WHERE activity_date <= '$end' and activity_date >= '$start' 
				GROUP BY issue
				ORDER BY count(a.ID) DESC
				";

		$result = $wpdb->get_results( $activity_sql );
		$count = count ( $result );
		$wic_query = (object) array ( 
			'result' =>$result,
			'entity' => 'trend',
			'showing_count' => $count,
			'fields' => $fields,
		);

		if ( 0 == count ( $wic_query->result ) ) {
			return array ( 'response_code' => 'true', 'output' => '<div class="dashboard-not-found">' . __( 'No activities found.', 'wp-issues-crm' ) . '</div>' );		
		} else { 
			$lister = new WIC_List_Activity_Type;
			$list = $lister->format_entity_list( $wic_query,'' );
			return array ( 'response_code' => true, 'output' => $list);			
		}
		
	}

}

