<?php
/*
*
*	wic-entity-email-inbox-parse.php
*
*/
Class WIC_Entity_Email_Inbox_Parse {


	public static function parse_messages() {

		// set time limit for job ( in later versions get as setting )
		$max_allowed_time = 110; // two minutes less some Wordpress set up time to get here and some time to complete parse
		ini_set( 'max_execution_time', $max_allowed_time + 10 );
  		$cut_off_time = time() + $max_allowed_time;

		// get the full folder string
		$wic_plugin_options = get_option( 'wp_issues_crm_plugin_options_array' ); 
		$folder = $wic_plugin_options['imap_inbox'];	

		// prepare for database access
		global $wpdb;
		$inbox_table = $wpdb->prefix . 'wic_inbox_image';

		$max_packet_size = self::get_max_packet_size();

		// define sql to get next blank message 
		// ordering just reduces probability of unnecessary parsing of to be deleted msgs
		$sql = "SELECT ID, folder_uid FROM $inbox_table 
				WHERE 
					full_folder_string = '$folder' AND
					no_longer_in_server_folder = 0 AND
					serialized_email_object = ''
				ORDER BY folder_uid ASC
				LIMIT 0,1";

		// do we have work to do? if so, open connection; if not, quit;
		if ( $wpdb->get_results ( $sql ) ) {
			// check connection to inbox folder and get processed folder string
			$connection_response = WIC_Entity_Email_Connect::check_connection();
			if ( false === $connection_response['response_code'] ) {
				if ( WP_DEBUG ) {
					error_log ( 'Failed to connect on WIC_Entity_Email_Inbox_Parse::parse_messages -- error was: ' . $connection_response['output'] );
				}
				die;
			}
			// after check connection, if no error, this global is the inbox stream resource
			global $wp_issues_crm_IMAP_stream;
		} else {
			if ( WP_DEBUG ) {
					error_log ( 'WIC_Entity_Email_Inbox_Parse:parse_messages found no messages to parse.' );
				}
			die;			
		}

		// set up counters
		$deleted_messages = 0;
		$parsed_messages = 0;
		$errors = 0;	
		// look for blank serialized_email_object and parse them for as long as allowed (and there are any)
		while ( 
			time() < $cut_off_time &&
			$result = $wpdb->get_results ( $sql )
			) {
			$current_uid = $result[0]->folder_uid;
			$ID = $result[0]->ID;
			$email_object = new WIC_DB_Email_Message_Object;
			$email_object->build_from_stream_uid ( $ID, $wp_issues_crm_IMAP_stream, $current_uid, $max_packet_size );
			// if uid has apparently been deleted while parse in progress mark it as such
			//  (but first check to make sure it was not a connection error)
			//  note that since working with uid's, no possibility of seq shift error
			//  note that deleting bad uid outside main delete process cannot conflict because 
			// 	   main UID process only deletes by range -- will simply show as not deleted if attempted
			//	   if throws off delete count, will exit loop by too many hunts or step beyond or time
			if ( ! $email_object->uid_still_valid ) {
				if ( false === imap_num_msg( $wp_issues_crm_IMAP_stream ) && WP_DEBUG ) {
					error_log ( "Apparently lost connection in WIC_Entity_Email_Inbox_Parse::parse_messages." );
					die;
				} else {
					$sql_delete = "UPDATE $inbox_table SET no_longer_in_server_folder = 1 WHERE folder_uid = $current_uid";
					$wpdb->query ( $sql_delete );
					$deleted_messages++;
					continue;
				}
			}
			/*
			* protect against oversized messages (likely due to insertion of inlined images)
			*	-- first discard html version -- should solve problem almost always
			*	-- next discard both text versions and alert user
			* even in degenerate case, save serialized_email_object without text
			*/
			$sql_update = self::prepare_message_update_sql ( $email_object, $folder, $current_uid );
			if ( strlen ( $sql_update ) > $max_packet_size ) { 
 				$email_object->raw_html_body = 
 					"<h3>Original message size (plus overhead) exceeds system max_packet_size ( $max_packet_size ) -- showing text only.</h3>" .
					wpautop( balanceTags( $email_object->raw_text_body ) ); // raw_text_body already sanitized through kses
				$sql_update = self::prepare_message_update_sql ( $email_object, $folder, $current_uid );
 			}
			if ( strlen ( $sql_update ) > $max_packet_size ) { 
 				$email_object->raw_html_body = 
 					"<h3>Message size (plus overhead) exceeds system max_packet_size ( $max_packet_size ) -- unable to process.</h3>"; 
				$email_object->raw_text_body = '';
				$email_object->text_body = '';
				$sql_update = self::prepare_message_update_sql ( $email_object, $folder, $current_uid );
 			}
			$result = $wpdb->query ( $sql_update );
 
			// save md5 representation of the non_address text of the message
			if ( false !== $result ) {
				WIC_Entity_Email_MD5::save_md5_digest ( $email_object->sentence_md5_array, $ID );
				$parsed_messages++;
			} else {
				$errors++;
			}
			
			
			// redefine query to limit to next UID -- preventing looping on update failure (database gone away or lost connection)
			$sql = "SELECT ID, folder_uid FROM $inbox_table 
				WHERE 
					full_folder_string = '$folder' AND
					no_longer_in_server_folder = 0 AND
					serialized_email_object = '' AND
					folder_uid > $current_uid
				ORDER BY folder_uid ASC
				LIMIT 0,1";
			
		}
		// log completion in debug mode
		if ( WP_DEBUG ) {
			error_log ( "WP Issues CRM -- message parser completed, parsing $parsed_messages messages and deleting $deleted_messages messages; $errors messages had update errors.  WIC_Entity_Email_Inbox_Parse:parse_messages." );
		}
		// close connection
		imap_close ( $wp_issues_crm_IMAP_stream );

		die; 
	}

	public static function get_max_packet_size () {
		global $wpdb;
		// determine maximum packet size
		$vars = $wpdb->get_results( " select VARIABLE_VALUE from information_schema.GLOBAL_VARIABLES where VARIABLE_NAME = 'max_allowed_packet' ");
		$max_packet_size = isset( $vars[0]->VARIABLE_VALUE ) ? $vars[0]->VARIABLE_VALUE : 'UNSET' ;
		// if this query worked as expected -- nonzero number -- use it, other wise use 1000000 (haven't seen lower; go much higher to 16M)
		if ( is_numeric ( $max_packet_size ) ) {
			$max_packet_size = $max_packet_size ? $max_packet_size : 1000000;
		} else {
			$max_packet_size = 1000000;
		}
		
		return $max_packet_size;
	}	
	
	private static function prepare_message_update_sql ( $email_object, $folder, $current_uid ) {

		// prepare for database access
		global $wpdb;
		$inbox_table = $wpdb->prefix . 'wic_inbox_image';

		// do md5 guess mapping
		$md5_map_result = WIC_Entity_Email_MD5::get_issue_and_pro_con_map_from_md5( $email_object->sentence_md5_array );

		// do subject mapping
		$subject_line_map_result = WIC_Entity_Email_Subject::get_subject_line_mapping( $email_object->subject );

		// do constituent lookup
		$constituent_id = self::do_constituent_lookup( $email_object );
		
		// apply rules to label parse quality
		$parse_quality = self::evaluate_parse_quality( $email_object );
		
		// apply message filtering
		$filtered = WIC_Entity_Email_Block::apply_address_filter ( $email_object->from_email );

		// prepare sql from message object 
		return $wpdb->prepare ( "
				UPDATE $inbox_table SET
					from_personal = %s,
					from_email = %s,
					from_domain = %s,	
					raw_date = %s,		 	
					email_date_time = %s,
					subject = %s,			
					activity_date = %s,
					guess_mapped_issue = %d,
					guess_mapped_issue_confidence = %d,
					guess_mapped_pro_con = %s,
					non_address_word_count = %s,
					mapped_issue = %s,
					mapped_pro_con = %s,
					serialized_email_object = %s,
					assigned_constituent = %d,
					parse_quality = %s,
					to_be_moved_on_server = %s
				WHERE full_folder_string = %s AND folder_uid = %d
				",
				array(
					$email_object->from_personal,
					$email_object->from_email,
					$email_object->from_domain,	
					$email_object->raw_date,		 	
					$email_object->email_date_time,		 	
					$email_object->subject,			
					$email_object->activity_date,
					$md5_map_result['guess_mapped_issue'],
					$md5_map_result['guess_mapped_issue_confidence'],
					$md5_map_result['guess_mapped_pro_con'],
					$md5_map_result['non_address_word_count'],
					$subject_line_map_result ? $subject_line_map_result['mapped_issue'] : '',
					$subject_line_map_result ? $subject_line_map_result['mapped_pro_con'] : '',
					serialize ( $email_object ),
					$constituent_id,
					$parse_quality,
					$filtered,
					$folder,
					$current_uid
				)							
			);

	}

	private static function do_constituent_lookup ( &$email_object ) {
		
		// passing only constituent lookup elements of array
		$parsed_message_keys = array ( 
				'email_address',
				'phone_number',
				'first_name',
				'middle_name',
				'last_name',
				'address_line',
				'city',
				'state',
				'zip',
				'activity_date'
		);	

		// push message parms onto key value array (all are set, although may be blank);
		$key_value_array = array();
		foreach ( $parsed_message_keys as $key ) {
			if (  $email_object->$key ) {
				$key_value_array[$key] = $email_object->$key;
			}
		}

		if ( ! $key_value_array ) {
			return 0;
		} else {
			$constituent_activity = new WIC_Interface_Transaction;
			$constituent_id = $constituent_activity->save_constituent_activity ( 
				$key_value_array, 
				array(), // no special key values 
				array(), // no special policies
				true, // unsanitized = true (so sanitize)
				array (
					'emailfn',	
					'email',
					'lnfnaddr',
					'lnfnaddr5',
					'lnfndob',
					'lnfnzip',
					'fnphone',
				),
				true // this is just a dry run
			);  
		}

		return $constituent_id;
		
	}
	
	// classify results from 6 worst to 1 best
	private static function evaluate_parse_quality ( &$email_object ) {
		if ( '' == $email_object->email_address ) {
			return 6; // below lowest quality
		} elseif ( 0 == $email_object->outcome || 4 == $email_object->outcome ) {
			return 5;
		} elseif ( 2 == $email_object->outcome || 6 == $email_object->outcome ) {
			return 4;
		} elseif ( 1 == $email_object->outcome || 3 == $email_object->outcome ) {
			return 3;
		} elseif ( 5 == $email_object->outcome || 8 == $email_object->outcome || 9 == $email_object->outcome ) {
			return 2;
		} // what is left? 7, 10, 11 kept in strict mode 1
		return 1;
	}


	public static function unparse_inbox() {
		/* 
		* note: this process could start in the middle of a parse process
		* (1) parse process runs monotonically by uid; it will not go back to retry parsed messages that this routine unparses
		* (2) this process has no effect on unparsed messages
		* (3) this process has no effect on half-parsed messages -- saved md5/attachments, but not object . . . but this will not be selected
		*
		* messages reset will get reparsed on the next parse run (which will start at lowest unparsed folder_uid)
		*
		* no need to unset activities, since not touching to_be_move_on_server records
		* could start this process after loading of inbox -
		* (1) if run this before, scroll to message, will get not found error
		* (2) if run this before attempt to process, will skip message and ignore -- will resurface in inbox
		* (3) if run this after reserved for processing, will complete processing and mark for deletion (may show as originally parsed in done list)-- OK outcome
		* (4) if run this after processed and released, will not see message
		* 
		*/
		global $wpdb;
		$wic_plugin_options = get_option( 'wp_issues_crm_plugin_options_array' ); 
		$folder = $wic_plugin_options['imap_inbox'];
		
		// table names
		$inbox_table = $wpdb->prefix . 'wic_inbox_image';
		$md5_table = $wpdb->prefix . 'wic_inbox_md5';
		$attachments_table = $wpdb->prefix . 'wic_inbox_image_attachments';
	
		// purge attachments and md5 coding of image text			
		$sql = $wpdb->prepare ( 
			"
			DELETE m5, a 
			FROM $inbox_table m LEFT JOIN 
				$md5_table m5 on m5.inbox_message_id = m.ID LEFT JOIN
				$attachments_table a on a.message_id = m.ID
			WHERE full_folder_string = %s and to_be_moved_on_server = 0 and no_longer_in_server_folder = 0 and serialized_email_object > ''
			", array ( $folder ) ); 		
		$result = $wpdb->query ( $sql );
	
		// reset the serialized_email_object to empty string			
		$sql = $wpdb->prepare ( 
			"
			UPDATE $inbox_table m 
			SET serialized_email_object = ''
			WHERE full_folder_string = %s and to_be_moved_on_server = 0 and no_longer_in_server_folder = 0 and serialized_email_object > ''
			", array ( $folder ) ); 		
		$result = $wpdb->query ( $sql );		
		
		if ( WP_DEBUG ) {
			error_log ( "WP Issues CRM -- inbox parse reset completed, resut $result messages.  WIC_Entity_Email_Inbox_Parse::unparse_inbox." );
		}
		
		return WIC_Entity_Email_Inbox_Synch::load_staged_inbox ( '', '' );  
	}
	
}