<?php
/*
*
*	wic-entity-email-inbox.php
*
*/
Class WIC_Entity_Email_Inbox extends WIC_Entity_Parent {


	/*
	*
	* basic entity functions
	*
	*
	*/
	protected function set_entity_parms( $args ) {
		$this->entity = 'email_inbox';
		$this->entity_instance = '';
	} 

	// special version of this function to allow checking of settings before form display 
	protected function new_blank_form( $args = '', $guidance = '' ) { 

		$wic_settings = get_option( 'wp_issues_crm_plugin_options_array' );
		
		// check if have basic imap settings in place
		if 	( 
			! $wic_settings['email_imap_server'] || 
			! $wic_settings['user_name_for_email_imap_interface'] ||
			! $wic_settings['password_for_email_imap_interface' ]
			){
			$this->display_email_process_not_configured_splash();
		} else {
			global $wic_db_dictionary;
			$this->fields = $wic_db_dictionary->get_form_fields( $this->entity );
			$this->initialize_data_object_array();
			$new_form = new WIC_Form_Email_Inbox;
			$new_form->layout_inbox( $this->data_object_array, $guidance, 'guidance' );
		}
	}	

	private function display_email_process_not_configured_splash() {
		echo '
			<div id="unconfigured-message-for-process-email">
				<p><strong>WP Issues CRM Email Processor: Review, classify <em>and automatically reply</em> to incoming email.</strong></p>
				<ul><li>Reads a standard IMAP email inbox.</li>
				<li>Parses the messages to extract name and address information.</li>
				<li>Logs messages as activities for existing constituents and creates new constituents as necessary.</li>
				<li>Suggests full reply texts based on your prior replies (mapping based on subject line, sentence content and/or word content).</li>
				<li>Allows fully automatic replies when you elect to train a mapping of incoming subject lines and content to Issues/Replies.</li>
				<li>Allows you to use public or private issues/posts as email reply message templates.</li>
			</ul>
			<p><strong>To get started, configure server, user and password at WP Issues CRM &raquo; Settings &raquo; Email In.</strong></p>
		';
	}

   /*
   * functions supporting field definitions
   */
   // pass through from original entity
	public static function get_issue_options( $value ) {
		return ( WIC_Entity_Activity::get_issue_options( $value ) );
	}
	public static function get_inbox_options ( $value ) {
		return array (
			array ( 'value' => 'inbox'    		,  'label' => 'Inbox'   ),
			array ( 'value' => 'done'			,  'label' => 'Done'  ),
			array ( 'value' => 'outbox'			,  'label' => 'Outbox'   ),
			array ( 'value' => 'sent'			,  'label' => 'Sent'  ),
			array ( 'value' => 'saved'			,  'label' => 'Saved'  ),
			array ( 'value' => 'manage-subjects',  'label' => 'Subjects'  ),
			array ( 'value' => 'manage-blocks'	,  'label' => 'Blocked'   ),
			array ( 'value' => 'inbox-synch'  	,  'label' => 'Synch'   	 ),
			array ( 'value' => 'settings'  		,  'label' => 'Settings'   ),
		);
	}

	public static function load_inbox ( $dummy_id, $data ) { 
		/*
		* $data->mode coming from the client is "single" or "grouped" 
		* 
		* Since version 3.2, all inbox class functions run directly of the WP Issues CRM inbox image, not the IMAP server inbox
		* 
		*/
	
		// get date limits
		$form_variables_object = unserialize ( get_option ( 'wp-issues-crm-email-processing-options' ) );
		$parse_strictness = isset ( $form_variables_object->parse_strictness ) ? $form_variables_object->parse_strictness : '5'; // default to loosest standard (max strictness = 1; 6 = no email address)
		// get confidence thresholds
		$word_minimum_threshold = isset ( $form_variables_object->word_minimum_threshold ) ? $form_variables_object->word_minimum_threshold : 200;
		$mapped_threshold = isset ( $form_variables_object->mapped_threshold ) ? $form_variables_object->mapped_threshold : 85;
		// get current folder
		$wic_plugin_options = get_option( 'wp_issues_crm_plugin_options_array' ); 
		$folder = isset ( $wic_plugin_options['imap_inbox'] ) ?  $wic_plugin_options['imap_inbox'] : ''  ;
		// bounce if no folder selected
		if ( '' == $folder ) {
			return array ( 'response_code' => false , 'output' => 'Check settings -- no inbox folder selected.' ) ;
		}
		
		/*
		* Compose sql statements to assemble inbox lines from constant and mode-dependent elements
		*
		* Note: the if statements in this query (and in load_message_detail ) implement the following conservative logic tree for choosing when to group issues:
		*
		* Group messages if and only if:
		*   (1) There is a subject map record that they share (so, same subject) (not expired);
		*	(2) They all have content mapped to the same issue/pro-con, which agrees with the subject mapped result
		*   (3) The content match meets both the required confidence percentage and the required word count percentage
		*   NOTE: There may or may not be a reply already assigned to the issue -- could be just recording
		*
		*	Note: not grouping without a subject line match -- can hit high confidence numbers on reply and forward strings, possibly in other ways.
		*   Solution for variable subject lines is wild card.
		*
		*   most of the inbox is cosmetic: the only processing consequences of the inbox line content flow from 
		*		* "have_trained_issue" (which determines, by adding a class below, whether a line is grouped and so eligible for sweeps per above strict criterion) 
		*	 	*  and the folder_uid list (which is the basis of all line processing)
		*
		*   note that there is an overriding ungroup rule which is parse quality vs. strictness (lower is better/stricter, so ungroup if quality > strictness)
		*/
		global $wpdb;
		// start by attempting to set group_concat_max_len to handle long UID lists
		try {
			// wordpress should trap and report error if can't set it, but don't want to see the error
			ob_start();
			$wpdb->query( "SET SESSION group_concat_max_len = 100000" );
			ob_end_clean();
		} catch ( Exception $err ) {
			// catch is belt and suspenders in case wordpress fails to trap error
			error_log ( print_r ( $err, true ) );
		}
		// check setting of group_concat_max_len
		$shown_vars = $wpdb->get_results ( "SHOW VARIABLES LIKE 'group_concat_max_len'" );
		if ( is_array( $shown_vars  ) && ( 1 == count ( $shown_vars ) ) ){
			$group_concat_max_length = $shown_vars[0]->Value;
		} else {
			$group_concat_max_length = 1024; // assume default if not accessible
		}
		/*
		* sweep_definition is critical concept that enforces strictness of automated reply processing per notes above
		*/
		$inbox_image_table = $wpdb->prefix . 'wic_inbox_image';
		$sweep_definition = " 
			mapped_issue > 0 AND 
			guess_mapped_issue = mapped_issue AND 
			guess_mapped_pro_con = mapped_pro_con AND
			guess_mapped_issue_confidence >= $mapped_threshold AND 
			non_address_word_count >= $word_minimum_threshold ";

		$filter = sanitize_text_field ( $data->filter );
		if ( $filter > '' ) {
			$filter_where = $wpdb->prepare ( " LOCATE( %s, subject ) > 0 AND ", array( $filter ) );
		} else {
			$filter_where = '';
		}

		// limit selection to inbox content
		$other_where_terms = 
			"full_folder_string = '$folder' AND
				no_longer_in_server_folder = 0 AND
				to_be_moved_on_server = 0 AND
				serialized_email_object > ''
			";	

		// key implementing language for group options (note that higher parse_quality number is worse parse, up to 6 with no email address
		$group_lines =
			"GROUP BY BINARY
				IF ( parse_quality > '$parse_strictness', 
					folder_uid,  
					IF( $sweep_definition, subject, folder_uid )
				)

 			ORDER BY min( if ( email_date_time > '', email_date_time, activity_date ) ) " . $data->sort . ' '; // the conditional is for transition to email_date_time -- sb removed in a later version
		$ungroup_lines = 
			" ORDER BY IF ( email_date_time > '', email_date_time, activity_date )  " . $data->sort . " ";


		// set max count (fixed )
		$max_count = 50;
 		// set page variable
 		$page_base = $data->page * $max_count;
		
		// assemble sql statements -- two version of SELECT, one for ungrouped, one for grouped
		$subjects_array_sql = 
			(
			'grouped' != $data->mode ?
			" 
			SELECT SQL_CALC_FOUND_ROWS
				subject,
				from_email,
				from_domain,
				if ( email_date_time > '', email_date_time, activity_date ) as oldest,
				1 as `count`,
				folder_uid as UIDs,
				from_personal as `from`,
				IF( $sweep_definition, 1, 0 ) as have_trained_issue,
				0 as conversation,
				0 as many
			"
			:
			"
			SELECT SQL_CALC_FOUND_ROWS
				subject,
				from_email,
				from_domain,
				min( if ( email_date_time > '', email_date_time, activity_date ) ) as oldest, 
				count(folder_uid) as `count`, 
				group_concat( folder_uid ORDER BY folder_uid ASC SEPARATOR ',' ) as UIDs," . // this uid list becomes the array that drives processing when the user sweeps or shifts to inbox view
				"from_personal as 'from'," . // used only in conjunction with many for display purposes
				"IF( $sweep_definition, 1, 0 ) as have_trained_issue," . // have_trained_issue is variable that determines eligibility for sweep by setting class 'trained-class' -- group by logic assures that all in group have same sweep definition
				"IF(LEFT(subject,3)='re:', 1, 0 ) as conversation,
				if(COUNT(DISTINCT from_personal) > 1, 1, 0 ) as many
			"
			) . 
			"
			FROM $inbox_image_table
			WHERE 
				$filter_where
				$other_where_terms
			" . 
			(
			'grouped' == $data->mode ?
				$group_lines :
				$ungroup_lines
			) .
			"
			LIMIT $page_base, $max_count 					
			"
			;

		// get subjects array
		$subjects_array = $wpdb->get_results( $subjects_array_sql );
		// get count total messages ( as filtered and/or grouped )
		$found_result = $wpdb->get_results( "SELECT FOUND_ROWS() AS found_count" );	
		$found_count = $found_result[0]->found_count;
		
		// choose terms based on parms for use in both branches of the conditional
		$sort_order = ( 'ASC' == $data->sort ) ? ' first-arrived ' : ' last-arrived ';
		$loaded_object = 'grouped' == $data->mode ? 'subject lines' :   'messages';
		$filter_statement = $filter ? ' (filtered by "' . $filter . '")' : '';
		$view_statement = "Viewing $loaded_object, $sort_order first" . "$filter_statement. $max_count per page." ;

		$count_subjects = 0;
		if ( $subjects_array ) {
			/*
			* create inbox output -- consider this an interface from prior message analysis, but only some elements of each subject line have consequences down stream:
			* -- trained-subject class, driven by query analysis of message above (does the message in this subject line meet strict sweep criteria?);
			*	 . . . used in wpIssuesCRM.processEmail (email-process.js) to aggregate a list sweepable uids
			*	 . . . if NOT in sweep mode, processing runs off the issue/pro_con and template coming from the form
			* -- array (comma separated in each line) of UIDs defines which messages are acted on
			* -- count of messages is used in chrome for both inbox and inbox detail
			*/
			$output = '<ul id="inbox-subject-list">';
			foreach ( $subjects_array as $subject  ) {
				$from_summary = $subject->from . ( $subject->many ? ' +' : '' );
				// regardless of count, if already mapped mark with class and show the already mapped legend
				if ( $subject->have_trained_issue > 0  ) {
					$trained_class = ' trained-subject ';
					$trained_legend = 'Trained: ';
				} else {
					$trained_class = '';
					$trained_legend = '';
				}
				/*
				* manage truncated UID list -- can't count on ability of user to set session value for group_concat_max_len over 1024
				*/
				// if apparently truncated (OK if wrong, just unnecessarily discard a UID)
				if ( $group_concat_max_length == strlen ( $subject->UIDs ) ) {
					$UIDs = explode( ',',  $subject->UIDs );
					array_pop( $UIDs ); // discard last element, possibly truncated
					$uid_count = count ( $UIDs );
					$uid_list = implode ( ',', $UIDs );
					$from_summary = "Many -- over group_concat_max_length";
				} else {
					$uid_count = $subject->count;
					$uid_list = $subject->UIDs;
				}
				// format an li for the inbox
				$output .= '<li class="inbox-subject-line '. $trained_class .'"><ul class="inbox-subject-line-inner-list">' . // *class determines sweepability*
					'<li class = "subject-line-item from-email">' . $subject->from_email . '</li>' . // hidden, used to filter out blocked
					'<li class = "subject-line-item from-domain">' . $subject->from_domain . '</li>' . // hidden, used to filter out blocked
					'<li class = "subject-line-item from-summary">' . $from_summary. '</li>' . // just display
					'<li class = "subject-line-item count" title = "Message Count"><span class="inner-count">' . $uid_count . '</span></li>' . // *supports multiple UI elements*
					'<li class = "subject-line-item subject' . $trained_class . '">' . $trained_legend . '<span class="actual-email-subject">' . $subject->subject . '</span></li>' . // just display
					'<li class = "subject-line-item oldest" title="Date of oldest">' . $subject->oldest . '</li>' . // just display
					'<li class = "subject-line-item UIDs">' . $uid_list . '</li>' . // *pass through for all processing*
				'</ul></li>';
				$count_subjects++;
			}
			$output .= '</ul>';
			/*
			* assemble page links and explanatory legend at end of inbox display
			*
			*/
			$output .= 
				'<div id = "wic-inbox-list-footer">' .
					'<div class = "wic-inbox-footer-legend">' . $view_statement . '</div>' . 
				'</div>';
		// no messages found
		} else {
			$output = $filter  ? 
				( '<div id = "filtered-all-warning">No message subject line containing "' . $filter . '".</div>' ) : 
				('<div id="inbox-congrats"><h1>Congrats!  You have hit the bottom of your inbox.</h1>' . 
				'<p><em>If your inbox is empty and that seems wrong, check the Synch page -- it could be that parsing is in progress.</em><p>
				 <p><b><em>Note that upgrade to version 3.3 or beyond from lower versions triggers a full inbox reparse (which make take 1 or more minutes per 100 messages).</b></em></div>');
		}

		$connection_failures = WIC_Entity_Email_Connect::check_connection_failure_count() ? '' : 
			( 
			  '<p>WP Issues CRM encountered connection failures while polling your inbox.</p>' .
			  '<p>Perhaps you changed your email password?</p>' .
			  '<p>Please visit <a href="' . site_url() .'/wp-admin/admin.php?page=wp-issues-crm-settings">WP Issues CRM Settings </a>&raquo; Email In and "Test Saved Settings".</p>' .
			  '<p>Testing settings will reset the failure counter.</p>' .
			  '<p>If your settings are good, you may want to adjust upwards the accepted number of failures.</p>'

			);

		// construct inbox title
		$inbox_header = 
			'"' . WIC_Form_Email_Inbox::full_folder_string_to_folder ( $folder ). '": ' . 
			( $page_base + 1 ) . '-' . ( $page_base + $count_subjects ) . ' of ' . $found_count . ' ' . $loaded_object; 
		
		if ( $filter ) {
			$inbox_header .= " filtered by `$filter`";
		}
		
		$load_parms = (object) array ( 
			'folder'				=>   $folder,
			'filter'				=> 	 $filter,
			'page_ok'				=>	 ( $found_count > $page_base || 0 == $found_count ), // flag in case pages have shifted through record consolidation
		);		

		$return_array = (object) array (
			'inbox' => $output,
			'inbox_header' => $inbox_header,
			'nav_buttons' => array ( 'disable_prev' => $data->page == 0, 'disable_next' => ( $page_base + $max_count > $found_count ) ),
			'stuck' => WIC_Entity_Email_UID_Reservation::check_old_uid_reservations(),
			'connection' => $connection_failures,
			'last_load_parms' => $load_parms,
		);
		
		return array ( 'response_code' => true, 'output' => $return_array  ); 
	}

	public static function load_sent ( $dummy_id, $data ) {
		return self::load_sent_outbox ( 1, $data );
	}

	public static function load_outbox ( $dummy_id, $data ) {
		return self::load_sent_outbox ( 0, $data );
	}

	private static function load_sent_outbox ( $sent_ok, $data ) { 
		/*
		* Return looks like inbox to js
		*
		*/
		global $wpdb;
		$outbox = $wpdb->prefix . 'wic_outbox';
		$filter = sanitize_text_field ( $data->filter );
		if ( $filter > '' ) {
			$filter_where = $wpdb->prepare ( " LOCATE( %s, subject ) > 0 AND ", array( $filter ) );
		} else {
			$filter_where = '';
		}
		// limit selection to sent content
		$other_where_terms = 
			" sent_ok = $sent_ok ";	
			
		$oldest = $sent_ok ? "sent_date_time" : "queued_date_time";

		// set max count (fixed )
		$max_count = 50;
 		// set page variable
 		$page_base = $data->page * $max_count;
		
		// assemble sql statements -- two version of SELECT, one for ungrouped, one for grouped
		$subjects_array_sql = 
			(
			'grouped' != $data->mode ?
			" 
			SELECT SQL_CALC_FOUND_ROWS
				ID,
				subject,
				serialized_email_object,
				$oldest as oldest,
				1 as `count`
			"
			:
			"
			SELECT SQL_CALC_FOUND_ROWS
				ID,
				subject,
				serialized_email_object,
				min( $oldest ) as oldest, 
				count(ID) as `count`
			"
			) . 
			"
			FROM $outbox
			WHERE 
				$filter_where
				$other_where_terms
			" . 
			(
				'grouped' == $data->mode ? 
					( " GROUP BY SUBJECT ORDER BY MIN( $oldest ) " . $data->sort . ' ' ) : 
					( " ORDER BY $oldest " . $data->sort . ' ')
			) .
			"
			LIMIT $page_base, $max_count 					
			"
			;
		// get subjects array
		$subjects_array = $wpdb->get_results( $subjects_array_sql );
		// get count total messages ( as filtered and/or grouped )
		$found_result = $wpdb->get_results( "SELECT FOUND_ROWS() AS found_count" );	
		$found_count = $found_result[0]->found_count;
		
		// choose terms based on parms for use in both branches of the conditional
		$past_tense = $sent_ok ? 'sent' : 'queued';
		$sort_order = ( 'ASC' == $data->sort ) ? " first-$past_tense " : " last-$past_tense ";
		$loaded_object = 'grouped' == $data->mode ? 'subject lines' :   'messages';
		$filter_statement = $filter ? ' (filtered by "' . $filter . '")' : '';
		$view_statement = "Viewing $loaded_object, $sort_order first" . "$filter_statement. $max_count per page." ;

		$count_subjects = 0;
		if ( $subjects_array ) {
			$output = '<ul id="inbox-subject-list">';
			foreach ( $subjects_array as $subject  ) {
				$email_object = unserialize ( $subject->serialized_email_object );
				$to_summary = ( trim($email_object->to_array[0][0]) ? trim( $email_object->to_array[0][0] ) : $email_object->to_array[0][1] ) . ( $subject->count > 1 ? '+' : '' );
				// format an li for the inbox
				$output .= '<li class="inbox-subject-line "><ul class="inbox-subject-line-inner-list">' .
					'<li class = "subject-line-item message-ID">' . $subject->ID . '</li>' .
					'<li class = "subject-line-item from-summary">' . $to_summary. '</li>' .
					'<li class = "subject-line-item count" title = "Message Count"><span class="inner-count">' . $subject->count . '</span></li>' .
					'<li class = "subject-line-item subject"><span class="actual-email-subject">' . $subject->subject . '</span></li>' . 
					'<li class = "subject-line-item oldest" title="Date of oldest">' . $subject->oldest . '</li>' .
				'</ul></li>';
				$count_subjects++;
			}
			$output .= '</ul>';
			/*
			* assemble page links and explanatory legend at end of inbox display
			*
			*/
			$output .= 
				'<div id = "wic-inbox-list-footer">' .
					'<div class = "wic-inbox-footer-legend">' . $view_statement . '</div>' . 
				'</div>';
		// no messages found
		} else {
			$output = $filter  ? 
				( '<div id = "filtered-all-warning">No message subject line containing "' . $filter . '".</div>' ) : 
				('<div id="inbox-congrats"><h1>No ' . ( $sent_ok ? 'sent' : 'unsent' ) . ' messages.</h1>' );
		}

		// construct inbox title
		$inbox_header = 
			( $sent_ok ? 'Sent: ' : 'Outbox: ' ) . 
			'<span id="outbox-lower-range">' . ( $found_count ? $page_base + 1 : 0 ) . '</span>-<span id="outbox-upper-range">' . ( $page_base + $count_subjects ) . '</span> of <span id="outbox-total-count">' . $found_count . '</span> ' . $loaded_object; 
		
		$load_parms = (object) array ( 
			'filter'				=> 	 $filter,
			'page_ok'				=>	 ( $found_count > $page_base || 0 == $found_count ), // flag in case pages have shifted through record consolidation
		);		

		$return_array = (object) array (
			'inbox' => $output,
			'inbox_header' => $inbox_header,
			'nav_buttons' => array ( 'disable_prev' => $data->page == 0, 'disable_next' => ( $page_base + $max_count > $found_count ) ),
			'stuck' => false,
			'connection' => false,
			'last_load_parms' => $load_parms,
		);
		
		return array ( 'response_code' => true, 'output' => $return_array  ); 
	}


	public static function load_done ( $dummy_id, $data ) { 
		/*
		* looks like inbox to js and css
		* 
		*/
		// get current folder
		$wic_plugin_options = get_option( 'wp_issues_crm_plugin_options_array' ); 
		$folder = isset ( $wic_plugin_options['imap_inbox'] ) ?  $wic_plugin_options['imap_inbox'] : ''  ;
		// bounce if no folder selected
		if ( '' == $folder ) {
			return array ( 'response_code' => false , 'output' => 'Check settings -- no inbox folder selected.' ) ;
		}
		// done messages remain in the inbox		
		global $wpdb;
		$inbox_image_table = $wpdb->prefix . 'wic_inbox_image';

		$filter = sanitize_text_field ( $data->filter );
		if ( $filter > '' ) {
			$filter_where = $wpdb->prepare ( " LOCATE( %s, subject ) > 0 AND ", array( $filter ) );
		} else {
			$filter_where = '';
		}

		// limit selection to inbox content
		$other_where_terms = 
			"full_folder_string = '$folder' AND
				to_be_moved_on_server = 1 
			";	

		// set max count (fixed )
		$max_count = 50;
 		// set page variable
 		$page_base = $data->page * $max_count;
		
		// assemble sql statements -- two version of SELECT, one for ungrouped, one for grouped
		$subjects_array_sql = 
			(
			'grouped' != $data->mode ?
			" 
			SELECT SQL_CALC_FOUND_ROWS
				ID,
				subject,
				from_personal as `from`,
				if ( email_date_time > '', email_date_time, activity_date ) as oldest,
				1 as `count`,
				0 as many
			"
			:
			"
			SELECT SQL_CALC_FOUND_ROWS
				ID,
				subject,
				from_personal as `from`,
				min( if ( email_date_time > '', email_date_time, activity_date ) ) as oldest,
				count(ID) as `count`, 
				if(COUNT(DISTINCT from_personal) > 1, 1, 0 ) as many
			"
			) . 
			"
			FROM $inbox_image_table
			WHERE 
				$filter_where
				$other_where_terms
			" . 
			(
				'grouped' == $data->mode ? 
					( " GROUP BY SUBJECT ORDER BY MIN( if ( email_date_time > '', email_date_time, activity_date ) ) " . $data->sort . ' ' ) : 
					( " ORDER BY if ( email_date_time > '', email_date_time, activity_date )  " . $data->sort . ' ' )
			) .
			"
			LIMIT $page_base, $max_count 					
			"
			;

		// get subjects array
		$subjects_array = $wpdb->get_results( $subjects_array_sql );
		// get count total messages ( as filtered and/or grouped )
		$found_result = $wpdb->get_results( "SELECT FOUND_ROWS() AS found_count" );	
		$found_count = $found_result[0]->found_count;
		
		// choose terms based on parms for use in both branches of the conditional
		$sort_order = ( 'ASC' == $data->sort ) ? ' first-arrived ' : ' last-arrived ';
		$loaded_object = 'grouped' == $data->mode ? 'subject lines' :   'messages';
		$filter_statement = $filter ? ' (filtered by "' . $filter . '")' : '';
		$view_statement = "Viewing $loaded_object, $sort_order first" . "$filter_statement. $max_count per page." ;

		$count_subjects = 0;
		if ( $subjects_array ) {
			$output = '<ul id="inbox-subject-list">';
			foreach ( $subjects_array as $subject  ) {
				$from_summary = $subject->from . ( $subject->many ? ' +' : '' );
				// format an li for the inbox
				$output .= '<li class="inbox-subject-line "><ul class="inbox-subject-line-inner-list">' . // *class determines sweepability*
					'<li class = "subject-line-item message-ID">' . $subject->ID . '</li>' .
					'<li class = "subject-line-item from-summary">' . $from_summary. '</li>' . // just display
					'<li class = "subject-line-item count" title = "Message Count"><span class="inner-count">' . $subject->count . '</span></li>' . // *supports multiple UI elements*
					'<li class = "subject-line-item subject"><span class="actual-email-subject">' . $subject->subject . '</span></li>' . // just display
					'<li class = "subject-line-item oldest" title="Date of oldest">' . $subject->oldest . '</li>' . // just display
				'</ul></li>';
				$count_subjects++;
			}
			$output .= '</ul>';
			/*
			* assemble page links and explanatory legend at end of inbox display
			*
			*/
			$output .= 
				'<div id = "wic-inbox-list-footer">' .
					'<div class = "wic-inbox-footer-legend">' . $view_statement . '</div>' . 
				'</div>';
		// no messages found
		} else {
			$output = $filter  ? 
				( '<div id = "filtered-all-warning">No processed message subject line containing "' . $filter . '".</div>' ) : 
				('<div id="inbox-congrats"><h1>No processed messages.</h1>');
		}


		// construct inbox title
		$inbox_header = 
			'Processed: ' . 
			( $page_base + 1 ) . '-' . ( $page_base + $count_subjects ) . ' of ' . $found_count . ' ' . $loaded_object; 
		
		$load_parms = (object) array ( 
			'folder'				=>   $folder,
			'filter'				=> 	 $filter,
			'page_ok'				=>	 ( $found_count > $page_base || 0 == $found_count ), // flag in case pages have shifted through record consolidation
		);		

		$return_array = (object) array (
			'inbox' => $output,
			'inbox_header' => $inbox_header,
			'nav_buttons' => array ( 'disable_prev' => $data->page == 0, 'disable_next' => ( $page_base + $max_count > $found_count ) ),
			'stuck' => false,
			'connection' => false,
			'last_load_parms' => $load_parms,
		);
		
		return array ( 'response_code' => true, 'output' => $return_array  ); 
	}
	
	public static function load_saved ( $dummy_id, $data ) { 
		/*
		* looks like inbox to js and css
		* 
		*/
		global $wpdb;
		$posts = $wpdb->posts;
		$meta  = $wpdb->postmeta;


		$filter = sanitize_text_field ( $data->filter );
		if ( $filter > '' ) {
			$filter_where = $wpdb->prepare ( " LOCATE( %s, p2.post_title ) > 0 AND ", array( $filter ) );
		} else {
			$filter_where = '';
		}

		// limit selection to inbox content
		$other_where_terms = 
			" m.meta_key LIKE '" . WIC_Entity_Email_Message::WIC_REPLY_METAKEY_STUB . "%'";	

		// set max count (fixed )
		$max_count = 50;
 		// set page variable
 		$page_base = $data->page * $max_count;
		
		// assemble sql statements -- two version of SELECT, one for ungrouped, one for grouped
		$subjects_array_sql = 
			(
			'grouped' != $data->mode ?
			" 
			SELECT SQL_CALC_FOUND_ROWS
				p.ID,
				p.post_title as subject,
				meta_key,
				meta_value,
				p2.post_modified as oldest,
				1 as count

			"
			:
			"
			SELECT SQL_CALC_FOUND_ROWS
				p.ID,
				p.post_title as subject,
				GROUP_CONCAT( meta_key ORDER BY meta_key SEPARATOR ','  ) as meta_key,
				meta_value,
				min( p2.post_modified )  as oldest,
				count( p.ID ) as count
			"
			) . 
			"
			FROM $posts p INNER JOIN $meta m ON p.ID = m.post_id inner join $posts p2 on p2.ID = meta_value
			WHERE 
				$filter_where
				$other_where_terms
			" . 
			(
				'grouped' == $data->mode ? 
					( " GROUP BY p.post_title ORDER BY MIN( p2.post_modified )  " . $data->sort . ' ' ) : 
					( " ORDER BY p2.post_modified  " . $data->sort . ' ' )
			) .
			"
			LIMIT $page_base, $max_count 					
			"
			;

		// get subjects array
		$subjects_array = $wpdb->get_results( $subjects_array_sql );
		// get count total messages ( as filtered and/or grouped )
		$found_result = $wpdb->get_results( "SELECT FOUND_ROWS() AS found_count" );	
		$found_count = $found_result[0]->found_count;
		
		// choose terms based on parms for use in both branches of the conditional
		$sort_order = ( 'ASC' == $data->sort ) ? ' first-modified ' : ' last-modified ';
		$loaded_object = 'grouped' == $data->mode ? 'issues' :   'issue/pro-con';
		$filter_statement = $filter ? ' (filtered by "' . $filter . '")' : '';
		$view_statement = "Viewing $loaded_object, $sort_order first" . "$filter_statement. $max_count per page." ;

		// get pro_con options
		global $wic_db_dictionary;
		$option_array = $wic_db_dictionary->lookup_option_values( 'pro_con_options' );
		

		// process subject array
		$count_subjects = 0;
		if ( $subjects_array ) {
			$output = '<ul id="inbox-subject-list">';
			foreach ( $subjects_array as $subject  ) {
				$meta_keys = explode ( ',' , $subject->meta_key );
				$pro_con_label = ''; 
				$later = false;
				foreach ( $meta_keys as $meta_key ) {
					if ( false === $later  ) {
						$later = true;
					} else {
						$pro_con_label .= '|';
					}
					$pro_con_label .=  WIC_Function_Utilities::value_label_lookup ( substr( $meta_key, 24), $option_array ) ;
				}
				// format an li for the inbox
				$output .= '<li class="inbox-subject-line "><ul class="inbox-subject-line-inner-list">' . // *class determines sweepability*
					'<li class = "subject-line-item message-ID">' . $subject->ID . '</li>' .
					'<li class = "subject-line-item reply-ID">' . $subject->meta_value . '</li>' .
					'<li class = "subject-line-item subject"><span class="actual-email-subject">' . $subject->subject . '</span></li>' . // just display
					'<li class = "subject-line-item from-summary">' . $pro_con_label . '</li>' . // just display
					'<li class = "subject-line-item pro-con-value">' . substr( $subject->meta_key, 24) . '</li>' . // just display
					'<li class = "subject-line-item count" title = "Message Count"><span class="inner-count">' . $subject->count . '</span></li>' . // *supports multiple UI elements*
					'<li class = "subject-line-item oldest" title="Date of oldest">' . $subject->oldest . '</li>' . // just display
				'</ul></li>';
				$count_subjects++;
			}
			$output .= '</ul>';
			/*
			* assemble page links and explanatory legend at end of inbox display
			*
			*/
			$output .= 
				'<div id = "wic-inbox-list-footer">' .
					'<div class = "wic-inbox-footer-legend">' . $view_statement . '</div>' . 
				'</div>';
		// no messages found
		} else {
			$output = $filter  ? 
				( '<div id = "filtered-all-warning">No saved reply standards with titles containing "' . $filter . '".</div>' ) : 
				('<div id="inbox-congrats"><h1>No saved reply standards.</h1>');
		}


		// construct inbox title
		$inbox_header = 
			'Saved Reply Standards: ' . 
			( $page_base + 1 ) . '-' . ( $page_base + $count_subjects ) . ' of ' . $found_count . ' ' . $loaded_object; 
		
		$load_parms = (object) array ( 
			'filter'				=> 	 $filter,
			'page_ok'				=>	 ( $found_count > $page_base || 0 == $found_count ), // flag in case pages have shifted through record consolidation
		);		

		$return_array = (object) array (
			'inbox' => $output,
			'inbox_header' => $inbox_header,
			'nav_buttons' => array ( 'disable_prev' => $data->page == 0, 'disable_next' => ( $page_base + $max_count > $found_count ) ),
			'stuck' => false,
			'connection' => false,
			'last_load_parms' => $load_parms,
		);
		
		return array ( 'response_code' => true, 'output' => $return_array  ); 
	}

}