<?php
/*
*
*	wic-entity-email-send.php
*
*/
class WIC_Entity_Email_Send {

	const body_break_string = "\n\n-------------------\n\n";


	// utility to clean up outgoing html for messages
	public static function filter_rough_html_for_output( $html ) {
		/*
		*  notes re filtering of final message content
		*
		*  note that esc_html not an option since escapes html characters -- can't use for html output
		*  sanitize_text_field strips tags entirely -- can't use for html output
		*
		*  first: wp_kses_post -- passes only whitelisted tags (THIS IS THE ONLY STEP WITH SECURITY SIGNIFICANCE)
		*	strips ascii control characters
		*   strips certain antiquated js entity strings
		*   screens &entities for allowed values
		*	screens tags for allowed values (a, div)
		*	screens tag attributes for allowed ( quite limited )
		*		within style attributes, accepts only defined css tags (adequate)
		*   DOES NOT PASS SCRIPT TAGS -- REMOVES THEM
		*
		*  second: wp_kses_post does not balance tags, so see what passes and then balance tags
		*
		*  third: the_content filter does the following by default -- it does not sanitize
		* 	'wptexturize' -- does smart quotes, elipses, dashes, etc.                   
		* 	'convert_smilies' -- converts ;) to images                  
		* 	'wpautop' -- convert linebreaks to p and br markers;                         
		* 	'shortcode_unautop' -- unwraps shortcodes from autop                
		* 	'prepend_attachment' -- wrap attachement in paragraph code before content               
		* 	'wp_make_content_images_responsive' -- images
		*    it DOES NOT sanitize output!
		*   don't want to actually apply the filter, because might have other things hooked in
		*
		*  NOTE: incoming message text goes through wp_kses_post in email-message-object (after charset cleaned)
		*	repeat in this routine which is used on messages before output, because template may introduce problems
		*
		*  3.3: No longer using wpautop, since using tinymce html editor 
		*  Not that balance_tags and kses are there to defend against bad content of the incoming message included in the outgoing body
		*/
		return  		
			balanceTags ( // balance tags
				wp_kses_post ( // lose scripts, stray quotes, bad tags that could be introduced in template or sig (these have not been sanitized for output outside a textarea)
					$html 
				),
				true
			);	
	}

	/*
	* Similar filtering for text as for html, but remove tags and entities, not the reverse, don't add html
	*
	* have to assume that incoming message body could have arbitrary html and, if contained in message, likely to not parse properly with regex,
	* so keep it simple: http://stackoverflow.com/questions/1732348/regex-match-open-tags-except-xhtml-self-contained-tags
	* -- only attempt to parse out links; would like to be able to include the same set of links; may affect deliverability if parts do not correspond
	* 
	* link parsing approach derived from http://www.the-art-of-web.com/php/parse-links/
	*/
	public static function filter_rough_html_to_text ( $html ) {
		$link_regex = "/<a\s[^>]*href\s*=\s*([\"\']??)([^\" >]*?)\\1[^>]*>(.*)<\/a>/siU";
		$break_patterns_array = WIC_DB_Email_Message_Object::break_patterns_array();
		return 
			preg_replace(									// discard any high plane UTF-8 characters coming of out of the decode
				'/[\x{10000}-\x{10FFFF}]/u', 
				"\xEF\xBF\xBD",
				html_entity_decode (						// decode html entities
					strip_tags( 							// remove other tags
						preg_replace( 						// put in line breaks in lieu of html breaks and paragaraphs
							$break_patterns_array['break_patterns'], 
							array ( "\n","\n" ),			// since tinymce (3.3) -- tinymce follows p with \n so get 3x \n if use $break_patterns_array['replacements'], 
							preg_replace ( 					// put links as plain text in parens after their text 
								$link_regex, 
								' \\0 ( \\2 )',
								wp_kses_post (				// redo in case of problems introduced in template
									$html				
								)
							)
						)
					),
					0,
					'UTF-8' 								// decode to utf-8
				)
			)
		;		
	}
	
	public static function reply_template_to_html ( $sample_object, $plain_text = false ) {

		/*
		*
		* First replace tokens if present
		*
		* This functionality is deprecated, and support will cease for it in a coming version
		*
		*/
		global $wic_db_dictionary;
		
		//defaults
		$first_name 		= '';
		$last_name  		= '';
		$issue				= '';
		$original_subject 	= '';
		$raw_text_body		= '';
		$raw_html_body		= '';
		$pro_con			= '';
		$template 			= '';

		// extract object vars
		extract ( (array) $sample_object, EXTR_OVERWRITE);
		
		$matches = array();
		$pattern = '#\|\*(' 		.
					'fn|' 		. // first name
					'dear|'		. // Dear first name or Greetings if blank
					'ln|' 		. // last name
					'title|'	. // issue title
					'content|' 	. // issue content
					'subject|'	. // email subject
					'pro_con|' 	. // pro or con label
					'\d*|'		. // link to post with specified id # 
					'\d*!|'		. // content of post with specified id # ;
					')\*\|#';
		
		$found_tokens =  preg_match_all ( $pattern, $template, $matches, 0, 0 ); // no flags, no offset
		
		// from array of matched tokens (without the delimiters)
		$replacement_array = array();
		$matches_with_delimiters = array();
		foreach ( $matches[1] as $match ) {
			// put the pattern delimiters and token delimiters back  
			$matches_with_delimiters[] = '#\|\*(' . $match . ')\*\|#';
		
			// construct parallel array of replacement values
			if ( 'fn' == $match ) {
				$replacement_array[] = ucfirst(strtolower( $first_name ));
			} elseif ( 'dear' == $match ) {
				$replacement_array[] = $first_name > '' ? 'Dear '. ucfirst(strtolower( $first_name )) . ',' : 'Greetings,';
			} elseif ( 'ln' == $match ) {
				$replacement_array[] = ucfirst(strtolower( $last_name ));
			} elseif ( 'title' == $match ) {
				$replacement_array[] = get_the_title(  $issue ); // blank preferable to an error here if $issue deleted instantaneously after queued messages
			} elseif ( 'content' == $match ) {
				$replacement_array[] = get_post( $issue, OBJECT, 'raw' )->post_content; // in error case, null is converted to empty string by php
			} elseif ( 'subject' == $match ) {
				$replacement_array[] = $original_subject;
			} elseif ( 'pro_con' == $match ) {
				$replacement_array[] = WIC_Function_Utilities::value_label_lookup ( $pro_con, $wic_db_dictionary->lookup_option_values( 'pro_con_options' ) );
			} elseif ( is_numeric ( $match ) ) {
				$link = get_the_permalink (  $match );
				$replacement_array[] = 
				( $plain_text ? 
					( get_the_title( $match ) . ' ( ' . $link . ' ) ' ) :
					( $link ? 
						( '<a target="_blank" href="' . $link . '">' . get_the_title( $match ) . '</a>' ) : 
						( '|*' . $match . ' is not a post id *|' ) 
					)
				);
			} elseif (  is_numeric ( trim ( $match , '!' ) ) ) {
				if ( get_post( trim ( $match , '!' ), OBJECT, 'raw' ) ) {
					$replacement_array[] = get_post( trim ( $match , '!' ), OBJECT, 'raw' )->post_content;				
				} else {
					$replacement_array[] = ( '|*' . $match . ' is not a post id *|' );
				}
			}
		}

		$html_raw = preg_replace ( $matches_with_delimiters, $replacement_array, $template );
		/*
		*
		* second add header/footer and body repetition if requested
		* |*body*| and |*user_sig*| obscelesced
		*
		*/
		$form_variables_object = unserialize ( get_option ( 'wp-issues-crm-email-processing-options' ) );
		$email_reply_body = $plain_text ? $raw_text_body : $raw_html_body;
		$separator = '';
		if ( $email_reply_body ) { // will be blank if straight send
			$separator = $plain_text ? self::body_break_string : '<hr/>';
		}
		
		$html_wrapped =
			( isset ( $form_variables_object->email_header ) ? $form_variables_object->email_header : '' ).
			$html_raw .
			WIC_Entity_User::get_current_user_sig () . 
			$separator .
			$email_reply_body  
			;
		/*
		*
		*
		* do final filter
		*
		*
		*/
		if ( $plain_text ) { 
			return self::filter_rough_html_to_text ( $html_wrapped );
		} else {
			return self::filter_rough_html_for_output ( $html_wrapped );
		}
	}

	public static function queue_reply ( $sample_object ) {

		global $wpdb;
		$outbox_table 	= $wpdb->prefix . "wic_outbox";
		$activity_table = $wpdb->prefix . "wic_activity";
		$current_user 	= get_current_user_id();

		$html_body = WIC_Entity_Email_Send::reply_template_to_html( $sample_object, false );
		$text_body = WIC_Entity_Email_Send::reply_template_to_html( $sample_object, true );
	
		$serialized_outgoing_object = serialize ( 
			new WIC_DB_Email_Outgoing_Object (
				$sample_object->addresses, 
				'RE: ' . $sample_object->original_subject,
				$html_body, 
				$text_body, 
				$sample_object->is_reply_to, 
				$sample_object->constituent_id, 
				$sample_object->issue	
			)
		);
		$sql = $wpdb->prepare (
			"INSERT INTO $outbox_table SET
				is_reply_to = %d,
				subject = %s,
				queued_date_time = NOW(),
				serialized_email_object = %s
			",
			array ( 
				$sample_object->is_reply_to,
				'RE: ' . $sample_object->original_subject,
				$serialized_outgoing_object
			)
		);
		$save_result = $wpdb->query( $sql );
		// do activity_record_insert
		if ( $save_result ) {
			$sql = $wpdb->prepare ( 
				"INSERT INTO $activity_table SET
					constituent_id = %d,
					activity_date = now(),
					activity_type = 'wic_reserved_99999999', 
					issue = %d, 
					pro_con = %s, 
					activity_note = %s , 
					last_updated_time = now(), 
					last_updated_by = $current_user, 
					related_outbox_record = %d
				",
				array (
					$sample_object->constituent_id,
					$sample_object->issue,
					$sample_object->pro_con,
					'<p>Subject: RE: ' . $sample_object->original_subject . '</p><hr/>' . $html_body, 
					$wpdb->insert_id 		
				)
			);
			$save_result = $wpdb->query( $sql );
		}	
		// link back to original ID	
		if ( 1 == $save_result ) {
			$response_code = true;
		} else {		
			$response_code = false;
		}

		return array ( 'response_code' => $response_code, 'output' => $response_code ? 'Queued reply OK.' : ( 'Database error queuing_reply -- message recorded but left in inbox: ' . print_r( $sample_object, true ) ) ); 
	} 


	/******/
	public static function prepare_list_send_confirm_window ( $dummy, $email_object ) {

		// update saved most recent template for user
		$last_saved_template_option_name = 	'wp-issues-crm-last-saved-template-' . get_current_user_id();
		update_option ( $last_saved_template_option_name, $email_object->replyTemplate ) ; 

		// act as if downloading the search ID, but just get the SQL to the temp file instead of doing the download
		$temp_table = WIC_List_Constituent_Export::do_constituent_download ( $email_object->searchType, $email_object->searchID );
		// read one from the temp file and get the count of the temp file
		$count_and_one_record = WIC_List_Constituent_Export::get_count_and_one_record ( $temp_table );	

		$sample_object = $count_and_one_record[1];
		$sample_object->issue = $email_object->issue;
		$sample_object->template = $email_object->replyTemplate;

		$outgoing_html = WIC_Entity_Email_Send::reply_template_to_html( $sample_object ) ; 
	
		$dialog_window_html = 
			'<div id="confirm_list_send_message" title="Email to '  . $count_and_one_record[0]  . ' constituent' . (  $count_and_one_record[0] == 1 ? '' : 's' ) . ' (showing sample email for one constituent)">' .
				$outgoing_html . 
				'<span id="wic_true_outgoing_email_count">'  . $count_and_one_record[0]  . '</span>' .
			'</div>';
		return array ( 'response_code' => true, 'output' => $dialog_window_html );
	}
	
	public static function email_to_list_of_constituents ( $dummy, $email_object ) {

		$temp_table = WIC_List_Constituent_Export::do_constituent_download ( $email_object->searchType, $email_object->searchID );
		
		global $wpdb;
		$sql = "SELECT * FROM $temp_table";
		$constituent_list = $wpdb->get_results ( $sql );

		$current_user = get_current_user_id();
		$activity_table = $wpdb->prefix . "wic_activity";
		$outbox_table = $wpdb->prefix . "wic_outbox";

		$successful_sends = 0;

		// $constituent_object is object containing c.ID, first_name, last_name, email_address (see assemble_constituent_export_sql for type email_send)
		foreach ( $constituent_list as $constituent_object ) {
			// add properties to constituent object
			$constituent_object->issue = $email_object->issue;
			$constituent_object->template = $email_object->replyTemplate;
			// construct message bodies from object
			$html_body = WIC_Entity_Email_Send::reply_template_to_html( $constituent_object, false );
			$text_body = WIC_Entity_Email_Send::reply_template_to_html( $constituent_object, true );
			// construct send object
			$to_array  = array( array( $constituent_object->first_name . ' ' . $constituent_object->last_name, $constituent_object->email_address ) );
			$addresses = array( 'to_array' => $to_array );
			$serialized_outgoing_object = serialize ( 
				new WIC_DB_Email_Outgoing_Object (
					$addresses, 
					$email_object->subject,
					$html_body, 
					$text_body, 
					0, 
					$constituent_object->ID, 
					$email_object->issue	
				)
			);
			// queue message
			$sql = $wpdb->prepare (
				"INSERT INTO $outbox_table SET
					subject = %s,
					queued_date_time = NOW(),
					serialized_email_object = %s
				",
				array ( 
					$email_object->subject,
					$serialized_outgoing_object
				)
			);
			$save_result = $wpdb->query( $sql );
			// do activity_record insert
			if ( $save_result ) {
				$sql = $wpdb->prepare ( 
					"INSERT INTO $activity_table SET
						constituent_id = %d,
						activity_date = now() ,
						activity_type = 'wic_reserved_99999999', 
						issue = %d, 
						activity_note = %s , 
						last_updated_time = now(), 
						last_updated_by = $current_user, 
						related_outbox_record = %d
					",
					array (
						$constituent_object->ID,
						$email_object->issue,
						'<p>Subject: ' . $email_object->subject . '</p><hr/>' . $html_body, 
						$wpdb->insert_id 		
					)
				);
				$save_result = $wpdb->query( $sql );
			}
			if ( $save_result ) {
				$successful_sends++;
			}
		}
		
		return array ( 'response_code' => true, 'output' => $successful_sends );

	}

	public static function delete_message_from_send_queue ( $dummy, $id ) {

		global $wpdb;
		$activity_table = $wpdb->prefix . "wic_activity";
		$outbox_table = $wpdb->prefix . "wic_outbox";

		// delete the queued id
		$sql = "DELETE a, o FROM $activity_table a INNER JOIN $outbox_table o on a.related_outbox_record = o.ID WHERE o.ID = $id"; 
		
		$response_code = $wpdb->query ( $sql );
		
		return array ( 	'response_code' => false !== $response_code, 
						'output' =>  
							false !== $response_code 
							? 
							"Message deleted OK"
							: 
							"Database error in deletion of message.",
		);	

	}
	
	public static function purge_mail_queue ( $dummy, $id ) {

		global $wpdb;
		$activity_table = $wpdb->prefix . "wic_activity";
		$outbox_table = $wpdb->prefix . "wic_outbox";

		// delete the queued id
		$sql = "DELETE a, o FROM $activity_table a INNER JOIN $outbox_table o on a.related_outbox_record = o.ID WHERE o.sent_ok = 0"; 
		
		$response_code = $wpdb->query ( $sql );
		
		return array ( 	'response_code' => false !== $response_code, 
						'output' =>  
							false !== $response_code 
							? 
							"Outbox purged OK"
							: 
							"Database error in outbox purge.",
		);	

	}
}