<?php
/*
* class-wic-list-activity.php
*
*
*/ 

class WIC_List_Activity extends WIC_List_Parent {
	/*
	* No message header
	*
	*/
	
	protected function list_entity_field_filter ( $fields, &$wic_query ) {
		$filtered_fields = array();
		if ( false === $wic_query->financial_activities_in_results ) {
			foreach ( $fields as $field ) {
				if ( 'activity_amount' != $field->field_slug ) {
					$filtered_fields[] = $field;				
				}			
			}
			return ( $filtered_fields );		
		} else {
			return ( $fields );		
		}
			
	}	
	
	public function format_entity_list( &$wic_query, $header ) { 

  		// set up form
		$output = '<div id="wic-post-list"><form id="wic_activity_list_form" method="POST">';
		$output .= $this->get_the_buttons( $wic_query );	
		$output .= $this->set_up_rows ( $wic_query );
		$output .= 	wp_nonce_field( 'wp_issues_crm_post', 'wp_issues_crm_post_form_nonce_field', true, true ) .
		'</form></div>'; 
		$output .= WIC_List_Parent::list_terminator ( $wic_query );
	
		return $output;
   } // close function

	
	
	protected function format_rows( &$wic_query, &$fields ) {

		$output = '';
		$line_count = 1;

		// get financial activity types to avoid showing nonsensical zero amounts
		$wic_option_array = get_option('wp_issues_crm_plugin_options_array'); 
		$financial_types_array = explode (',' , isset ( $wic_option_array['financial_activity_types'] ) ? $wic_option_array['financial_activity_types'] : '') ;
		
		// get custom field set 
		global $wic_db_dictionary;
		$custom_fields = $wic_db_dictionary->custom_fields_match_array ();
		$count_custom = count ( $custom_fields );
		
		foreach ( $wic_query->result as $row_array ) {

			$row= '';
			$line_count++;
			$row_class = ( 0 == $line_count % 2 ) ? "pl-even" : "pl-odd";
			
			$row .= '<ul class = "wic-post-list-line">';			
				foreach ( $fields as $field ) { 
					// showing fields other than ID with positive listing order ( in left to right listing order )
					if ( 'ID' != $field->field_slug && $field->listing_order > 0 ) {
						$row .= '<li class = "wic-post-list-field pl-' . $wic_query->entity . '-' . $field->field_slug . ' "> ';
							if ( 'constituent_id' == $field->field_slug ) {
								$row .= esc_html ( $row_array->last_name . ', ' . $row_array->first_name );	
								if ( $count_custom > 0 ) {
									$row .= '<div class = activity-list-custom-add-ons>';
										foreach ( $custom_fields as $custom => $match_array ) {
											if ( $row_array->$custom > '' ) {
												$row .= '<span class = "activity-custom-field pl-activity-' . $custom .'">&nbsp;-- ' . esc_html ( $row_array->$custom ) . '</span>';
											}
										}
									$row .= '</div>';
								}						
							} elseif ( 'issue' == $field->field_slug ) {
								$row .= esc_html ( $row_array->post_title );
							} elseif ( 'activity_amount' == $field->field_slug ) {								
								$row .= in_array ( $row_array->activity_type, $financial_types_array ) ? esc_html ( $row_array->activity_amount ) : '--'; 							
							} else  {
								$row .=  $this->format_item ( $wic_query->entity, $field->list_formatter, $row_array->{$field->field_slug} ) ;
							}
						$row .= '</li>';			
					}	
				}
			$row .='</ul>';				
			
			$list_button_args = array(
					'entity_requested'	=> 'constituent',
					'action_requested'	=> 'id_search',
					'button_class' 		=> 'wic-post-list-button ' . $row_class,
					'id_requested'			=> $row_array->constituent_id,
					'button_label' 		=> $row,				
			);			
			$output .= '<li>' . WIC_Form_Parent::create_wic_form_button( $list_button_args ) . '</li>';	
		} // close for each row
		return ( $output );		
	} // close function 

   
   // the top row of buttons over the list -- down load button alone in this case, since search-again is main button
  	protected function get_the_buttons( &$wic_query ) { 

		$buttons = '';
		
		// wic-activity-export-button
		$button_args = array (
				'name' 			=> 'wic-activity-export-button',
				'value' 		=> 'activity,activity,activities,' . $wic_query->search_id,  // see wic-admin-navigation::do_download
				'button_label'	=>	__( 'Download Activities', 'wp-issues-crm' ),
				'id'			=> 'wic-activity-export-button',
				'button_class'	=> 'wic-form-button wic-download-button', // selector for download listener
				'title'			=>	__( 'Download all activities meeting search criteria together with related constituent information', 'wp-issues-crm' ),
			);
		$buttons .= WIC_Form_Parent::create_wic_form_button( $button_args );

		// show search form with parameters 
		if ( isset ( $wic_query->advanced_search ) ) { 
			$buttons .= WIC_List_Parent::back_to_search_form_button ( $wic_query, false );
			$buttons .= WIC_List_Parent::hidden_search_id_control( $wic_query );
			$buttons .= WIC_List_Parent::search_inspection_button( $wic_query );
			$buttons .= WIC_List_Parent::search_name_control ( $wic_query );
		}
		
		return ( $buttons );
	}

	public function format_message( &$wic_query, $header='' ) {
	
		$financial_total = $wic_query->financial_activities_in_results ? sprintf ( __( ' Total amount for found activities is %1$s.', 'wp-issues-crm'), $wic_query->amount_total ) : '';	
		$found_string = intval( $wic_query->found_count ) > 1 ? sprintf ( __( 'Found %1$s activities.', 'wp-issues-crm'), $wic_query->found_count ) : 
				__( 'Found one activity.', 'wp-issues-crm' );
		$header_message = $header . $found_string. $financial_total;		
		$header_message = WIC_Entity_Advanced_Search::add_blank_rows_message ( $wic_query, $header_message );
		return $header_message;
	}

}	

