<?php
/*
* class-wic-list-constituent-export.php
*
* 
*/ 

class WIC_List_Constituent_Export {

	public static function do_constituent_download ( $download_type, $search_id ) { 
		// if user changed download type back to empty (initial button value), do nothing.
		// barred by js, but bullet proofing
		if ( '' == $download_type ) {
			return;		
		}		
	
		// naming the outfile
		$current_user = wp_get_current_user();	
		$file_name = 'wic-constituent-export-' . $current_user->user_firstname . '-' .  current_time( 'Y-m-d-H-i-s' )  .  '.csv' ;
		
		// create query object directly if downloading for a single issue (no search log entry, $search_id is just the issue (i.e., post ID ))
		if ( 'issue_activity_list_email_send' == $download_type ) {		

			$wic_query = WIC_DB_Access_Factory::make_a_db_access_object( 'constituent' );

			$search_parameters= array(
				'retrieve_limit' 	=> 99999999,
				'select_mode'		=> 'download',
			);

			$search_array = array (
				array (
					 'table'	=> 'activity',
					 'key'	=> 'issue',
					 'value'	=>  $search_id, 
					 'compare'	=> '=', 
					 'wp_query_parameter' => '',
				),
				array (
					 'table'	=> 'email',
					 'key'	=> 'email_address',
					 'value'	=> '', 
					 'compare'	=> '>', 
					 'wp_query_parameter' => '',
				), 
			);

			$wic_query->search ( $search_array, $search_parameters ); // create a temp table of id's 
			// do the rest as for standard email_send download
			$download_type = 'email_send';			

		} else {
			// retrieves only the meta array, not the search parameters, since will supply own
			$search = WIC_Entity_Search_Log::get_search_from_search_log ( array( 'id_requested' => $search_id ) );	
			// can be issue or constituent	
			$wic_query = WIC_DB_Access_Factory::make_a_db_access_object( $search['entity'] );
		
			$search_parameters = array (		
				'select_mode' 		=> 'download', // with this setting, object will create the temp table that export sql assembler is looking for
				'sort_order' 		=> true,
				'compute_total' 	=> false,
				'retrieve_limit' 	=> 999999999,
				'redo_search'		=> true,
				'old_search_id'		=> $search_id,	
			);
		
			// do the search, saving retrieved id list in temp table if constituent, setting up next step if issue	
			$wic_query->search ( $search['unserialized_search_array'], $search_parameters );

		} 

		
		$sql = self::assemble_constituent_export_sql( $download_type ); // runs off temp table

		if ( 'email_send' == $download_type ) {
			return ( $sql );
		} else {
			self::do_the_export( $file_name, $sql );
		}

		if ( $search_id ) {
			WIC_Entity_Search_Log::mark_search_as_downloaded ( $search_id );
		}
		
		exit;
		
	}

	public static function do_staging_table_download ( $file_requested,  $staging_table_name  ) {

		global $wpdb;		
		
		// naming the download file
		$file_name = $staging_table_name  . '_' . $file_requested . '.csv' ;
		$sql = "SELECT * FROM $staging_table_name ";
		// set up the sql			
		if ( 'validate' == $file_requested ) {
			// validation errors; can export after validation
			$sql .= " WHERE VALIDATION_STATUS = 'n' "; 										
		} else if ( 'new_constituents' == $file_requested ) {
			// not matched, but have matching fields; can export after define matching 
			$sql .= " WHERE MATCH_PASS = '' AND FIRST_NOT_FOUND_MATCH_PASS > '' "; 	
		} else if ( 'match' == $file_requested ) {
			// actually matched; can export after define matching
			$sql .= " WHERE MATCH_PASS > '' AND MATCHED_CONSTITUENT_ID > 0 "; 		
		} else if ( 'bad_match' == $file_requested ) {				
			// but no match attempted -- most likely missing data although passing validation
			$sql .= " WHERE ( VALIDATION_STATUS = 'y' AND MATCH_PASS = '' AND FIRST_NOT_FOUND_MATCH_PASS = '' ) "; 
		} else if ( 'not_loaded' == $file_requested ) {
			// not linked -- meaningful only after completion										
			$sql .= " WHERE MATCHED_CONSTITUENT_ID = 0 ";
		}	
		// 'dump' has no where clause
		self::do_the_export( $file_name, $sql );	
		
		exit;

	}


	/*
	*	sql points to temporary table created early in transaction
	*/
	public static function assemble_constituent_export_sql ( $download_type ) { 
		
		// reference global wp query object	
		global $wpdb;	
		$prefix = $wpdb->prefix . 'wic_';
		
		$constituent 	= $prefix . 'constituent';
		$email 			= $prefix . 'email';
		$phone			= $prefix . 'phone';
		$address			= $prefix . 'address';
		
		// id list passed through user's temporary file wp_wic_temporary_id_list, lasts only through the server transaction (multiple db transactions)
		$temp_table = $wpdb->prefix . 'wic_temporary_id_list' . get_current_user_id();
		
		// pass constituent list through repeated chunks to db as temp table
		$temp_constituent_table = $wpdb->prefix . 'wic_temporary_constituent_list' . time();		

		global $wic_db_dictionary;
		$custom_fields = $wic_db_dictionary->custom_fields_match_array ();
		$custom_fields_string = '';
		if ( count ( $custom_fields ) > 0 ) {
			foreach ( $custom_fields as $field => $match_array ) {
				$custom_fields_string .= ', ' . $field . ' as `' . $match_array['label'] . ' -- ' . $field . '` ';			
			}
	 		
		}


		// initialize download sql -- if remains blank, will bypass download
		$download_sql = '';
		switch ( $download_type ) {
			case 'emails':		
				$download_sql =" 
					SELECT  first_name, last_name,  
						email_type, email_address, 
						max( city ) as city,
						max( zip ) as zip $custom_fields_string , i.*
					FROM $temp_table i INNER JOIN $constituent c on c.ID = i.ID
					inner join $email e on e.constituent_id = c.ID
					left join $address a on a.constituent_id = c.ID	
					GROUP BY c.ID, e.ID
					"; 
				break;
			case 'phones':		
				$download_sql =" 
					SELECT  first_name, last_name,  
						phone_type, phone_number, extension, 
						max( city ) as city $custom_fields_string , i.*
					FROM $temp_table i INNER JOIN $constituent c on c.ID = i.ID
					inner join $phone p on p.constituent_id = c.ID
					left join $address a on a.constituent_id = c.ID	
					GROUP BY c.ID, p.ID
					"; 		
				break;
			case 'addresses':		
				$download_sql =" 
					SELECT  first_name, last_name,  
						address_type, 
						address_line as wic_address_line_1,
						concat ( city, ', ', state, ' ',  zip ) as wic_address_line_2,
						city, state, zip $custom_fields_string , i.*
					FROM $temp_table i INNER JOIN $constituent c on c.ID = i.ID
					inner join $address a on a.constituent_id = c.ID	
					GROUP BY c.ID, a.ID
					"; 		
				break;		
			case 'type0':
				// note: assumes user has only one home address -- could mix elements from two if present
				// will show blank address/email/phone rather than dropping constituent if no home information
				$download_sql = " 		
					SELECT  first_name, last_name,  
						max( if ( address_type = '0', city, null )) as city, 
						max( if ( email_type = '0', email_address, null ) ) as email, 
						max( if ( phone_type = '0', phone_number, null ) ) as phone,
						max( if ( address_type = '0', address_line, null ) ) as wic_address_line_1,
						max( if ( address_type = '0', concat ( city, ', ', state, ' ',  zip ), null ) )as wic_address_line_2,
						max( if ( address_type = '0', state, null ) ) as state,
						max( if ( address_type = '0', zip, null ) ) as zip
						$custom_fields_string , i.*
					FROM $temp_table i INNER JOIN $constituent c on c.ID = i.ID
					left join $email e on e.constituent_id = c.ID
					left join $phone p on p.constituent_id = c.ID
					left join $address a on a.constituent_id = c.ID	
					group by c.ID
					";	
				break;
			case 'dump':
				$download_sql = " 		
					SELECT  first_name, last_name, middle_name,  
						email_address, 
						phone_number,
						address_line,
						city, state, zip,
						date_of_birth, is_deceased, mark_deleted, 
						case_assigned, case_review_date, case_status, 
						gender, c.last_updated_time, c.last_updated_by $custom_fields_string ,
						i.*
					FROM $temp_table i INNER JOIN $constituent c on c.ID = i.ID
					left join $email e on e.constituent_id = c.ID
					left join $phone p on p.constituent_id = c.ID
					left join $address a on a.constituent_id = c.ID	
					";	
				break;				
			case 'email_send':
				$download_sql = " 		
					SELECT  c.ID, first_name, last_name, email_address 
					FROM $temp_table i INNER JOIN $constituent c on c.ID = i.ID
					inner join $email e on e.constituent_id = c.ID
					group by c.ID
					";	
				break;	
		} 	
	
   	// go direct to database and do customized search and write temp table
		$sql = 	"CREATE TEMPORARY TABLE $temp_constituent_table
					$download_sql
					"; 
		
		$result = $wpdb->query ( $sql );

		if ( 'email_send' == $download_type ) {
			$sql = $temp_constituent_table;
		} else {
			$sql = "SELECT * FROM $temp_constituent_table ";
		}
		return ( $sql);
	}




	// do_the_export runs the $sql in chunks and exports to filename
	protected static function do_the_export ( $file_name, $sql ) {

		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header('Content-Description: File Transfer');
		header("Content-type: text/csv");
		header("Content-Disposition: attachment; filename={$file_name}");
		header("Expires: 0");
		header("Pragma: public");

		$fh = fopen( 'php://output', 'wt' ); // writing plain text files; good to support txt 
		
		global $wpdb;
		
		// chunking the retrievals to keep $results array size down on large downloads
		// 10000 peaks memory usage at just under 100M for constituent download without any especially long custom fields 
		// believe packet size not an issue -- outgoing sql is short and the packet is just a row coming back (?).
		// 1000 chunk size cuts memory usage to 13M ( WP is roughly 4M) and but is only about 30% slower on large files
		$i = 0;
		$header_displayed = false;
		while ( $results = $wpdb->get_results ( $sql . " LIMIT " . $i * 1000 . ", 1000 ", ARRAY_A ) ) {
			$i++;	
			foreach ( $results as $result ) {
				if ( !$header_displayed ) {
		      	fputcsv($fh, array_keys($result));  
		        	$header_displayed = true;
		   	}
		    fputcsv($fh, $result); // defaults are delimiter ',', enclosure '"', escape '/'
			}
		} 

		if ( 0 == $i ) { // write a not found header if no records were found
			fputcsv( $fh, array ( "No records found for $file_name" ) ); 		
		}

		fclose ( $fh );		

	}

	public static function get_count_and_one_record ( $temp_table ) {
		global $wpdb;
		$row_one = $wpdb->get_results ( "SELECT SQL_CALC_FOUND_ROWS $temp_table.* from $temp_table LIMIT 0, 1" );
		$count 	 = $wpdb->get_results ( "SELECT FOUND_ROWS() as found_count" );
		return ( array( $count[0]->found_count, $row_one[0] ) );
	}
}	

